@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2013-2014 - Mauro Carvalho Chehab <m.chehab@samsung.com>
+ * Copyright (c) 2013-2014 - Mauro Carvalho Chehab <mchehab@kernel.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -215,7 +215,7 @@ extern const unsigned dvbt2_transmission_mode[];
  * @brief converts from internal representation to string the SISO_MISO
  *	  field of dvb_desc_t2_delivery:SISO_MISO field.
  */
-const char *siso_miso[4];
+extern const char *siso_miso[4];
 
 #ifdef __cplusplus
 }
