@@ -30,6 +30,7 @@
 #include <ctype.h>
 #include <inttypes.h>
 #include <libudev.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <locale.h>
@@ -78,9 +79,10 @@ struct ringbuffer {
 	pthread_mutex_t lock;
 };
 
+#define CMD_SIZE	80
 struct queued_msg {
 	int seq;
-	char cmd[80];
+	char cmd[CMD_SIZE];
 	int retval;
 
 	pthread_mutex_t lock;
@@ -112,7 +114,17 @@ struct dvb_dev_remote_priv {
 	void *user_priv;
 };
 
-void stack_dump(struct dvb_v5_fe_parms_priv *parms)
+static char *my_strlcpy(char *dst, const char *src, size_t siz)
+{
+	char *rc;
+
+	rc = strncpy(dst, src, siz - 1);
+	dst[siz - 1] = '\0';
+
+	return rc;
+}
+
+static void stack_dump(struct dvb_v5_fe_parms_priv *parms)
 {
 #ifdef HAVE_BACKTRACE
 	int i, nptrs = 0;
@@ -216,11 +228,11 @@ static ssize_t prepare_data(struct dvb_v5_fe_parms_priv *parms,
 }
 
 static struct queued_msg *send_fmt(struct dvb_device_priv *dvb, int fd,
-				   const char *cmd, const char *fmt, ...)
+				   const char cmd[CMD_SIZE], const char *fmt, ...)
 	__attribute__ (( format( printf, 4, 5 )));
 
 static struct queued_msg *send_fmt(struct dvb_device_priv *dvb, int fd,
-				   const char *cmd, const char *fmt, ...)
+				   const char cmd[CMD_SIZE], const char *fmt, ...)
 {
 	struct dvb_v5_fe_parms_priv *parms = (void *)dvb->d.fe_parms;
 	struct dvb_dev_remote_priv *priv = dvb->priv;
@@ -239,7 +251,7 @@ static struct queued_msg *send_fmt(struct dvb_device_priv *dvb, int fd,
 
 	pthread_mutex_init(&msg->lock, NULL);
 	pthread_cond_init(&msg->cond, NULL);
-	strcpy(msg->cmd, cmd);
+	my_strlcpy(msg->cmd, cmd, sizeof(msg->cmd));
 
 	pthread_mutex_lock(&priv->lock_io);
 	msg->seq = ++priv->seq;
@@ -250,6 +262,7 @@ static struct queued_msg *send_fmt(struct dvb_device_priv *dvb, int fd,
 		dvb_logdbg("buffer to short for int32_t");
 		stack_dump(parms);
 		pthread_mutex_unlock(&priv->lock_io);
+		free(msg);
 		return NULL;
 	}
 	memcpy(p, &i32, 4);
@@ -262,6 +275,7 @@ static struct queued_msg *send_fmt(struct dvb_device_priv *dvb, int fd,
 				p - buf, len, sizeof(buf));
 		stack_dump(parms);
 		pthread_mutex_unlock(&priv->lock_io);
+		free(msg);
 		return NULL;
 	}
 	i32 = htobe32(len);
@@ -278,6 +292,7 @@ static struct queued_msg *send_fmt(struct dvb_device_priv *dvb, int fd,
 
 	if (ret < 0) {
 		pthread_mutex_unlock(&priv->lock_io);
+		free(msg);
 		return NULL;
 	}
 
@@ -313,7 +328,7 @@ static struct queued_msg *send_fmt(struct dvb_device_priv *dvb, int fd,
 }
 
 static struct queued_msg *send_buf(struct dvb_device_priv *dvb, int fd,
-				   const char *cmd,
+				   const char cmd[CMD_SIZE],
 				   const char *in_buf, const size_t in_size)
 {
 	struct dvb_v5_fe_parms_priv *parms = (void *)dvb->d.fe_parms;
@@ -332,7 +347,7 @@ static struct queued_msg *send_buf(struct dvb_device_priv *dvb, int fd,
 
 	pthread_mutex_init(&msg->lock, NULL);
 	pthread_cond_init(&msg->cond, NULL);
-	strcpy(msg->cmd, cmd);
+	my_strlcpy(msg->cmd, cmd, sizeof(msg->cmd));
 
 	pthread_mutex_lock(&priv->lock_io);
 	msg->seq = ++priv->seq;
@@ -343,6 +358,7 @@ static struct queued_msg *send_buf(struct dvb_device_priv *dvb, int fd,
 		dvb_logdbg("buffer to short for int32_t");
 		stack_dump(parms);
 		pthread_mutex_unlock(&priv->lock_io);
+		free(msg);
 		return NULL;
 	}
 	memcpy(p, &i32, 4);
@@ -355,6 +371,7 @@ static struct queued_msg *send_buf(struct dvb_device_priv *dvb, int fd,
 				p - buf, len, sizeof(buf));
 		stack_dump(parms);
 		pthread_mutex_unlock(&priv->lock_io);
+		free(msg);
 		return NULL;
 	}
 	i32 = htobe32(len);
@@ -368,6 +385,7 @@ static struct queued_msg *send_buf(struct dvb_device_priv *dvb, int fd,
 		dvb_logdbg("buffer to big!");
 		stack_dump(parms);
 		pthread_mutex_unlock(&priv->lock_io);
+		free(msg);
 		return NULL;
 	}
 
@@ -1040,8 +1058,10 @@ static struct dvb_open_descriptor *dvb_remote_open(struct dvb_device_priv *dvb,
 	open_dev = &ringbuf->open_dev;
 
 	msg = send_fmt(dvb, priv->fd, "dev_open", "%s%i", sysname, flags);
-	if (!msg)
+	if (!msg) {
+		free(ringbuf);
 		return NULL;
+	}
 
 	ret = pthread_cond_wait(&msg->cond, &msg->lock);
 	if (ret < 0) {
@@ -1076,6 +1096,7 @@ error:
 	pthread_mutex_unlock(&msg->lock);
 
 	free_msg(dvb, msg);
+	free(ringbuf);
 	return NULL;
 }
 
@@ -1732,8 +1753,10 @@ int dvb_dev_remote_init(struct dvb_device *d, char *server, int port)
 
 	/* Set large buffer for read() to work better */
 	bufsize = REMOTE_BUF_SIZE;
-	setsockopt(fd, SOL_SOCKET, SO_RCVBUF,
-		   (void *)&bufsize, (int)sizeof(bufsize));
+	if (setsockopt(fd, SOL_SOCKET, SO_RCVBUF,
+		       (void *)&bufsize, (int)sizeof(bufsize))) {
+		dvb_perror("can't set buffer size");
+	}
 
 	/* Start receiving messsages from the server */
 	pthread_mutex_init(&priv->lock_io, NULL);
