@@ -8,10 +8,6 @@
 #ifndef _CEC_COMPLIANCE_H_
 #define _CEC_COMPLIANCE_H_
 
-#include <cstdlib>
-#include <ctime>
-#include <string>
-
 #include <linux/cec-funcs.h>
 #include "cec-htng-funcs.h"
 
@@ -23,6 +19,8 @@
 
 #include <cec-info.h>
 
+#include <vector>
+
 #define TAG_AUDIO_RATE_CONTROL		1
 #define TAG_ARC_CONTROL 		(1 << 1)
 #define TAG_CAP_DISCOVERY_CONTROL 	(1 << 2)
@@ -149,6 +147,7 @@ struct remote {
 	__u8 mute;
 	bool has_aud_rate;
 	bool has_deck_ctl;
+	bool has_analogue_timer;
 	__u8 bcast_sys;
 	__u8 dig_bcast_sys;
 	bool has_rec_tv;
@@ -168,6 +167,8 @@ struct node {
 	struct remote remote[16];
 	__u16 phys_addr;
 	bool in_standby;
+	__u8 prim_devtype;
+	time_t current_time;
 };
 
 struct remote_subtest {
@@ -178,6 +179,8 @@ struct remote_subtest {
 	bool for_cec20;
 };
 
+using vec_remote_subtests = std::vector<remote_subtest>;
+
 #define OK			0
 #define FAIL			1
 #define OK_PRESUMED		2
@@ -191,9 +194,6 @@ struct remote_subtest {
 
 #define CEC_LOG_ADDR_MASK_ALL 0xffff
 
-#define ARRAY_SIZE(a) \
-	(sizeof(a) / sizeof(*a))
-
 #define COLOR_GREEN(s) "\033[32m" s "\033[0m"
 #define COLOR_RED(s) "\033[1;31m" s "\033[0m"
 #define COLOR_BOLD(s) "\033[1m" s "\033[0m"
@@ -263,6 +263,16 @@ struct remote_subtest {
 	FAIL;								\
 })
 
+#define fail_once(fmt, args...)						\
+	do {								\
+		static bool show;					\
+									\
+		if (!show) {						\
+			show = true;					\
+			fail(fmt, ##args);				\
+		}							\
+	} while (0)
+
 #define fail_or_warn(node, fmt, args...)				\
 ({									\
  	if ((node)->in_standby)						\
@@ -352,6 +362,11 @@ static inline bool refused(const struct cec_msg *msg)
 	return cec_msg_status_is_abort(msg) && abort_reason(msg) == CEC_OP_ABORT_REFUSED;
 }
 
+static inline bool incorrect_mode(const struct cec_msg *msg)
+{
+	return cec_msg_status_is_abort(msg) && abort_reason(msg) == CEC_OP_ABORT_INCORRECT_MODE;
+}
+
 static inline bool timed_out(const struct cec_msg *msg)
 {
 	return msg->rx_status & CEC_RX_STATUS_TIMEOUT;
@@ -406,14 +421,8 @@ static inline unsigned get_ts_ms()
 const char *result_name(int res, bool show_colors);
 const char *ok(int res);
 const char *power_status2s(__u8 power_status);
-std::string short_audio_desc2s(const struct short_audio_desc &sad);
-void sad_decode(struct short_audio_desc *sad, __u32 descriptor);
 const char *bcast_system2s(__u8 bcast_system);
 const char *dig_bcast_system2s(__u8 bcast_system);
-const char *hec_func_state2s(__u8 hfs);
-const char *host_func_state2s(__u8 hfs);
-const char *enc_func_state2s(__u8 efs);
-const char *cdc_errcode2s(__u8 cdc_errcode);
 int check_0(const void *p, int len);
 int util_receive(struct node *node, unsigned la, unsigned timeout,
 		 struct cec_msg *msg, __u8 sent_msg,
@@ -449,26 +458,23 @@ int setExpectedResult(char *optarg, bool no_warnings);
 void testRemote(struct node *node, unsigned me, unsigned la, unsigned test_tags,
 			     bool interactive);
 
+// cec-test-tuner-record-timer.cpp
+extern const vec_remote_subtests tuner_ctl_subtests;
+extern const vec_remote_subtests one_touch_rec_subtests;
+extern const vec_remote_subtests timer_prog_subtests;
+
 // cec-test-audio.cpp
-extern struct remote_subtest sac_subtests[];
-extern const unsigned sac_subtests_size;
-extern struct remote_subtest dal_subtests[];
-extern const unsigned dal_subtests_size;
-extern struct remote_subtest arc_subtests[];
-extern const unsigned arc_subtests_size;
-extern struct remote_subtest audio_rate_ctl_subtests[];
-extern const unsigned audio_rate_ctl_subtests_size;
+extern const vec_remote_subtests sac_subtests;
+extern const vec_remote_subtests dal_subtests;
+extern const vec_remote_subtests arc_subtests;
+extern const vec_remote_subtests audio_rate_ctl_subtests;
 
 // cec-test-power.cpp
 bool util_interactive_ensure_power_state(struct node *node, unsigned me, unsigned la, bool interactive,
 					 __u8 target_pwr);
-extern struct remote_subtest standby_subtests[];
-extern const unsigned standby_subtests_size;
-extern struct remote_subtest one_touch_play_subtests[];
-extern const unsigned one_touch_play_subtests_size;
-extern struct remote_subtest power_status_subtests[];
-extern const unsigned power_status_subtests_size;
-extern struct remote_subtest standby_resume_subtests[];
-extern const unsigned standby_resume_subtests_size;
+extern const vec_remote_subtests standby_subtests;
+extern const vec_remote_subtests one_touch_play_subtests;
+extern const vec_remote_subtests power_status_subtests;
+extern const vec_remote_subtests standby_resume_subtests;
 
 #endif
