@@ -3,28 +3,217 @@
  * Copyright 2016 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  */
 
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <inttypes.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
+#include <ctime>
+#include <sstream>
+#include <string>
+
 #include <sys/ioctl.h>
-#include <config.h>
-#include <cstdlib>
+#include <unistd.h>
 
 #include "cec-compliance.h"
+#include "compiler.h"
 
+static std::string audio_format_code2s(__u8 format_code)
+{
+	switch (format_code) {
+	case 0:
+		return "Reserved";
+	case SAD_FMT_CODE_LPCM:
+		return "L-PCM";
+	case SAD_FMT_CODE_AC3:
+		return "AC-3";
+	case SAD_FMT_CODE_MPEG1:
+		return "MPEG-1";
+	case SAD_FMT_CODE_MP3:
+		return "MP3";
+	case SAD_FMT_CODE_MPEG2:
+		return "MPEG2";
+	case SAD_FMT_CODE_AAC_LC:
+		return "AAC LC";
+	case SAD_FMT_CODE_DTS:
+		return "DTS";
+	case SAD_FMT_CODE_ATRAC:
+		return "ATRAC";
+	case SAD_FMT_CODE_ONE_BIT_AUDIO:
+		return "One Bit Audio";
+	case SAD_FMT_CODE_ENHANCED_AC3:
+		return "Enhanced AC-3";
+	case SAD_FMT_CODE_DTS_HD:
+		return "DTS-HD";
+	case SAD_FMT_CODE_MAT:
+		return "MAT";
+	case SAD_FMT_CODE_DST:
+		return "DST";
+	case SAD_FMT_CODE_WMA_PRO:
+		return "WMA Pro";
+	case SAD_FMT_CODE_EXTENDED:
+		return "Extended";
+	default:
+		return "Illegal";
+	}
+}
+
+static std::string extension_type_code2s(__u8 type_code)
+{
+	switch (type_code) {
+	case 0:
+	case 1:
+	case 2:
+	case 3:
+		return "Not in use";
+	case SAD_EXT_TYPE_MPEG4_HE_AAC:
+		return "MPEG-4 HE AAC";
+	case SAD_EXT_TYPE_MPEG4_HE_AACv2:
+		return "MPEG-4 HE AAC v2";
+	case SAD_EXT_TYPE_MPEG4_AAC_LC:
+		return "MPEG-4 AAC LC";
+	case SAD_EXT_TYPE_DRA:
+		return "DRA";
+	case SAD_EXT_TYPE_MPEG4_HE_AAC_SURROUND:
+		return "MPEG-4 HE AAC + MPEG Surround";
+	case SAD_EXT_TYPE_MPEG4_AAC_LC_SURROUND:
+		return "MPEG-4 AAC LC + MPEG Surround";
+	case SAD_EXT_TYPE_MPEG_H_3D_AUDIO:
+		return "MPEG-H 3D Audio";
+	case SAD_EXT_TYPE_AC_4:
+		return "AC-4";
+	case SAD_EXT_TYPE_LPCM_3D_AUDIO:
+		return "L-PCM 3D Audio";
+	default:
+		return "Reserved";
+	}
+}
+
+static std::string short_audio_desc2s(const struct short_audio_desc &sad)
+{
+	std::stringstream oss;
+
+	if (sad.format_code != SAD_FMT_CODE_EXTENDED)
+		oss << audio_format_code2s(sad.format_code);
+	else
+		oss << extension_type_code2s(sad.extension_type_code);
+	oss << ", " << static_cast<int>(sad.num_channels) << " channels";
+
+	oss << ", sampling rates (kHz): ";
+	if (sad.sample_freq_mask & SAD_SAMPLE_FREQ_MASK_32)
+		oss << "32,";
+	if (sad.sample_freq_mask & SAD_SAMPLE_FREQ_MASK_44_1)
+		oss << "44.1,";
+	if (sad.sample_freq_mask & SAD_SAMPLE_FREQ_MASK_48)
+		oss << "48,";
+	if (sad.sample_freq_mask & SAD_SAMPLE_FREQ_MASK_88_2)
+		oss << "88.2,";
+	if (sad.sample_freq_mask & SAD_SAMPLE_FREQ_MASK_96)
+		oss << "96,";
+	if (sad.sample_freq_mask & SAD_SAMPLE_FREQ_MASK_176_4)
+		oss << "176.4,";
+	if (sad.sample_freq_mask & SAD_SAMPLE_FREQ_MASK_192)
+		oss << "192,";
+	if (sad.sample_freq_mask & (1 << 7))
+		oss << "Reserved,";
+	oss << "\b \b";
+
+	if (sad.format_code == SAD_FMT_CODE_LPCM ||
+	    (sad.format_code == SAD_FMT_CODE_EXTENDED &&
+	     sad.extension_type_code == SAD_EXT_TYPE_LPCM_3D_AUDIO)) {
+		oss << ", bit depth: ";
+		if (sad.bit_depth_mask & SAD_BIT_DEPTH_MASK_16)
+			oss << "16,";
+		if (sad.bit_depth_mask & SAD_BIT_DEPTH_MASK_20)
+			oss << "20,";
+		if (sad.bit_depth_mask & SAD_BIT_DEPTH_MASK_24)
+			oss << "24,";
+		oss << "\b \b";
+	} else if (sad.format_code >= 2 && sad.format_code <= 8)
+		oss << " max bitrate (kbit/s): " << 8 * sad.max_bitrate;
+
+	if (sad.format_code == SAD_FMT_CODE_EXTENDED) {
+		switch (sad.extension_type_code) {
+		case 4:
+		case 5:
+		case 6:
+		case 8:
+		case 10:
+			oss << ", frame length: ";
+			if (sad.frame_length_mask & SAD_FRAME_LENGTH_MASK_960)
+				oss << "960,";
+			if (sad.frame_length_mask & SAD_FRAME_LENGTH_MASK_1024)
+				oss << "1024,";
+			oss << "\b";
+			break;
+		}
+
+		if (sad.extension_type_code == 8 || sad.extension_type_code == 10)
+			oss << ", MPS";
+	}
+
+	return oss.str();
+}
+
+static void sad_decode(struct short_audio_desc *sad, __u32 descriptor)
+{
+	__u8 b1 = (descriptor >> 16) & 0xff;
+	__u8 b2 = (descriptor >> 8) & 0xff;
+	__u8 b3 = descriptor & 0xff;
+
+	sad->num_channels = (b1 & 0x07) + 1;
+	sad->format_code = (b1 >> 3) & 0x0f;
+	sad->sample_freq_mask = b2;
+
+	switch (sad->format_code) {
+	case SAD_FMT_CODE_LPCM:
+		sad->bit_depth_mask = b3 & 0x07;
+		break;
+	case 2:
+	case 3:
+	case 4:
+	case 5:
+	case 6:
+	case 7:
+	case 8:
+		sad->max_bitrate = b3;
+		break;
+	case 9:
+	case 10:
+	case 11:
+	case 12:
+	case 13:
+		sad->format_dependent = b3;
+		break;
+	case SAD_FMT_CODE_WMA_PRO:
+		sad->wma_profile = b3 & 0x03;
+		break;
+	case SAD_FMT_CODE_EXTENDED:
+	        sad->extension_type_code = (b3 >> 3) & 0x1f;
+
+		switch (sad->extension_type_code) {
+		case 4:
+		case 5:
+		case 6:
+			sad->frame_length_mask = (b3 >> 1) & 0x03;
+			break;
+		case 8:
+		case 10:
+			sad->frame_length_mask = (b3 >> 1) & 0x03;
+			sad->mps = b3 & 1;
+			break;
+		case SAD_EXT_TYPE_MPEG_H_3D_AUDIO:
+		case SAD_EXT_TYPE_AC_4:
+			sad->format_dependent = b3 & 0x07;
+			fallthrough;
+		case SAD_EXT_TYPE_LPCM_3D_AUDIO:
+			sad->bit_depth_mask = b3 & 0x07;
+			break;
+		}
+		break;
+	}
+}
 
 /* Dynamic Auto Lipsync */
 
 static int dal_request_current_latency(struct node *node, unsigned me, unsigned la, bool interactive)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	cec_msg_init(&msg, me, la);
 	cec_msg_request_current_latency(&msg, true, node->remote[la].phys_addr);
@@ -48,6 +237,13 @@ static int dal_request_current_latency(struct node *node, unsigned me, unsigned
 
 	cec_ops_report_current_latency(&msg, &phys_addr, &video_latency, &low_latency_mode,
 				       &audio_out_compensated, &audio_out_delay);
+	// cec_ops_report_current_latency will hardcode audio_out_delay
+	// if it is unused, but for this test we want the real value, so
+	// get it from the actual message.
+	if (msg.len >= 7)
+		audio_out_delay = msg.msg[6];
+	else
+		audio_out_delay = 1;
 	fail_on_test(phys_addr != node->remote[la].phys_addr);
 	info("Video latency: %d (%dms)\n", video_latency, (video_latency - 1) * 2);
 	info("Low latency mode: %d\n", low_latency_mode);
@@ -57,6 +253,14 @@ static int dal_request_current_latency(struct node *node, unsigned me, unsigned
 		fail_on_test(audio_out_delay == 0 || audio_out_delay > 251);
 		// Warn if the delay is more than 50 ms
 		warn_on_test(audio_out_delay > (50 / 2) + 1);
+	} else {
+		// Although this value will be ignored, it shouldn't use
+		// reserved values.
+		if (audio_out_delay == 0 || audio_out_delay > 251)
+			warn("Audio out delay is set to a reserved value (%d), set it to 1 instead (recommended value when this field is unused).\n", audio_out_delay);
+		else if (audio_out_delay != 1)
+			warn("Audio out delay is %d (%dms), but value 1 is recommended when this field is unused.\n",
+			     audio_out_delay, (audio_out_delay - 1) * 2);
 	}
 	fail_on_test(video_latency == 0 || video_latency > 251);
 	// Warn if the delay is more than 50 ms and low latency mode is set
@@ -72,7 +276,7 @@ static int dal_request_current_latency(struct node *node, unsigned me, unsigned
 
 static int dal_req_current_latency_invalid(struct node *node, unsigned me, unsigned la, bool interactive)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	/* Test that there is no reply when the physical address operand is not the
 	   physical address of the remote device. */
@@ -84,14 +288,11 @@ static int dal_req_current_latency_invalid(struct node *node, unsigned me, unsig
 	return 0;
 }
 
-struct remote_subtest dal_subtests[] = {
+const vec_remote_subtests dal_subtests{
 	{ "Request Current Latency", CEC_LOG_ADDR_MASK_ALL, dal_request_current_latency },
 	{ "Request Current Latency with invalid PA", CEC_LOG_ADDR_MASK_ALL, dal_req_current_latency_invalid },
 };
 
-const unsigned dal_subtests_size = ARRAY_SIZE(dal_subtests);
-
-
 /* Audio Return Channel Control */
 
 static __u16 pa_common_mask(__u16 pa1, __u16 pa2)
@@ -133,7 +334,7 @@ static int arc_initiate_tx(struct node *node, unsigned me, unsigned la, bool int
 	if (pa_is_upstream_from(node->phys_addr, node->remote[la].phys_addr))
 		return NOTAPPLICABLE;
 
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	/*
 	 * Note that this is a special case: INITIATE_ARC can reply with two possible
@@ -176,7 +377,7 @@ static int arc_terminate_tx(struct node *node, unsigned me, unsigned la, bool in
 	if (!node->remote[la].arc_initiated)
 		return NOTAPPLICABLE;
 
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	cec_msg_init(&msg, me, la);
 	cec_msg_terminate_arc(&msg, true);
@@ -203,7 +404,7 @@ static int arc_initiate_rx(struct node *node, unsigned me, unsigned la, bool int
 	if (pa_is_upstream_from(node->remote[la].phys_addr, node->phys_addr))
 		return NOTAPPLICABLE;
 
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	cec_msg_init(&msg, me, la);
 	cec_msg_request_arc_initiation(&msg, true);
@@ -251,7 +452,7 @@ static int arc_terminate_rx(struct node *node, unsigned me, unsigned la, bool in
 	if (!node->remote[la].arc_initiated)
 		return NOTAPPLICABLE;
 
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	cec_msg_init(&msg, me, la);
 	cec_msg_request_arc_termination(&msg, true);
@@ -276,16 +477,13 @@ static int arc_terminate_rx(struct node *node, unsigned me, unsigned la, bool in
 	return 0;
 }
 
-struct remote_subtest arc_subtests[] = {
+const vec_remote_subtests arc_subtests{
 	{ "Initiate ARC (RX)", CEC_LOG_ADDR_MASK_ALL, arc_initiate_rx },
 	{ "Terminate ARC (RX)", CEC_LOG_ADDR_MASK_ALL, arc_terminate_rx },
 	{ "Initiate ARC (TX)", CEC_LOG_ADDR_MASK_ALL, arc_initiate_tx },
 	{ "Terminate ARC (TX)", CEC_LOG_ADDR_MASK_ALL, arc_terminate_tx },
 };
 
-const unsigned arc_subtests_size = ARRAY_SIZE(arc_subtests);
-
-
 /* System Audio Control */
 
 /*
@@ -328,7 +526,7 @@ const unsigned arc_subtests_size = ARRAY_SIZE(arc_subtests);
 
 static int sac_request_sad_probe(struct node *node, unsigned me, unsigned la, bool interactive)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 	__u8 audio_format_id = 0;
 	__u8 audio_format_code = 1;
 
@@ -352,7 +550,7 @@ static int sac_request_sad_invalid(struct node *node, unsigned me, unsigned la,
 	if (!node->remote[la].has_sad)
 		return NOTAPPLICABLE;
 
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 	__u8 audio_format_id = CEC_OP_AUD_FMT_ID_CEA861;
 	__u8 audio_format_code = 63; // This is outside the range of CEA861-F
 
@@ -374,7 +572,7 @@ static int sac_sad_format_check(struct node *node, unsigned me, unsigned la, boo
 	if (!node->remote[la].has_sad)
 		return NOTAPPLICABLE;
 
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 	__u8 audio_format_id;
 	__u8 audio_format_code;
 
@@ -402,7 +600,7 @@ static int sac_sad_format_check(struct node *node, unsigned me, unsigned la, boo
 				warn("The device has CEC version < 2.0 but reports audio format(s) introduced in CEC 2.0.\n");
 
 			for (int j = 0; j < num_descriptors; j++) {
-				struct short_audio_desc sad;
+				struct short_audio_desc sad = {};
 
 				sad_decode(&sad, descriptors[j]);
 				if ((id == 0 && sad.format_code != fmt_code) ||
@@ -429,7 +627,7 @@ static int sac_sad_req_multiple(struct node *node, unsigned me, unsigned la, boo
 	/* Check that if we got a response to a Request Short Audio Descriptor
 	   with a single format, we also get a response when the same audio format
 	   occurs in a request together with other formats. */
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 	__u8 audio_format_id[4] = { };
 	__u8 audio_format_code[4];
 
@@ -450,7 +648,7 @@ static int sac_sad_req_multiple(struct node *node, unsigned me, unsigned la, boo
 
 static int sac_set_system_audio_mode_direct(struct node *node, unsigned me, unsigned la, bool interactive)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	cec_msg_init(&msg, me, la);
 	cec_msg_set_system_audio_mode(&msg, CEC_OP_SYS_AUD_STATUS_ON);
@@ -468,7 +666,7 @@ static int sac_set_system_audio_mode_direct(struct node *node, unsigned me, unsi
 
 static int sac_set_system_audio_mode_broadcast_on(struct node *node, unsigned me, unsigned la, bool interactive)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	cec_msg_init(&msg, me, CEC_LOG_ADDR_BROADCAST);
 	cec_msg_set_system_audio_mode(&msg, CEC_OP_SYS_AUD_STATUS_ON);
@@ -479,7 +677,7 @@ static int sac_set_system_audio_mode_broadcast_on(struct node *node, unsigned me
 
 static int sac_system_audio_mode_status(struct node *node, unsigned me, unsigned la, bool interactive)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	/* The device shall not feature abort System Audio Status if it did not
 	   feature abort Set System Audio Mode.
@@ -503,7 +701,7 @@ static int sac_system_audio_mode_status(struct node *node, unsigned me, unsigned
 
 static int sac_set_system_audio_mode_broadcast_off(struct node *node, unsigned me, unsigned la, bool interactive)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	cec_msg_init(&msg, me, CEC_LOG_ADDR_BROADCAST);
 	cec_msg_set_system_audio_mode(&msg, CEC_OP_SYS_AUD_STATUS_OFF);
@@ -514,7 +712,7 @@ static int sac_set_system_audio_mode_broadcast_off(struct node *node, unsigned m
 
 static int sac_system_audio_mode_req_on(struct node *node, unsigned me, unsigned la, bool interactive)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 	__u8 status;
 
 	/* Send a System Audio Mode Request to the audio system. This notifies the
@@ -543,7 +741,7 @@ static int sac_system_audio_mode_req_on(struct node *node, unsigned me, unsigned
 
 static int sac_give_system_audio_mode_status(struct node *node, unsigned me, unsigned la, bool interactive)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 	__u8 system_audio_status;
 
 	/* The device shall not feature abort Give System Audio Mode Status if it did not
@@ -571,7 +769,7 @@ static int sac_give_system_audio_mode_status(struct node *node, unsigned me, uns
 
 static int sac_give_audio_status(struct node *node, unsigned me, unsigned la, bool interactive)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	/* Give Audio Status is mandatory for audio systems in CEC 2.0, except
 	   for systems that lack external controls for volume/mute status. */
@@ -597,7 +795,7 @@ static int sac_give_audio_status(struct node *node, unsigned me, unsigned la, bo
 
 static int sac_util_send_user_control_press(struct node *node, unsigned me, unsigned la, __u8 ui_cmd)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 	struct cec_op_ui_command rc_press = {};
 
 	/* The device shall not feature abort
@@ -716,7 +914,7 @@ static int sac_user_control_press_restore_volume_function(struct node *node, uns
 
 static int sac_user_control_release(struct node *node, unsigned me, unsigned la, bool interactive)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	/* The device shall not feature abort User Control Released if it did not
 	   feature abort System Audio Mode Request
@@ -743,7 +941,7 @@ static int sac_system_audio_mode_req_off(struct node *node, unsigned me, unsigne
 	if (!node->remote[la].has_sys_audio_mode_req)
 		return NOTAPPLICABLE;
 
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 	__u8 status;
 
 	cec_msg_init(&msg, me, la);
@@ -764,75 +962,104 @@ static int sac_system_audio_mode_req_off(struct node *node, unsigned me, unsigne
 	return 0;
 }
 
-struct remote_subtest sac_subtests[] = {
-	{ "Request Short Audio Descriptor",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
-	  sac_request_sad_probe },
-	{ "Request Short Audio Descriptor, invalid",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
-	  sac_request_sad_invalid },
-	{ "Report Short Audio Descriptor consistency",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
-	  sac_sad_format_check },
-	{ "Report Short Audio Descriptor, multiple requests in one",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
-	  sac_sad_req_multiple },
-	{ "Set System Audio Mode (directly addressed)",
-	  CEC_LOG_ADDR_MASK_TV,
-	  sac_set_system_audio_mode_direct },
-	{ "Set System Audio Mode (broadcast on)",
-	  CEC_LOG_ADDR_MASK_TV,
-	  sac_set_system_audio_mode_broadcast_on },
-	{ "System Audio Mode Status",
-	  CEC_LOG_ADDR_MASK_TV,
-	  sac_system_audio_mode_status },
-	{ "System Audio Mode Request (on)",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
-	  sac_system_audio_mode_req_on },
-	{ "Give System Audio Mode Status",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
-	  sac_give_system_audio_mode_status },
-	{ "Give Audio Status",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
-	  sac_give_audio_status },
-	{ "User Control Pressed (Volume Up)",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
-	  sac_user_control_press_vol_up },
-	{ "User Control Pressed (Volume Down)",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
-	  sac_user_control_press_vol_down },
-	{ "User Control Pressed (Mute)",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
-	  sac_user_control_press_mute },
-	{ "User Control Pressed (Restore Volume Function)",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
-	  sac_user_control_press_restore_volume_function },
-	{ "User Control Pressed (Mute Function)",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
-	  sac_user_control_press_mute_function },
-	{ "User Control Released (Audio)",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
-	  sac_user_control_release },
-	{ "Set System Audio Mode (broadcast off)",
-	  CEC_LOG_ADDR_MASK_TV,
-	  sac_set_system_audio_mode_broadcast_off },
-	{ "System Audio Mode Request (off)",
-	  CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
-	  sac_system_audio_mode_req_off },
+const vec_remote_subtests sac_subtests{
+	{
+		"Request Short Audio Descriptor",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
+		sac_request_sad_probe,
+	},
+	{
+		"Request Short Audio Descriptor, invalid",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
+		sac_request_sad_invalid,
+	},
+	{
+		"Report Short Audio Descriptor consistency",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
+		sac_sad_format_check,
+	},
+	{
+		"Report Short Audio Descriptor, multiple requests in one",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
+		sac_sad_req_multiple,
+	},
+	{
+		"Set System Audio Mode (directly addressed)",
+		CEC_LOG_ADDR_MASK_TV,
+		sac_set_system_audio_mode_direct,
+	},
+	{
+		"Set System Audio Mode (broadcast on)",
+		CEC_LOG_ADDR_MASK_TV,
+		sac_set_system_audio_mode_broadcast_on,
+	},
+	{
+		"System Audio Mode Status",
+		CEC_LOG_ADDR_MASK_TV,
+		sac_system_audio_mode_status,
+	},
+	{
+		"System Audio Mode Request (on)",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
+		sac_system_audio_mode_req_on,
+	},
+	{
+		"Give System Audio Mode Status",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
+		sac_give_system_audio_mode_status,
+	},
+	{
+		"Give Audio Status",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
+		sac_give_audio_status,
+	},
+	{
+		"User Control Pressed (Volume Up)",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
+		sac_user_control_press_vol_up,
+	},
+	{
+		"User Control Pressed (Volume Down)",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
+		sac_user_control_press_vol_down,
+	},
+	{
+		"User Control Pressed (Mute)",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
+		sac_user_control_press_mute,
+	},
+	{
+		"User Control Pressed (Restore Volume Function)",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
+		sac_user_control_press_restore_volume_function,
+	},
+	{
+		"User Control Pressed (Mute Function)",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
+		sac_user_control_press_mute_function,
+	},
+	{
+		"User Control Released (Audio)",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM | CEC_LOG_ADDR_MASK_TV,
+		sac_user_control_release,
+	},
+	{
+		"Set System Audio Mode (broadcast off)",
+		CEC_LOG_ADDR_MASK_TV,
+		sac_set_system_audio_mode_broadcast_off,
+	},
+	{
+		"System Audio Mode Request (off)",
+		CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
+		sac_system_audio_mode_req_off,
+	},
 };
 
-const unsigned sac_subtests_size = ARRAY_SIZE(sac_subtests);
-
-
 /* Audio Rate Control */
 
-/*
-  TODO: These are very rudimentary tests which should be expanded.
- */
-
 static int audio_rate_ctl_set_audio_rate(struct node *node, unsigned me, unsigned la, bool interactive)
 {
-	struct cec_msg msg = {};
+	struct cec_msg msg;
 
 	cec_msg_init(&msg, me, la);
 	cec_msg_set_audio_rate(&msg, CEC_OP_AUD_RATE_WIDE_STD);
@@ -850,10 +1077,66 @@ static int audio_rate_ctl_set_audio_rate(struct node *node, unsigned me, unsigne
 	return OK_PRESUMED;
 }
 
-struct remote_subtest audio_rate_ctl_subtests[] = {
-	{ "Set Audio Rate",
-	  CEC_LOG_ADDR_MASK_PLAYBACK | CEC_LOG_ADDR_MASK_RECORD | CEC_LOG_ADDR_MASK_TUNER,
-	  audio_rate_ctl_set_audio_rate },
-};
+static int audio_rate_ctl_active_sensing(struct node *node, unsigned me, unsigned la, bool interactive)
+{
+	/*
+	 * The source shall go back to Rate Control Off if no Set Audio Rate message is
+	 * received for more than 2 seconds.
+	 */
+	if (!node->remote[la].has_aud_rate)
+		return NOTAPPLICABLE;
 
-const unsigned audio_rate_ctl_subtests_size = ARRAY_SIZE(audio_rate_ctl_subtests);
+	struct cec_msg msg;
+
+	cec_msg_init(&msg, me, la);
+
+	/*
+	 * Since this subtest runs immediately after Set Audio Rate, delaying the interval
+	 * between the two tests is sufficient to test that the Source turns off rate control.
+	 */
+	sleep(3);
+	cec_msg_set_audio_rate(&msg, CEC_OP_AUD_RATE_OFF);
+	fail_on_test(!transmit_timeout(node, &msg));
+	fail_on_test_v2(node->remote[la].cec_version, unrecognized_op(&msg));
+	return OK_PRESUMED;
+}
+
+static int audio_rate_ctl_invalid(struct node *node, unsigned me, unsigned la, bool interactive)
+{
+	if (!node->remote[la].has_aud_rate)
+		return NOTAPPLICABLE;
+
+	struct cec_msg msg;
+
+	cec_msg_init(&msg, me, la);
+	cec_msg_set_audio_rate(&msg, 0xa); /* Invalid Audio Rate Control message operand */
+	fail_on_test(!transmit_timeout(node, &msg));
+	fail_on_test(timed_out(&msg));
+	fail_on_test(!cec_msg_status_is_abort(&msg));
+	if (abort_reason(&msg) != CEC_OP_ABORT_INVALID_OP) {
+		warn("Expected Feature Abort [Invalid operand]\n");
+		return FAIL;
+	}
+	return OK;
+}
+
+const vec_remote_subtests audio_rate_ctl_subtests{
+	{
+		"Set Audio Rate",
+		CEC_LOG_ADDR_MASK_PLAYBACK | CEC_LOG_ADDR_MASK_RECORD |
+		CEC_LOG_ADDR_MASK_TUNER | CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
+		audio_rate_ctl_set_audio_rate,
+	},
+	{
+		"Audio Rate Active Sensing",
+		CEC_LOG_ADDR_MASK_PLAYBACK | CEC_LOG_ADDR_MASK_RECORD |
+		CEC_LOG_ADDR_MASK_TUNER | CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
+		audio_rate_ctl_active_sensing,
+	},
+	{
+		"Audio Rate Invalid Operand",
+		CEC_LOG_ADDR_MASK_PLAYBACK | CEC_LOG_ADDR_MASK_RECORD |
+		CEC_LOG_ADDR_MASK_TUNER | CEC_LOG_ADDR_MASK_AUDIOSYSTEM,
+		audio_rate_ctl_invalid,
+	},
+};
