@@ -3,21 +3,15 @@
  * Copyright 2016 Cisco Systems, Inc. and/or its affiliates. All rights reserved.
  */
 
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <inttypes.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/time.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
+#include <cerrno>
+#include <ctime>
+#include <string>
+
 #include <sys/ioctl.h>
-#include <config.h>
+#include <sys/time.h>
 
 #include "cec-follower.h"
+#include "compiler.h"
 
 #define VOLUME_MAX 0x64
 #define VOLUME_MIN 0
@@ -35,8 +29,11 @@
 /* Time between each polling message sent to a device */
 #define POLL_PERIOD 15000
 
-#define ARRAY_SIZE(a) \
-	(sizeof(a) / sizeof(*a))
+/* The maximum interval in nanoseconds between audio rate messages as defined in the spec */
+#define MAX_AUD_RATE_MSG_INTERVAL_NS (2 * 1000000000ULL)
+
+/* The maximum interval in nanoseconds to allow a deck to skip forward/reverse */
+#define MAX_DECK_SKIP_NS (2 * 1000000000ULL)
 
 struct cec_enum_values {
 	const char *type_name;
@@ -47,105 +44,12 @@ struct la_info la_info[15];
 
 static struct timespec start_monotonic;
 static struct timeval start_timeofday;
-
-static const struct cec_enum_values type_ui_cmd[] = {
-	{ "Select", 0x00 },
-	{ "Up", 0x01 },
-	{ "Down", 0x02 },
-	{ "Left", 0x03 },
-	{ "Right", 0x04 },
-	{ "Right-Up", 0x05 },
-	{ "Right-Down", 0x06 },
-	{ "Left-Up", 0x07 },
-	{ "Left-Down", 0x08 },
-	{ "Device Root Menu", 0x09 },
-	{ "Device Setup Menu", 0x0a },
-	{ "Contents Menu", 0x0b },
-	{ "Favorite Menu", 0x0c },
-	{ "Back", 0x0d },
-	{ "Media Top Menu", 0x10 },
-	{ "Media Context-sensitive Menu", 0x11 },
-	{ "Number Entry Mode", 0x1d },
-	{ "Number 11", 0x1e },
-	{ "Number 12", 0x1f },
-	{ "Number 0 or Number 10", 0x20 },
-	{ "Number 1", 0x21 },
-	{ "Number 2", 0x22 },
-	{ "Number 3", 0x23 },
-	{ "Number 4", 0x24 },
-	{ "Number 5", 0x25 },
-	{ "Number 6", 0x26 },
-	{ "Number 7", 0x27 },
-	{ "Number 8", 0x28 },
-	{ "Number 9", 0x29 },
-	{ "Dot", 0x2a },
-	{ "Enter", 0x2b },
-	{ "Clear", 0x2c },
-	{ "Next Favorite", 0x2f },
-	{ "Channel Up", 0x30 },
-	{ "Channel Down", 0x31 },
-	{ "Previous Channel", 0x32 },
-	{ "Sound Select", 0x33 },
-	{ "Input Select", 0x34 },
-	{ "Display Information", 0x35 },
-	{ "Help", 0x36 },
-	{ "Page Up", 0x37 },
-	{ "Page Down", 0x38 },
-	{ "Power", 0x40 },
-	{ "Volume Up", 0x41 },
-	{ "Volume Down", 0x42 },
-	{ "Mute", 0x43 },
-	{ "Play", 0x44 },
-	{ "Stop", 0x45 },
-	{ "Pause", 0x46 },
-	{ "Record", 0x47 },
-	{ "Rewind", 0x48 },
-	{ "Fast forward", 0x49 },
-	{ "Eject", 0x4a },
-	{ "Skip Forward", 0x4b },
-	{ "Skip Backward", 0x4c },
-	{ "Stop-Record", 0x4d },
-	{ "Pause-Record", 0x4e },
-	{ "Angle", 0x50 },
-	{ "Sub picture", 0x51 },
-	{ "Video on Demand", 0x52 },
-	{ "Electronic Program Guide", 0x53 },
-	{ "Timer Programming", 0x54 },
-	{ "Initial Configuration", 0x55 },
-	{ "Select Broadcast Type", 0x56 },
-	{ "Select Sound Presentation", 0x57 },
-	{ "Audio Description", 0x58 },
-	{ "Internet", 0x59 },
-	{ "3D Mode", 0x5a },
-	{ "Play Function", 0x60 },
-	{ "Pause-Play Function", 0x61 },
-	{ "Record Function", 0x62 },
-	{ "Pause-Record Function", 0x63 },
-	{ "Stop Function", 0x64 },
-	{ "Mute Function", 0x65 },
-	{ "Restore Volume Function", 0x66 },
-	{ "Tune Function", 0x67 },
-	{ "Select Media Function", 0x68 },
-	{ "Select A/V Input Function", 0x69 },
-	{ "Select Audio Input Function", 0x6a },
-	{ "Power Toggle Function", 0x6b },
-	{ "Power Off Function", 0x6c },
-	{ "Power On Function", 0x6d },
-	{ "F1 (Blue)", 0x71 },
-	{ "F2 (Red)", 0x72 },
-	{ "F3 (Green)", 0x73 },
-	{ "F4 (Yellow)", 0x74 },
-	{ "F5", 0x75 },
-	{ "Data", 0x76 },
-};
+static constexpr time_t time_to_transient = 1;
+static constexpr time_t time_to_stable = 8;
 
 static const char *get_ui_cmd_string(__u8 ui_cmd)
 {
-	for (unsigned i = 0; i < ARRAY_SIZE(type_ui_cmd); i++) {
-		if (type_ui_cmd[i].value == ui_cmd)
-			return type_ui_cmd[i].type_name;
-	}
-	return "Unknown";
+	return cec_log_ui_cmd_string(ui_cmd) ? : "Unknown";
 }
 
 static std::string ts2s(__u64 ts, bool wallclock)
@@ -184,8 +88,7 @@ static void log_event(struct cec_event &ev, bool wallclock)
 	case CEC_EVENT_STATE_CHANGE:
 		pa = ev.state_change.phys_addr;
 		printf("Event: State Change: PA: %x.%x.%x.%x, LA mask: 0x%04x\n",
-		       pa >> 12, (pa >> 8) & 0xf,
-		       (pa >> 4) & 0xf, pa & 0xf,
+		       cec_phys_addr_exp(pa),
 		       ev.state_change.log_addr_mask);
 		break;
 	case CEC_EVENT_LOST_MSGS:
@@ -226,7 +129,7 @@ void reply_feature_abort(struct node *node, struct cec_msg *msg, __u8 reason)
 			   the same message to the same Follower again at that time to avoid saturating
 			   the bus. */
 			warn("Received message %s from LA %d (%s) shortly after\n",
-			     opcode2s(msg).c_str(), la, la2s(la));
+			     opcode2s(msg).c_str(), la, cec_la2s(la));
 			warn("replying Feature Abort [Unrecognized Opcode] to the same message.\n");
 		}
 	}
@@ -243,24 +146,37 @@ void reply_feature_abort(struct node *node, struct cec_msg *msg, __u8 reason)
 
 static bool exit_standby(struct node *node)
 {
+	/* Cancel any standby request that was pending. */
+	node->state.record_received_standby = false;
+
 	if (node->state.power_status == CEC_OP_POWER_STATUS_STANDBY ||
 	    node->state.power_status == CEC_OP_POWER_STATUS_TO_STANDBY) {
 		node->state.old_power_status = node->state.power_status;
 		node->state.power_status = CEC_OP_POWER_STATUS_ON;
-		node->state.power_status_changed_time = time(NULL);
+		node->state.power_status_changed_time = time(nullptr);
 		dev_info("Changing state to on\n");
 		return true;
 	}
 	return false;
 }
 
-static bool enter_standby(struct node *node)
+bool enter_standby(struct node *node)
 {
 	if (node->state.power_status == CEC_OP_POWER_STATUS_ON ||
 	    node->state.power_status == CEC_OP_POWER_STATUS_TO_ON) {
+		/*
+		 * Standby should not interrupt a recording in progress, but
+		 * remember to go to standby once the recording is finished.
+		 */
+		if (node->state.one_touch_record_on) {
+			node->state.record_received_standby = true;
+			return false;
+		}
 		node->state.old_power_status = node->state.power_status;
 		node->state.power_status = CEC_OP_POWER_STATUS_STANDBY;
-		node->state.power_status_changed_time = time(NULL);
+		node->state.power_status_changed_time = time(nullptr);
+		node->state.deck_skip_start = 0;
+		node->state.record_received_standby = false;
 		dev_info("Changing state to standby\n");
 		return true;
 	}
@@ -308,9 +224,7 @@ static bool pa_are_adjacent(__u16 pa1, __u16 pa2)
 		return false;
 	if ((pa1 & trail_mask) || (pa2 & trail_mask))
 		return false;
-	if (!((pa1 & ~mask) && (pa2 & ~mask)))
-		return true;
-	return false;
+	return !((pa1 & ~mask) && (pa2 & ~mask));
 }
 
 static bool pa_is_upstream_from(__u16 pa1, __u16 pa2)
@@ -319,19 +233,62 @@ static bool pa_is_upstream_from(__u16 pa1, __u16 pa2)
 
 	if (pa1 == CEC_PHYS_ADDR_INVALID || pa2 == CEC_PHYS_ADDR_INVALID)
 		return false;
-	if (!(pa1 & ~mask) && (pa2 & ~mask))
-		return true;
-	return false;
+	return !(pa1 & ~mask) && (pa2 & ~mask);
 }
 
-static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
+static __u8 current_power_state(struct node *node)
+{
+	time_t t = time(nullptr);
+
+	if (t - node->state.power_status_changed_time <= time_to_transient)
+		return node->state.old_power_status;
+	if (t - node->state.power_status_changed_time >= time_to_stable)
+		return node->state.power_status;
+	if (node->state.power_status == CEC_OP_POWER_STATUS_ON)
+		return CEC_OP_POWER_STATUS_TO_ON;
+	return CEC_OP_POWER_STATUS_TO_STANDBY;
+}
+
+static void aud_rate_msg_interval_check(struct node *node, __u64 ts_new)
+{
+	/*
+	 * The interval since the last audio rate message is only relevant
+	 * if the Source is currently in audio rate controlled mode
+	 * (i.e. state.last_aud_rate_rx_ts != 0).
+	 */
+	__u64 ts_old = node->state.last_aud_rate_rx_ts;
+
+	if (ts_old) {
+		__u64 interval = ts_new - ts_old;
+
+		if (interval > MAX_AUD_RATE_MSG_INTERVAL_NS) {
+			warn("The interval since the last Audio Rate Control message was > 2s.\n");
+			node->state.last_aud_rate_rx_ts = 0;
+		}
+	}
+}
+
+static void update_deck_state(struct node *node, unsigned me, __u8 deck_state_new)
+{
+	if (node->state.deck_state != deck_state_new) {
+		node->state.deck_state = deck_state_new;
+
+		if (node->state.deck_report_changes) {
+			struct cec_msg msg;
+
+			cec_msg_init(&msg, me, node->state.deck_report_changes_to);
+			cec_msg_deck_status(&msg, node->state.deck_state);
+			transmit(node, &msg);
+		}
+	}
+}
+
+static void processMsg(struct node *node, struct cec_msg &msg, unsigned me, __u8 type)
 {
 	__u8 to = cec_msg_destination(&msg);
 	__u8 from = cec_msg_initiator(&msg);
 	bool is_bcast = cec_msg_is_broadcast(&msg);
 	__u16 remote_pa = (from < 15) ? node->remote_phys_addr[from] : CEC_PHYS_ADDR_INVALID;
-	const time_t time_to_transient = 4;
-	const time_t time_to_stable = 8;
 
 	switch (msg.msg[1]) {
 
@@ -361,23 +318,12 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 
 		/* Give Device Power Status */
 
-	case CEC_MSG_GIVE_DEVICE_POWER_STATUS: {
-		__u8 report_status;
-
-		if (time(NULL) - node->state.power_status_changed_time <= time_to_transient)
-			report_status = node->state.old_power_status;
-		else if (time(NULL) - node->state.power_status_changed_time >= time_to_stable)
-			report_status = node->state.power_status;
-		else if (node->state.power_status == CEC_OP_POWER_STATUS_ON)
-			report_status = CEC_OP_POWER_STATUS_TO_ON;
-		else
-			report_status = CEC_OP_POWER_STATUS_TO_STANDBY;
-
+	case CEC_MSG_GIVE_DEVICE_POWER_STATUS:
 		cec_msg_set_reply_to(&msg, &msg);
-		cec_msg_report_power_status(&msg, report_status);
+		cec_msg_report_power_status(&msg, current_power_state(node));
 		transmit(node, &msg);
 		return;
-	}
+
 	case CEC_MSG_REPORT_POWER_STATUS:
 		// Nothing to do here for now.
 		return;
@@ -386,13 +332,7 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 		/* Standby */
 
 	case CEC_MSG_STANDBY:
-		if (node->state.power_status == CEC_OP_POWER_STATUS_ON ||
-		    node->state.power_status == CEC_OP_POWER_STATUS_TO_ON) {
-			node->state.old_power_status = node->state.power_status;
-			node->state.power_status = CEC_OP_POWER_STATUS_STANDBY;
-			node->state.power_status_changed_time = time(NULL);
-			dev_info("Changing state to standby\n");
-		}
+		enter_standby(node);
 		return;
 
 
@@ -406,6 +346,22 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 		dev_info("New active source: %x.%x.%x.%x\n", cec_phys_addr_exp(phys_addr));
 		return;
 	}
+	case CEC_MSG_INACTIVE_SOURCE: {
+		__u16 phys_addr;
+
+		if (node->phys_addr)
+			break;
+
+		cec_ops_active_source(&msg, &phys_addr);
+		if (node->state.active_source_pa != phys_addr)
+			break;
+		node->state.active_source_pa = 0;
+		cec_msg_init(&msg, me, CEC_LOG_ADDR_BROADCAST);
+		cec_msg_active_source(&msg, node->phys_addr);
+		transmit(node, &msg);
+		dev_info("New active source: 0.0.0.0\n");
+		return;
+	}
 	case CEC_MSG_IMAGE_VIEW_ON:
 	case CEC_MSG_TEXT_VIEW_ON:
 		if (!cec_has_tv(1 << me))
@@ -433,9 +389,9 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 
 		if (cec_has_tv(1 << la) && la_info[la].phys_addr == 0)
 			warn("TV (0) at 0.0.0.0 sent Routing Information.");
+		return;
 	}
 
-
 		/* System Information */
 
 	case CEC_MSG_GET_MENU_LANGUAGE:
@@ -452,8 +408,10 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 		__u8 prim_dev_type;
 
 		cec_ops_report_physical_addr(&msg, &phys_addr, &prim_dev_type);
-		if (from < 15)
+		if (from < 15) {
 			node->remote_phys_addr[from] = phys_addr;
+			node->remote_prim_devtype[from] = prim_dev_type;
+		}
 		return;
 	}
 
@@ -505,35 +463,35 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 		node->state.rc_press_rx_ts = msg.rx_ts;
 
 		switch (rc_press.ui_cmd) {
-		case 0x41:
+		case CEC_OP_UI_CMD_VOLUME_UP:
 			if (node->state.volume < VOLUME_MAX)
 				node->state.volume++;
 			break;
-		case 0x42:
+		case CEC_OP_UI_CMD_VOLUME_DOWN:
 			if (node->state.volume > VOLUME_MIN)
 				node->state.volume--;
 			break;
-		case 0x43:
+		case CEC_OP_UI_CMD_MUTE:
 			node->state.mute = !node->state.mute;
 			break;
-		case 0x65:
+		case CEC_OP_UI_CMD_MUTE_FUNCTION:
 			node->state.mute = true;
 			break;
-		case 0x66:
+		case CEC_OP_UI_CMD_RESTORE_VOLUME_FUNCTION:
 			node->state.mute = false;
 			break;
-		case 0x6B:
+		case CEC_OP_UI_CMD_POWER_TOGGLE_FUNCTION:
 			if (!enter_standby(node))
 				exit_standby(node);
 			break;
-		case 0x6C:
+		case CEC_OP_UI_CMD_POWER_OFF_FUNCTION:
 			enter_standby(node);
 			break;
-		case 0x6D:
+		case CEC_OP_UI_CMD_POWER_ON_FUNCTION:
 			exit_standby(node);
 			break;
 		}
-		if (rc_press.ui_cmd >= 0x41 && rc_press.ui_cmd <= 0x43) {
+		if (rc_press.ui_cmd >= CEC_OP_UI_CMD_VOLUME_UP && rc_press.ui_cmd <= CEC_OP_UI_CMD_MUTE) {
 			dev_info("Volume: %d%s\n", node->state.volume, node->state.mute ? ", muted" : "");
 			cec_msg_set_reply_to(&msg, &msg);
 			cec_msg_report_audio_status(&msg, node->state.mute ? 1 : 0, node->state.volume);
@@ -580,31 +538,130 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 		break;
 
 
-		/*
-		  Deck Control
-
-		  This is only a basic implementation.
-
-		  TODO: Device state should reflect whether we are playing,
-		  fast forwarding, etc.
-		*/
+		/* Deck Control */
 
 	case CEC_MSG_GIVE_DECK_STATUS:
-		if (node->has_deck_ctl) {
+		if (!node->has_deck_ctl)
+			break;
+
+		__u8 status_req;
+		cec_ops_give_deck_status(&msg, &status_req);
+
+		switch (status_req) {
+		case CEC_OP_STATUS_REQ_ON:
+			node->state.deck_report_changes = true;
+			node->state.deck_report_changes_to = cec_msg_initiator(&msg);
+			fallthrough;
+		case CEC_OP_STATUS_REQ_ONCE:
 			cec_msg_set_reply_to(&msg, &msg);
-			cec_msg_deck_status(&msg, CEC_OP_DECK_INFO_STOP);
+			cec_msg_deck_status(&msg, node->state.deck_state);
+			transmit(node, &msg);
+			return;
+		case CEC_OP_STATUS_REQ_OFF:
+			node->state.deck_report_changes = false;
+			return;
+		default:
+			reply_feature_abort(node, &msg, CEC_OP_ABORT_INVALID_OP);
+			return;
+		}
+	case CEC_MSG_PLAY:
+		if (!node->has_deck_ctl)
+			break;
+
+		__u8 deck_state;
+		__u8 play_mode;
+
+		cec_ops_play(&msg, &play_mode);
+
+		switch (play_mode) {
+		case CEC_OP_PLAY_MODE_PLAY_FWD:
+			exit_standby(node);
+			deck_state = CEC_OP_DECK_INFO_PLAY;
+			break;
+		case CEC_OP_PLAY_MODE_PLAY_REV:
+			deck_state = CEC_OP_DECK_INFO_PLAY_REV;
+			break;
+		case CEC_OP_PLAY_MODE_PLAY_STILL:
+			deck_state = CEC_OP_DECK_INFO_STILL;
+			break;
+		case CEC_OP_PLAY_MODE_PLAY_FAST_FWD_MIN:
+		case CEC_OP_PLAY_MODE_PLAY_FAST_FWD_MED:
+		case CEC_OP_PLAY_MODE_PLAY_FAST_FWD_MAX:
+			deck_state = CEC_OP_DECK_INFO_FAST_FWD;
+			break;
+		case CEC_OP_PLAY_MODE_PLAY_FAST_REV_MIN:
+		case CEC_OP_PLAY_MODE_PLAY_FAST_REV_MED:
+		case CEC_OP_PLAY_MODE_PLAY_FAST_REV_MAX:
+			deck_state = CEC_OP_DECK_INFO_FAST_REV;
+			break;
+		case CEC_OP_PLAY_MODE_PLAY_SLOW_FWD_MIN:
+		case CEC_OP_PLAY_MODE_PLAY_SLOW_FWD_MED:
+		case CEC_OP_PLAY_MODE_PLAY_SLOW_FWD_MAX:
+			deck_state = CEC_OP_DECK_INFO_SLOW;
+			break;
+		case CEC_OP_PLAY_MODE_PLAY_SLOW_REV_MIN:
+		case CEC_OP_PLAY_MODE_PLAY_SLOW_REV_MED:
+		case CEC_OP_PLAY_MODE_PLAY_SLOW_REV_MAX:
+			deck_state = CEC_OP_DECK_INFO_SLOW_REV;
+			break;
+		default:
+			cec_msg_reply_feature_abort(&msg, CEC_OP_ABORT_INVALID_OP);
 			transmit(node, &msg);
 			return;
 		}
-		break;
-	case CEC_MSG_PLAY:
-		if (node->has_deck_ctl)
+		/* Only Play Forward and Play Still will close tray if open. */
+		if (play_mode != CEC_OP_PLAY_MODE_PLAY_FWD &&
+		    play_mode != CEC_OP_PLAY_MODE_PLAY_STILL &&
+		    node->state.deck_state == CEC_OP_DECK_INFO_NO_MEDIA) {
+			reply_feature_abort(node, &msg, CEC_OP_ABORT_INCORRECT_MODE);
 			return;
-		break;
+		}
+		node->state.deck_skip_start = 0;
+		update_deck_state(node, me, deck_state);
+		return;
 	case CEC_MSG_DECK_CONTROL:
-		if (node->has_deck_ctl)
+		if (!node->has_deck_ctl)
+			break;
+
+		__u8 deck_control_mode;
+
+		cec_ops_deck_control(&msg, &deck_control_mode);
+
+		switch (deck_control_mode) {
+		case CEC_OP_DECK_CTL_MODE_STOP:
+			deck_state = CEC_OP_DECK_INFO_STOP;
+			node->state.deck_skip_start = 0;
+			break;
+		case CEC_OP_DECK_CTL_MODE_EJECT:
+			exit_standby(node);
+			deck_state = CEC_OP_DECK_INFO_NO_MEDIA;
+			node->state.deck_skip_start = 0;
+			break;
+		case CEC_OP_DECK_CTL_MODE_SKIP_FWD:
+			/* Skip Forward will not retract the deck tray. */
+			if (node->state.deck_state == CEC_OP_DECK_INFO_NO_MEDIA) {
+				reply_feature_abort(node, &msg, CEC_OP_ABORT_INCORRECT_MODE);
+				return;
+			}
+			deck_state = CEC_OP_DECK_INFO_SKIP_FWD;
+			node->state.deck_skip_start = msg.rx_ts;
+			break;
+		case CEC_OP_DECK_CTL_MODE_SKIP_REV:
+			/* Skip Reverse will not retract the deck tray. */
+			if (node->state.deck_state == CEC_OP_DECK_INFO_NO_MEDIA) {
+				reply_feature_abort(node, &msg, CEC_OP_ABORT_INCORRECT_MODE);
+				return;
+			}
+			deck_state = CEC_OP_DECK_INFO_SKIP_REV;
+			node->state.deck_skip_start = msg.rx_ts;
+			break;
+		default:
+			cec_msg_reply_feature_abort(&msg, CEC_OP_ABORT_INVALID_OP);
+			transmit(node, &msg);
 			return;
-		break;
+		}
+		update_deck_state(node, me, deck_state);
+		return;
 	case CEC_MSG_DECK_STATUS:
 		return;
 
@@ -616,10 +673,14 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 	case CEC_MSG_SELECT_DIGITAL_SERVICE:
 	case CEC_MSG_TUNER_STEP_DECREMENT:
 	case CEC_MSG_TUNER_STEP_INCREMENT:
+		process_tuner_msgs(node, msg, me, type);
+		return;
 	case CEC_MSG_RECORD_TV_SCREEN:
 	case CEC_MSG_RECORD_ON:
 	case CEC_MSG_RECORD_OFF:
 	case CEC_MSG_RECORD_STATUS:
+		process_record_msgs(node, msg, me, type);
+		return;
 	case CEC_MSG_SET_ANALOGUE_TIMER:
 	case CEC_MSG_SET_DIGITAL_TIMER:
 	case CEC_MSG_SET_EXT_TIMER:
@@ -629,7 +690,7 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 	case CEC_MSG_SET_TIMER_PROGRAM_TITLE:
 	case CEC_MSG_TIMER_CLEARED_STATUS:
 	case CEC_MSG_TIMER_STATUS:
-		process_tuner_record_timer_msgs(node, msg, me);
+		process_timer_msgs(node, msg, me, type);
 		return;
 
 		/* Dynamic Auto Lipsync */
@@ -757,8 +818,8 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 			   signalize that we support SAC. */
 			if (cec_has_tv(1 << me))
 				return;
-			else
-				break;
+
+			break;
 		}
 		cec_ops_set_system_audio_mode(&msg, &system_audio_status);
 		if (system_audio_status == CEC_OP_SYS_AUD_STATUS_ON)
@@ -777,6 +838,7 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 		cec_msg_system_audio_mode_status(&msg, node->state.sac_active ? CEC_OP_SYS_AUD_STATUS_ON :
 						 CEC_OP_SYS_AUD_STATUS_OFF);
 		transmit(node, &msg);
+		fallthrough;
 	case CEC_MSG_GIVE_AUDIO_STATUS:
 		if (!cec_has_audiosystem(1 << me))
 			break;
@@ -794,22 +856,22 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 		const struct short_audio_desc supported_formats[] = {
 			{ 2, SAD_FMT_CODE_AC3,
 			  SAD_SAMPLE_FREQ_MASK_32 | SAD_SAMPLE_FREQ_MASK_44_1,
-			  64, 0, 0 },
+			  { 64 }, 0, 0 },
 			{ 4, SAD_FMT_CODE_AC3,
 			  SAD_SAMPLE_FREQ_MASK_32,
-			  32, 0, 0 },
+			  { 32 }, 0, 0 },
 			{ 4, SAD_FMT_CODE_ONE_BIT_AUDIO,
 			  SAD_SAMPLE_FREQ_MASK_48 | SAD_SAMPLE_FREQ_MASK_192,
-			  123, 0, 0 },
+			  { 123 }, 0, 0 },
 			{ 8, SAD_FMT_CODE_EXTENDED,
 			  SAD_SAMPLE_FREQ_MASK_96,
-			  0, 0, SAD_EXT_TYPE_DRA },
+			  { 0 }, 0, SAD_EXT_TYPE_DRA },
 			{ 2, SAD_FMT_CODE_EXTENDED,
 			  SAD_SAMPLE_FREQ_MASK_176_4,
-			  SAD_FRAME_LENGTH_MASK_960 | SAD_FRAME_LENGTH_MASK_1024, 1, SAD_EXT_TYPE_MPEG4_HE_AAC_SURROUND },
+			  { SAD_FRAME_LENGTH_MASK_960 | SAD_FRAME_LENGTH_MASK_1024 }, 1, SAD_EXT_TYPE_MPEG4_HE_AAC_SURROUND },
 			{ 2, SAD_FMT_CODE_EXTENDED,
 			  SAD_SAMPLE_FREQ_MASK_44_1,
-			  SAD_BIT_DEPTH_MASK_16 | SAD_BIT_DEPTH_MASK_24, 0, SAD_EXT_TYPE_LPCM_3D_AUDIO },
+			  { SAD_BIT_DEPTH_MASK_16 | SAD_BIT_DEPTH_MASK_24 }, 0, SAD_EXT_TYPE_LPCM_3D_AUDIO },
 		};
 
 		__u8 num_descriptors, audio_format_id[4], audio_format_code[4];
@@ -830,14 +892,14 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 		format_list.erase(format_list.end() - 1);
 		dev_info("Requested descriptors: %s\n", format_list.c_str());
 		for (unsigned i = 0; i < num_descriptors; i++) {
-			for (unsigned j = 0; j < ARRAY_SIZE(supported_formats); j++) {
+			for (const auto &supported_format : supported_formats) {
 				if (found_descs >= 4)
 					break;
 				if ((audio_format_id[i] == 0 &&
-				     audio_format_code[i] == supported_formats[j].format_code) ||
+				     audio_format_code[i] == supported_format.format_code) ||
 				    (audio_format_id[i] == 1 &&
-				     audio_format_code[i] == supported_formats[j].extension_type_code))
-					sad_encode(&supported_formats[j], &descriptors[found_descs++]);
+				     audio_format_code[i] == supported_format.extension_type_code))
+					sad_encode(&supported_format, &descriptors[found_descs++]);
 			}
 		}
 
@@ -858,18 +920,31 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 		return;
 
 
-		/*
-		  Audio Rate Control
-
-		  This is only a basic implementation.
-
-		  TODO: Set Audio Rate shall be sent at least every 2 seconds by
-		  the controlling device. This should be checked and kept track of.
-		*/
+		/* Audio Rate Control */
 
 	case CEC_MSG_SET_AUDIO_RATE:
-		if (node->has_aud_rate)
+		if (!node->has_aud_rate)
+			break;
+
+		switch (msg.msg[2]) {
+		case CEC_OP_AUD_RATE_OFF:
+			aud_rate_msg_interval_check(node, msg.rx_ts);
+			node->state.last_aud_rate_rx_ts = 0;
 			return;
+		case CEC_OP_AUD_RATE_WIDE_STD:
+		case CEC_OP_AUD_RATE_WIDE_FAST:
+		case CEC_OP_AUD_RATE_WIDE_SLOW:
+		case CEC_OP_AUD_RATE_NARROW_STD:
+		case CEC_OP_AUD_RATE_NARROW_FAST:
+		case CEC_OP_AUD_RATE_NARROW_SLOW:
+			aud_rate_msg_interval_check(node, msg.rx_ts);
+			node->state.last_aud_rate_rx_ts = msg.rx_ts;
+			return;
+		default:
+			cec_msg_reply_feature_abort(&msg, CEC_OP_ABORT_INVALID_OP);
+			transmit(node, &msg);
+			break;
+		}
 		break;
 
 
@@ -911,8 +986,8 @@ static void processMsg(struct node *node, struct cec_msg &msg, unsigned me)
 static void poll_remote_devs(struct node *node, unsigned me)
 {
 	node->remote_la_mask = 0;
-	for (unsigned i = 0; i < 15; i++)
-		node->remote_phys_addr[i] = CEC_PHYS_ADDR_INVALID;
+	for (unsigned short & i : node->remote_phys_addr)
+		i = CEC_PHYS_ADDR_INVALID;
 
 	if (!(node->caps & CEC_CAP_TRANSMIT))
 		return;
@@ -934,6 +1009,73 @@ static void poll_remote_devs(struct node *node, unsigned me)
 	}
 }
 
+static void update_programmed_timers(struct node *node)
+{
+	std::set<struct Timer>::iterator it = programmed_timers.begin();
+	/* Use the current minute because timers do not have second precision. */
+	time_t current_minute = time(nullptr) / 60;
+	time_t timer_start_minute = it->start_time / 60;
+	time_t timer_end_minute = (it->start_time + it->duration) / 60;
+
+	/* Start the timed recording only if the deck is not already recording. */
+	if (timer_start_minute == current_minute && !node->state.one_touch_record_on) {
+		node->state.one_touch_record_on = true;
+		node->state.recording_controlled_by_timer = true;
+		print_timers(node);
+	}
+
+	/* Delete an overlapped timer. Recording will be at best incomplete. */
+	if (timer_start_minute < current_minute &&
+	    (!node->state.recording_controlled_by_timer || !node->state.one_touch_record_on)) {
+		programmed_timers.erase(*it);
+		if (show_info)
+			printf("Deleted overlapped timer.\n");
+		print_timers(node);
+	}
+
+	if (timer_end_minute != current_minute || !node->state.recording_controlled_by_timer)
+		return;
+
+	/* Delete finished timers. */
+	node->state.one_touch_record_on = false;
+	node->state.recording_controlled_by_timer = false;
+	node->state.media_space_available -= it->duration; /* 1 MB per second */
+	/*
+	 * TODO: We are only ever decreasing the amount of space available,
+	 * there is no heuristic that reclaims the space.
+	 */
+
+	if (it->recording_seq) {
+		struct tm *last_start_time = localtime(&(it->start_time));
+		int next_wday = (last_start_time->tm_wday + 1) % 7;
+		int days_to_move_ahead = 1;
+
+		while ((it->recording_seq & (1 << next_wday)) == 0) {
+			days_to_move_ahead++;
+			next_wday = (next_wday + 1) % 7;
+		}
+		struct Timer next_timer = {};
+		next_timer = *it;
+		last_start_time->tm_mday += days_to_move_ahead;
+		last_start_time->tm_isdst = -1;
+		next_timer.start_time = mktime(last_start_time);
+		programmed_timers.insert(next_timer);
+	}
+	programmed_timers.erase(*it);
+	if (show_info)
+		printf("Deleted finished timer.\n");
+	print_timers(node);
+	/*
+	 * If the finished timer was recording, and standby was received during recording,
+	 * enter standby when the recording stops unless recording device is the active source.
+	 */
+	if (node->state.record_received_standby) {
+		if (node->phys_addr != node->state.active_source_pa)
+			enter_standby(node);
+		node->state.record_received_standby = false;
+	}
+}
+
 void testProcessing(struct node *node, bool wallclock)
 {
 	struct cec_log_addrs laddrs;
@@ -943,17 +1085,20 @@ void testProcessing(struct node *node, bool wallclock)
 	__u32 mode = CEC_MODE_INITIATOR | CEC_MODE_FOLLOWER;
 	unsigned me;
 	unsigned last_poll_la = 15;
+	__u8 last_pwr_state = current_power_state(node);
+	time_t last_pwr_status_toggle = time(nullptr);
 
 	clock_gettime(CLOCK_MONOTONIC, &start_monotonic);
-	gettimeofday(&start_timeofday, NULL);
+	gettimeofday(&start_timeofday, nullptr);
 
 	doioctl(node, CEC_S_MODE, &mode);
 	doioctl(node, CEC_ADAP_G_LOG_ADDRS, &laddrs);
 	me = laddrs.log_addr[0];
+	__u8 type = laddrs.log_addr_type[0];
 
 	poll_remote_devs(node, me);
 
-	while (1) {
+	while (true) {
 		int res;
 		struct timeval timeval = {};
 
@@ -963,7 +1108,7 @@ void testProcessing(struct node *node, bool wallclock)
 		FD_ZERO(&ex_fds);
 		FD_SET(fd, &rd_fds);
 		FD_SET(fd, &ex_fds);
-		res = select(fd + 1, &rd_fds, NULL, &ex_fds, &timeval);
+		res = select(fd + 1, &rd_fds, nullptr, &ex_fds, &timeval);
 		if (res < 0)
 			break;
 		if (FD_ISSET(fd, &ex_fds)) {
@@ -1006,34 +1151,60 @@ void testProcessing(struct node *node, bool wallclock)
 			__u8 to = cec_msg_destination(&msg);
 			__u8 opcode = cec_msg_opcode(&msg);
 
+			if (node->ignore_la[from])
+				continue;
+			if (node->ignore_opcode[msg.msg[1]] & (1 << from))
+				continue;
+
 			if (from != CEC_LOG_ADDR_UNREGISTERED &&
 			    la_info[from].feature_aborted[opcode].ts &&
 			    ts_to_ms(get_ts() - la_info[from].feature_aborted[opcode].ts) < 200) {
 				warn("Received message %s from LA %d (%s) less than 200 ms after\n",
-				     opcode2s(&msg).c_str(), from, la2s(from));
+				     opcode2s(&msg).c_str(), from, cec_la2s(from));
 				warn("replying Feature Abort (not [Unrecognized Opcode]) to the same message.\n");
 			}
 			if (from != CEC_LOG_ADDR_UNREGISTERED && !la_info[from].ts)
-				dev_info("Logical address %d (%s) discovered.\n", from, la2s(from));
+				dev_info("Logical address %d (%s) discovered.\n", from, cec_la2s(from));
 			if (show_msgs) {
 				printf("    %s to %s (%d to %d): ",
-				       la2s(from), to == 0xf ? "all" : la2s(to), from, to);
-				log_msg(&msg);
+				       cec_la2s(from), to == 0xf ? "all" : cec_la2s(to), from, to);
+				cec_log_msg(&msg);
 				if (show_info)
 					printf("\tSequence: %u Rx Timestamp: %s\n",
 					       msg.sequence, ts2s(msg.rx_ts, wallclock).c_str());
 			}
 			if (node->adap_la_mask)
-				processMsg(node, msg, me);
+				processMsg(node, msg, me, type);
+		}
+
+		__u8 pwr_state = current_power_state(node);
+		if (node->cec_version >= CEC_OP_CEC_VERSION_2_0 &&
+		    last_pwr_state != pwr_state &&
+		    (time_to_stable > 2 || pwr_state < CEC_OP_POWER_STATUS_TO_ON)) {
+			struct cec_msg msg;
+
+			cec_msg_init(&msg, me, CEC_LOG_ADDR_BROADCAST);
+			cec_msg_report_power_status(&msg, pwr_state);
+			transmit(node, &msg);
+			last_pwr_state = pwr_state;
+		}
+
+		if (node->state.toggle_power_status && cec_has_tv(1 << me) &&
+		    (time(nullptr) - last_pwr_status_toggle > node->state.toggle_power_status)) {
+			last_pwr_status_toggle = time(nullptr);
+			if (pwr_state & 1) // standby or to-standby
+				exit_standby(node);
+			else
+				enter_standby(node);
 		}
 
 		__u64 ts_now = get_ts();
 		unsigned poll_la = ts_to_s(ts_now) % 16;
 
-		if (poll_la != me && 
+		if (poll_la != me &&
 		    poll_la != last_poll_la && poll_la < 15 && la_info[poll_la].ts &&
 		    ts_to_ms(ts_now - la_info[poll_la].ts) > POLL_PERIOD) {
-			struct cec_msg msg = {};
+			struct cec_msg msg;
 
 			cec_msg_init(&msg, me, poll_la);
 			transmit(node, &msg);
@@ -1056,6 +1227,17 @@ void testProcessing(struct node *node, bool wallclock)
 				node->state.rc_state = NOPRESS;
 			}
 		}
+
+		if (node->has_aud_rate)
+			aud_rate_msg_interval_check(node, ts_now);
+
+		if (node->state.deck_skip_start && ts_now - node->state.deck_skip_start > MAX_DECK_SKIP_NS) {
+			node->state.deck_skip_start = 0;
+			update_deck_state(node, me, CEC_OP_DECK_INFO_PLAY);
+		}
+
+		if (!programmed_timers.empty())
+			update_programmed_timers(node);
 	}
 	mode = CEC_MODE_INITIATOR;
 	doioctl(node, CEC_S_MODE, &mode);
