@@ -79,6 +79,8 @@
 const char *argp_program_version = PROGRAM_NAME " version " V4L_UTILS_VERSION;
 const char *argp_program_bug_address = "Mauro Carvalho Chehab <mchehab@kernel.org>";
 
+#define CMD_SIZE	80
+
 static const char doc[] = N_(
 	"\nA DVB remote daemon using API version 5\n");
 
@@ -214,7 +216,7 @@ static nfds_t numfds = 0;
 static char output_charset[256] = "utf-8";
 static char default_charset[256] = "iso-8859-1";
 
-void stack_dump()
+static void stack_dump(void)
 {
 #ifdef HAVE_BACKTRACE
 	int i, nptrs = 0;
@@ -731,7 +733,7 @@ static void *read_data(void *privdata)
 			if (read_ret < 0)
 				dbg("#%d: read error: %d on %p", fd, read_ret, open_dev);
 			else
-				dbg("#%d: read %d bytes (count %d)", fd, read_ret, count);
+				dbg("#%d: read %ul bytes (count %d)", fd, read_ret, count);
 		}
 
 		/* Initialize to the start of the buffer */
@@ -843,9 +845,8 @@ static int dev_open(uint32_t seq, char *cmd, int fd, char *buf, ssize_t size)
 	if (!p) {
 		local_perror("tsearch");
 		uid = 0;
-	}
-	if (*p != desc) {
-		err("uid %d was already opened!");
+	} else if (*p != desc) {
+		err("uid %d was already opened!", uid);
 	}
 
 	pthread_mutex_unlock(&msg_mutex);
@@ -1341,7 +1342,7 @@ static void *start_server(void *fd_pointer)
 {
 	const struct method_types *method;
 	int fd = *(int *)fd_pointer, ret, flag = 1;
-	char buf[REMOTE_BUF_SIZE + 8], cmd[80], *p;
+	char buf[REMOTE_BUF_SIZE + 8], cmd[CMD_SIZE], *p;
 	ssize_t size;
 	uint32_t seq;
 	int bufsize;
@@ -1351,11 +1352,15 @@ static void *start_server(void *fd_pointer)
 
 	/* Set a large buffer for read() to work better */
 	bufsize = REMOTE_BUF_SIZE;
-	setsockopt(fd, SOL_SOCKET, SO_SNDBUF,
-		   (void *)&bufsize, (int)sizeof(bufsize));
+	if (setsockopt(fd, SOL_SOCKET, SO_SNDBUF,
+		       (void *)&bufsize, (int)sizeof(bufsize))) {
+		dbg("Failed to set a large buffer size");
+	};
 
 	/* Disable Naggle algorithm, as we want errors to be sent ASAP */
-	setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (char *) &flag, sizeof(int));
+	if (setsockopt(fd, IPPROTO_TCP, TCP_NODELAY, (char *) &flag, sizeof(int))) {
+		dbg("Failed to avoid TCP delays");
+	};
 
 	/* Command dispatcher */
 	do {
@@ -1371,7 +1376,7 @@ static void *start_server(void *fd_pointer)
 		ret = scan_data(buf, size, "%i%s",  &seq, cmd);
 		if (ret < 0) {
 			if (verbose)
-				dbg("message too short: %d", size);
+				dbg("message too short: %ld", size);
 			send_data(fd, "%i%s%i%s", 0, "log", LOG_ERR,
 				  "msg too short");
 			continue;
@@ -1475,7 +1480,7 @@ int main(int argc, char *argv[])
 	}
 
 	/* Initialize listen address struct */
-	bzero((char *) &serv_addr, sizeof(serv_addr));
+	memset((char *) &serv_addr, 0, sizeof(serv_addr));
 	serv_addr.sin_family = AF_INET;
 	serv_addr.sin_addr.s_addr = INADDR_ANY;
 	serv_addr.sin_port = htons(port);
