@@ -501,6 +501,11 @@ static int check_frontend(struct arguments *args,
 
 		status = 0;
 		rc = dvb_fe_retrieve_stats(parms, DTV_STATUS, &status);
+		if (rc) {
+			ERROR("dvb_fe_retrieve_stats failed");
+			usleep(1000000);
+			continue;
+		}
 		if (!args->silent)
 			print_frontend_stats(stderr, args, parms);
 		if (status & FE_HAS_LOCK)
@@ -557,6 +562,9 @@ static void copy_to_file(struct dvb_open_descriptor *in_fd, int out_fd,
 	long long int rc = 0LL;
 	struct timespec start, *elapsed;
 
+	/* Initialize start time, due to -EOVERFLOW with first == 1 */
+	clock_gettime(CLOCK_MONOTONIC, &start);
+
 	while (timeout_flag == 0) {
 		r = dvb_dev_read(in_fd, buf, sizeof(buf));
 		if (r < 0) {
@@ -580,6 +588,8 @@ static void copy_to_file(struct dvb_open_descriptor *in_fd, int out_fd,
 		 * ensure that a program record will have the start amount of
 		 * time specified by the user is to restart the timeout alarm
 		 * here, after the first succeded read.
+		 *
+		 * So, let's reset the start time here.
 		 */
 		if (first) {
 			if (timeout > 0)
@@ -887,9 +897,6 @@ int do_traffic_monitor(struct arguments *args, struct dvb_device *dvb,
 			if (pid < 0x1fff && h->adaptation_field_control & 1) {
 				int discontinued = 0;
 
-				if (err_cnt[pid] < 0)
-					err_cnt[pid] = 0;
-
 				if (h->adaptation_field_control & 2) {
 					if (h->adaptation_field_length >= 1) {
 						discontinued = h->discontinued;
@@ -1028,6 +1035,7 @@ int main(int argc, char **argv)
 	int vpid = -1, apid = -1, sid = -1;
 	int pmtpid = 0;
 	struct dvb_open_descriptor *pat_fd = NULL, *pmt_fd = NULL;
+	struct dvb_open_descriptor *sdt_fd = NULL;
 	struct dvb_open_descriptor *sid_fd = NULL, *dvr_fd = NULL;
 	struct dvb_open_descriptor *audio_fd = NULL, *video_fd = NULL;
 	int file_fd = -1;
@@ -1228,6 +1236,19 @@ int main(int argc, char **argv)
 				args.dvr ? DMX_OUT_TS_TAP : DMX_OUT_DECODER,
 				args.dvr ? 64 * 1024 : 0) < 0)
 			goto err;
+
+		/*
+		 * SDT may also be needed in order to play some streams
+		 */
+		sdt_fd = dvb_dev_open(dvb, args.demux_dev, O_RDWR);
+		if (!sdt_fd) {
+			ERROR("opening sdt demux failed");
+			goto err;
+		}
+		if (dvb_dev_dmx_set_pesfilter(sdt_fd, 0x0011, DMX_PES_OTHER,
+				args.dvr ? DMX_OUT_TS_TAP : DMX_OUT_DECODER,
+				args.dvr ? 64 * 1024 : 0) < 0)
+			goto err;
 	}
 
 	if (args.all_pids++) {
@@ -1281,6 +1302,12 @@ int main(int argc, char **argv)
 			goto err;
 	}
 
+	if (((vpid >= 0 && vpid != 0x2000) || apid) && !args.rec_psi) {
+		printf(_("PMT record is disabled.\n"
+		         "Please notice that some streams can only be decoded with PMT data.\n"
+			 "Use '-p' option to also record PMT.\n"));
+	}
+
 	set_signals(&args);
 
 	if (!check_frontend(&args, parms)) {
@@ -1391,7 +1418,7 @@ err:
 	if (args.search)
 		free(args.search);
 	if (args.server)
-		free(args.search);
+		free(args.server);
 	if (args.dvr_pipe != default_dvr_pipe)
 		free(args.dvr_pipe);
 
