@@ -59,6 +59,9 @@
 /* See drivers/media/rc/lirc_dev.c line 22 */
 #define LIRCBUF_SIZE 1024
 #define IR_DEFAULT_TIMEOUT 125000
+#define UNSET UINT32_MAX
+/* Maximum number of columns per line */
+#define LINE_SIZE 8192
 
 const char *argp_program_version = "IR ctl version " V4L_UTILS_VERSION;
 const char *argp_program_bug_address = "Sean Young <sean@mess.org>";
@@ -158,17 +161,18 @@ static const char doc[] = N_(
 	"Note that most lirc setting have global state, i.e. the device will remain\n"
 	"in this state until set otherwise.");
 
-static int strtoint(const char *p, const char *unit)
+static bool strtoint(const char *p, const char *unit, unsigned *ret)
 {
 	char *end;
 	long arg = strtol(p, &end, 10);
 	if (end == NULL || (end[0] != 0 && strcasecmp(end, unit) != 0))
-		return 0;
+		return false;
 
-	if (arg <= 0 || arg >= 0xffffff)
-		return 0;
+	if (arg < 0 || arg >= 0xffffff)
+		return false;
 
-	return arg;
+	*ret = arg;
+	return true;
 }
 
 static bool strtoscancode(const char *p, unsigned *ret)
@@ -209,7 +213,7 @@ static struct send *read_file_pulse_space(struct arguments *args, const char *fn
 {
 	bool expect_pulse = true;
 	int lineno = 0, lastspace = 0;
-	char line[1024];
+	char line[LINE_SIZE];
 	int len = 0;
 	static const char whitespace[] = " \n\r\t";
 	struct send *f;
@@ -221,7 +225,7 @@ static struct send *read_file_pulse_space(struct arguments *args, const char *fn
 	}
 	f->is_scancode = false;
 	f->is_keycode = false;
-	f->carrier = 0;
+	f->carrier = UNSET;
 	f->fname = fname;
 
 	while (fgets(line, sizeof(line), input)) {
@@ -300,8 +304,8 @@ static struct send *read_file_pulse_space(struct arguments *args, const char *fn
 			continue;
 		}
 
-		int arg = strtoint(p, "");
-		if (arg == 0) {
+		unsigned int arg;
+		if (!strtoint(p, "", &arg)) {
 			fprintf(stderr, _("warning: %s:%d: invalid argument '%s'\n"), fname, lineno, p);
 			continue;
 		}
@@ -312,7 +316,11 @@ static struct send *read_file_pulse_space(struct arguments *args, const char *fn
 			continue;
 		}
 
-		if (strcmp(keyword, "space") == 0) {
+		if (!strcmp(keyword, "space") || !strcmp(keyword, "timeout")) {
+			if (arg == 0) {
+				fprintf(stderr, _("warning: %s:%d: invalid argument to space '%d'\n"), fname, lineno, arg);
+				continue;
+			}
 			if (expect_pulse) {
 				if (len == 0) {
 					fprintf(stderr, _("warning: %s:%d: leading space ignored\n"),
@@ -325,14 +333,18 @@ static struct send *read_file_pulse_space(struct arguments *args, const char *fn
 			}
 			lastspace = lineno;
 			expect_pulse = true;
-		} else if (strcmp(keyword, "pulse") == 0) {
+		} else if (!strcmp(keyword, "pulse")) {
+			if (arg == 0) {
+				fprintf(stderr, _("warning: %s:%d: invalid argument to pulse '%d'\n"), fname, lineno, arg);
+				continue;
+			}
 			if (!expect_pulse)
 				f->buf[len-1] += arg;
 			else
 				f->buf[len++] = arg;
 			expect_pulse = false;
-		} else if (strcmp(keyword, "carrier") == 0) {
-			if (f->carrier && f->carrier != arg) {
+		} else if (!strcmp(keyword, "carrier")) {
+			if (f->carrier != UNSET && f->carrier != arg) {
 				fprintf(stderr, _("warning: %s:%d: carrier already specified\n"), fname, lineno);
 			} else {
 				f->carrier = arg;
@@ -370,7 +382,7 @@ static struct send *read_file_pulse_space(struct arguments *args, const char *fn
 static struct send *read_file_raw(struct arguments *args, const char *fname, FILE *input)
 {
 	int lineno = 0, lastspace = 0;
-	char line[1024];
+	char line[LINE_SIZE];
 	int len = 0;
 	static const char whitespace[] = " \n\r\t,";
 	struct send *f;
@@ -383,7 +395,7 @@ static struct send *read_file_raw(struct arguments *args, const char *fname, FIL
 	}
 	f->is_scancode = false;
 	f->is_keycode = false;
-	f->carrier = 0;
+	f->carrier = UNSET;
 	f->fname = fname;
 
 	while (fgets(line, sizeof(line), input)) {
@@ -464,7 +476,7 @@ static struct send *read_file_raw(struct arguments *args, const char *fname, FIL
 static struct send *read_file(struct arguments *args, const char *fname)
 {
 	FILE *input = fopen(fname, "r");
-	char line[1024];
+	char line[LINE_SIZE];
 
 	if (!input) {
 		fprintf(stderr, _("%s: could not open: %m\n"), fname);
@@ -619,8 +631,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 		break;
 	}
 	case 't':
-		arguments->timeout = strtoint(arg, "µs");
-		if (arguments->timeout == 0)
+		if (!strtoint(arg, "µs", &arguments->timeout))
 			argp_error(state, _("cannot parse timeout `%s'"), arg);
 		break;
 
@@ -629,8 +640,7 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 		arguments->device = arg;
 		break;
 	case 'c':
-		arguments->carrier = strtoint(arg, "Hz");
-		if (arguments->carrier == 0)
+		if (!strtoint(arg, "Hz", &arguments->carrier))
 			argp_error(state, _("cannot parse carrier `%s'"), arg);
 		break;
 	case 'e':
@@ -639,13 +649,12 @@ static error_t parse_opt(int k, char *arg, struct argp_state *state)
 			argp_error(state, _("cannot parse emitters `%s'"), arg);
 		break;
 	case 'g':
-		arguments->gap = strtoint(arg, "");
-		if (arguments->gap == 0)
+		if (!strtoint(arg, "", &arguments->gap) || arguments->gap == 0)
 			argp_error(state, _("cannot parse gap `%s'"), arg);
 		break;
 	case 'D':
-		arguments->duty = strtoint(arg, "%");
-		if (arguments->duty == 0 || arguments->duty >= 100)
+		if (!strtoint(arg, "%", &arguments->duty) ||
+		     arguments->duty == 0 || arguments->duty >= 100)
 			argp_error(state, _("invalid duty cycle `%s'"), arg);
 		break;
 	case 's':
@@ -862,7 +871,7 @@ static int lirc_options(struct arguments *args, int fd, unsigned features)
 	const char *dev = args->device;
 	int rc;
 
-	if (args->timeout) {
+	if (args->timeout != UNSET) {
 		if (features & LIRC_CAN_SET_REC_TIMEOUT) {
 			rc = ioctl(fd, LIRC_SET_REC_TIMEOUT, &args->timeout);
 			if (rc)
@@ -910,7 +919,7 @@ static int lirc_options(struct arguments *args, int fd, unsigned features)
 			fprintf(stderr, _("%s: device does not support setting receiver carrier range\n"), dev);
 	}
 
-	if (args->carrier)
+	if (args->carrier != UNSET)
 		lirc_set_send_carrier(fd, dev, features, args->carrier);
 
 	if (args->duty) {
@@ -966,13 +975,20 @@ static void lirc_features(struct arguments *args, int fd, unsigned features)
 			printf(_(" - Use wideband receiver\n"));
 		if (features & LIRC_CAN_MEASURE_CARRIER)
 			printf(_(" - Can measure carrier\n"));
-		if (features & LIRC_CAN_SET_REC_TIMEOUT) {
-			unsigned min_timeout, max_timeout, timeout;
 
-			// This ioctl is only supported from kernel 4.18 onwards
-			int rc = ioctl(fd, LIRC_GET_REC_TIMEOUT, &timeout);
-			if (rc == 0)
+		// This ioctl is only supported from kernel 4.18 onwards
+		unsigned timeout;
+		int rc = ioctl(fd, LIRC_GET_REC_TIMEOUT, &timeout);
+		if (rc == 0) {
+			if (timeout == 0)
+				printf(_(" - Receiving timeout not set\n"));
+			else
 				printf(_(" - Receiving timeout %u microseconds\n"), timeout);
+		}
+
+		if (features & LIRC_CAN_SET_REC_TIMEOUT) {
+			unsigned min_timeout, max_timeout;
+
 			rc = ioctl(fd, LIRC_GET_MIN_TIMEOUT, &min_timeout);
 			if (rc) {
 				fprintf(stderr, _("warning: %s: device supports setting receiving timeout but LIRC_GET_MIN_TIMEOUT returns: %m\n"), dev);
@@ -1071,9 +1087,11 @@ static int lirc_send(struct arguments *args, int fd, unsigned features, struct s
 		f->carrier = protocol_carrier(proto);
 	}
 
-	if (args->carrier && f->carrier)
-		fprintf(stderr, _("warning: %s: carrier specified but overwritten on command line\n"), f->fname);
-	else if (f->carrier && args->carrier == 0)
+	if (args->carrier != UNSET) {
+		if (f->carrier != UNSET)
+			fprintf(stderr, _("warning: carrier specified but overwritten on command line\n"));
+		lirc_set_send_carrier(fd, dev, features, args->carrier);
+	} else if (f->carrier != UNSET)
 		lirc_set_send_carrier(fd, dev, features, f->carrier);
 
 	size_t size = f->len * sizeof(unsigned);
@@ -1156,9 +1174,8 @@ int lirc_receive(struct arguments *args, int fd, unsigned features)
 			// is meaningless, maybe fix the kernel?
 			if (leading_space && msg == LIRC_MODE2_SPACE)
 				continue;
-			else
-				leading_space = false;
 
+			leading_space = false;
 			if (args->oneshot &&
 				(msg == LIRC_MODE2_TIMEOUT ||
 				(msg == LIRC_MODE2_SPACE && val > 19000))) {
@@ -1185,10 +1202,9 @@ int lirc_receive(struct arguments *args, int fd, unsigned features)
 			} else {
 				switch (msg) {
 				case LIRC_MODE2_TIMEOUT:
+					fprintf(out, "-%u\n", val);
 					if (carrier)
-						fprintf(out, "# carrier %uHz, timeout %u\n", carrier, val);
-					else
-						fprintf(out, "# timeout %u\n", val);
+						fprintf(out, " # carrier %uHz, timeout %u\n", carrier, val);
 					leading_space = true;
 					carrier = 0;
 					break;
@@ -1218,7 +1234,11 @@ err:
 
 int main(int argc, char *argv[])
 {
-	struct arguments args = { .gap = IR_DEFAULT_TIMEOUT };
+	struct arguments args = {
+		.gap = IR_DEFAULT_TIMEOUT,
+		.carrier = UNSET,
+		.timeout = UNSET,
+	};
 
 #ifdef ENABLE_NLS
         setlocale (LC_ALL, "");
