@@ -385,7 +385,7 @@ static int setparams(snd_pcm_t *phandle, snd_pcm_t *chandle,
 
     /* Note we use twice as much periods for the playback buffer, since we
        will get a period size near the requested time and we don't want it to
-       end up smaller then the capture buffer as then we could end up blocking
+       end up smaller than the capture buffer as then we could end up blocking
        on writing to it. Note we will configure the playback dev to start
        playing as soon as it has 2 capture periods worth of data, so this
        won't influence latency */
@@ -433,7 +433,7 @@ static snd_pcm_sframes_t readbuf(snd_pcm_t *handle, char *buf, long len)
     snd_pcm_uframes_t frames;
     snd_pcm_htimestamp(handle, &frames, &timestamp);
     r = snd_pcm_readi(handle, buf, len);
-    if (r < 0 && r != -EAGAIN) {
+    if (r < 0 && !(r == -EAGAIN || r == -ENODEV)) {
 	r = snd_pcm_recover(handle, r, 0);
 	if (r < 0)
 	    fprintf(error_fp, "alsa: overrun recover error: %s\n", snd_strerror(r));
