@@ -72,6 +72,7 @@ static void usage()
 	       "\n"
 	       "  -W, --width=<width>      set width\n"
 	       "  -H, --height=<height>    set frame (not field!) height\n"
+	       "  -A, --padding=<bytes>    set additional horizontal padding (after width)\n"
 	       "  --fps=<fps>              set frames-per-second (default is 30)\n"
 	       "\n"
 	       "  The following option is only valid when reading from a file:\n"
@@ -318,7 +319,7 @@ __u32 read_u32(int fd)
 	n = read(fd, &v, sizeof(v));
 	if (n != sizeof(v)) {
 		fprintf(stderr, "could not read __u32\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	return ntohl(v);
 }
@@ -335,7 +336,7 @@ int initSocket(int port, cv4l_fmt &fmt, v4l2_fract &pixelaspect)
 		listen_fd = socket(AF_INET, SOCK_STREAM, 0);
 		if (listen_fd < 0) {
 			fprintf(stderr, "could not opening socket\n");
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		setsockopt(listen_fd, SOL_SOCKET, SO_REUSEADDR, &val, sizeof(int));
 
@@ -344,7 +345,7 @@ int initSocket(int port, cv4l_fmt &fmt, v4l2_fract &pixelaspect)
 		serv_addr.sin_port = htons(port);
 		if (bind(listen_fd, (struct sockaddr *)&serv_addr, sizeof(serv_addr)) < 0) {
 			fprintf(stderr, "could not bind: %s\n", strerror(errno));
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 	}
 	listen(listen_fd, 1);
@@ -352,17 +353,17 @@ int initSocket(int port, cv4l_fmt &fmt, v4l2_fract &pixelaspect)
 	sock_fd = accept(listen_fd, (struct sockaddr *)&cli_addr, &clilen);
 	if (sock_fd < 0) {
 		fprintf(stderr, "could not accept\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	if (read_u32(sock_fd) != V4L_STREAM_ID) {
 		fprintf(stderr, "unknown protocol ID\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	__u32 version = read_u32(sock_fd);
 
 	if (!version || version > V4L_STREAM_VERSION) {
 		fprintf(stderr, "unknown protocol version %u\n", version);
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	for (;;) {
 		__u32 packet = read_u32(sock_fd);
@@ -370,7 +371,7 @@ int initSocket(int port, cv4l_fmt &fmt, v4l2_fract &pixelaspect)
 
 		if (packet == V4L_STREAM_PACKET_END) {
 			fprintf(stderr, "END packet read\n");
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 
 		if (packet == V4L_STREAM_PACKET_FMT_VIDEO)
@@ -384,7 +385,7 @@ int initSocket(int port, cv4l_fmt &fmt, v4l2_fract &pixelaspect)
 			n = read(sock_fd, buf, rdsize);
 			if (n < 0) {
 				fprintf(stderr, "error reading %d bytes\n", sz);
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 			sz -= n;
 		}
@@ -395,7 +396,7 @@ int initSocket(int port, cv4l_fmt &fmt, v4l2_fract &pixelaspect)
 
 	if (sz != V4L_STREAM_PACKET_FMT_VIDEO_SIZE_FMT) {
 		fprintf(stderr, "unsupported FMT_VIDEO size\n");
-		exit(1);
+		std::exit(EXIT_FAILURE);
 	}
 	fmt.s_num_planes(read_u32(sock_fd));
 	fmt.s_pixelformat(read_u32(sock_fd));
@@ -415,7 +416,7 @@ int initSocket(int port, cv4l_fmt &fmt, v4l2_fract &pixelaspect)
 
 		if (sz != V4L_STREAM_PACKET_FMT_VIDEO_SIZE_FMT_PLANE) {
 			fprintf(stderr, "unsupported FMT_VIDEO plane size\n");
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		fmt.s_sizeimage(read_u32(sock_fd), i);
 		fmt.s_bytesperline(read_u32(sock_fd), i);
@@ -446,6 +447,7 @@ int main(int argc, char **argv)
 	__u32 overridePixelFormat = 0;
 	__u32 overrideWidth = 0;
 	__u32 overrideHeight = 0;
+	__u32 overrideHorPadding = 0;
 	__u32 overrideField = 0xffffffff;
 	__u32 overrideColorspace = 0xffffffff;
 	__u32 overrideYCbCrEnc = 0xffffffff;
@@ -506,7 +508,7 @@ int main(int argc, char **argv)
 				test = 2;
 			if (!pattern)
 				pattern = TPG_PAT_CSC_COLORBAR;
-		} else if (isOptArg(args[i], "--pixel-format", "-P")) {
+		} else if (isOptArg(args[i], "--pixelformat", "-P")) {
 			if (!processOption(args, i, s))
 				return 0;
 			overridePixelFormat = parse_pixel_format(s);
@@ -516,6 +518,9 @@ int main(int argc, char **argv)
 		} else if (isOptArg(args[i], "--height", "-H")) {
 			if (!processOption(args, i, overrideHeight))
 				return 0;
+		} else if (isOptArg(args[i], "--padding", "-A")) {
+			if (!processOption(args, i, overrideHorPadding))
+				return 0;
 		} else if (isOptArg(args[i], "--field", "-F")) {
 			if (!processOption(args, i, s))
 				return 0;
@@ -670,14 +675,27 @@ int main(int argc, char **argv)
 		video_device = getDeviceName("/dev/video", video_device);
 		if (fd.open(video_device.toUtf8().data(), true) < 0) {
 			perror((QString("could not open ") + video_device).toUtf8().data());
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		if (!fd.has_vid_cap()) {
 			fprintf(stderr, "%s is not a video capture device\n", video_device.toUtf8().data());
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		fd.g_fmt(fmt);
 
+		if (!overridePixelFormat) {
+			bool found = false;
+
+			for (unsigned i = 0; formats[i]; i++) {
+				if (fmt.g_pixelformat() == formats[i]) {
+					found = true;
+					break;
+				}
+			}
+			if (!found)
+				overridePixelFormat = V4L2_PIX_FMT_RGB24;
+		}
+
 		if (overridePixelFormat) {
 			fmt.s_pixelformat(overridePixelFormat);
 			fd.s_fmt(fmt);
@@ -736,6 +754,7 @@ int main(int argc, char **argv)
 	}
 	win.setOverrideWidth(overrideWidth);
 	win.setOverrideHeight(overrideHeight);
+	win.setOverrideHorPadding(overrideHorPadding);
 	win.setFps(fps);
 	win.setFormat(format);
 	win.setReportTimings(report_timings);
@@ -768,7 +787,7 @@ int main(int argc, char **argv)
 			fcc2s(fmt.g_pixelformat()).c_str(),
 			pixfmt2s(fmt.g_pixelformat()).c_str());
 		if (mode != AppModeSocket)
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		sock_fd = initSocket(port, fmt, pixelaspect);
 	}
 	win.setPixelAspect(pixelaspect);
@@ -794,7 +813,7 @@ int main(int argc, char **argv)
 		q.queue_all(&fd);
 		win.setQueue(&q);
 		if (fd.streamon())
-			exit(1);
+			std::exit(EXIT_FAILURE);
 	} else {
 		struct tpg_data *tpg = win.getTPG();
 
