@@ -18,21 +18,16 @@
     Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <inttypes.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
-#include <sys/ioctl.h>
+#include <map>
+#include <set>
 #include <vector>
-#include "v4l2-compliance.h"
 
-#define V4L2_CTRL_CLASS_VIVID 0x00f00000
+#include <sys/types.h>
+#include <sys/wait.h>
+#include <sys/epoll.h>
+
+#include "compiler.h"
+#include "v4l2-compliance.h"
 
 static int checkQCtrl(struct node *node, struct test_query_ext_ctrl &qctrl)
 {
@@ -45,7 +40,7 @@ static int checkQCtrl(struct node *node, struct test_query_ext_ctrl &qctrl)
 	qctrl.menu_mask = 0;
 	if (check_string(qctrl.name, sizeof(qctrl.name)))
 		return fail("invalid name\n");
-	info("checking v4l2_queryctrl of control '%s' (0x%08x)\n", qctrl.name, qctrl.id);
+	info("checking v4l2_query_ext_ctrl of control '%s' (0x%08x)\n", qctrl.name, qctrl.id);
 	if (qctrl.id & V4L2_CTRL_FLAG_NEXT_CTRL)
 		return fail("V4L2_CTRL_FLAG_NEXT_CTRL not cleared\n");
 	if (check_0(qctrl.reserved, sizeof(qctrl.reserved)))
@@ -61,28 +56,26 @@ static int checkQCtrl(struct node *node, struct test_query_ext_ctrl &qctrl)
 	case V4L2_CTRL_TYPE_BOOLEAN:
 		if (qctrl.maximum > 1)
 			return fail("invalid boolean max value\n");
-		/* fall through */
+		fallthrough;
 	case V4L2_CTRL_TYPE_MENU:
 	case V4L2_CTRL_TYPE_INTEGER_MENU:
 		if (qctrl.step != 1)
 			return fail("invalid step value %lld\n", qctrl.step);
 		if (qctrl.minimum < 0)
 			return fail("min < 0\n");
-		/* fall through */
+		fallthrough;
 	case V4L2_CTRL_TYPE_INTEGER:
 	case V4L2_CTRL_TYPE_INTEGER64:
 		if (qctrl.default_value < qctrl.minimum ||
 		    qctrl.default_value > qctrl.maximum)
 			return fail("def < min || def > max\n");
-		/* fall through */
+		fallthrough;
 	case V4L2_CTRL_TYPE_STRING:
 		if (qctrl.minimum > qctrl.maximum)
 			return fail("min > max\n");
 		if (qctrl.step == 0)
 			return fail("step == 0\n");
-		if (qctrl.step < 0)
-			return fail("step < 0\n");
-		if ((unsigned)qctrl.step > (unsigned)(qctrl.maximum - qctrl.minimum) &&
+		if (static_cast<unsigned>(qctrl.step) > static_cast<unsigned>(qctrl.maximum - qctrl.minimum) &&
 		    qctrl.maximum != qctrl.minimum)
 			return fail("step > max - min\n");
 		if ((qctrl.maximum - qctrl.minimum) % qctrl.step) {
@@ -118,7 +111,7 @@ static int checkQCtrl(struct node *node, struct test_query_ext_ctrl &qctrl)
 	case V4L2_CTRL_TYPE_BUTTON:
 		if ((fl & rw_mask) != V4L2_CTRL_FLAG_WRITE_ONLY)
 			return fail("button control not write only\n");
-		/* fall through */
+		fallthrough;
 	case V4L2_CTRL_TYPE_BOOLEAN:
 	case V4L2_CTRL_TYPE_MENU:
 	case V4L2_CTRL_TYPE_INTEGER_MENU:
@@ -126,7 +119,7 @@ static int checkQCtrl(struct node *node, struct test_query_ext_ctrl &qctrl)
 	case V4L2_CTRL_TYPE_BITMASK:
 		if (fl & V4L2_CTRL_FLAG_SLIDER)
 			return fail("slider makes only sense for integer controls\n");
-		/* fall through */
+		fallthrough;
 	case V4L2_CTRL_TYPE_INTEGER64:
 	case V4L2_CTRL_TYPE_INTEGER:
 		if ((fl & rw_mask) == rw_mask)
@@ -162,7 +155,7 @@ static int checkQCtrl(struct node *node, struct test_query_ext_ctrl &qctrl)
 			continue;
 		if (i < qctrl.minimum || i > qctrl.maximum)
 			return fail("can get menu for out-of-range index\n");
-		if (qmenu.index != (__u32)i || qmenu.id != qctrl.id)
+		if (qmenu.index != static_cast<__u32>(i) || qmenu.id != qctrl.id)
 			return fail("id or index changed\n");
 		if (qctrl.type == V4L2_CTRL_TYPE_MENU &&
 		    check_ustring(qmenu.name, sizeof(qmenu.name)))
@@ -238,11 +231,11 @@ int testQueryExtControls(struct node *node)
 		}
 		if (V4L2_CTRL_DRIVER_PRIV(id)) {
 			node->priv_controls++;
-			if (qctrl.type >= V4L2_CTRL_COMPOUND_TYPES)
+			if (qctrl.type >= V4L2_CTRL_COMPOUND_TYPES || qctrl.nr_of_dims)
 				node->priv_compound_controls++;
 		} else {
 			node->std_controls++;
-			if (qctrl.type >= V4L2_CTRL_COMPOUND_TYPES)
+			if (qctrl.type >= V4L2_CTRL_COMPOUND_TYPES || qctrl.nr_of_dims)
 				node->std_compound_controls++;
 		}
 		node->controls[qctrl.id] = qctrl;
@@ -254,6 +247,42 @@ int testQueryExtControls(struct node *node)
 	if (which && !class_count)
 		return fail("no controls in class %08x\n", which);
 
+	id = 0;
+	unsigned regular_ctrls = 0;
+	for (;;) {
+		memset(&qctrl, 0xff, sizeof(qctrl));
+		qctrl.id = id | V4L2_CTRL_FLAG_NEXT_CTRL;
+		ret = doioctl(node, VIDIOC_QUERY_EXT_CTRL, &qctrl);
+		if (ret)
+			break;
+		id = qctrl.id;
+		if (qctrl.type >= V4L2_CTRL_COMPOUND_TYPES || qctrl.nr_of_dims)
+			return fail("V4L2_CTRL_FLAG_NEXT_CTRL enumerates compound controls!\n");
+		regular_ctrls++;
+	}
+	unsigned num_compound_ctrls = node->std_compound_controls + node->priv_compound_controls;
+	unsigned num_regular_ctrls = node->std_controls + node->priv_controls - num_compound_ctrls;
+	if (regular_ctrls != num_regular_ctrls)
+		return fail("expected to find %d standard controls, got %d instead\n",
+			    num_regular_ctrls, regular_ctrls);
+
+	id = 0;
+	unsigned compound_ctrls = 0;
+	for (;;) {
+		memset(&qctrl, 0xff, sizeof(qctrl));
+		qctrl.id = id | V4L2_CTRL_FLAG_NEXT_COMPOUND;
+		ret = doioctl(node, VIDIOC_QUERY_EXT_CTRL, &qctrl);
+		if (ret)
+			break;
+		id = qctrl.id;
+		if (qctrl.type < V4L2_CTRL_COMPOUND_TYPES && !qctrl.nr_of_dims)
+			return fail("V4L2_CTRL_FLAG_NEXT_COMPOUND enumerates regular controls!\n");
+		compound_ctrls++;
+	}
+	if (compound_ctrls != num_compound_ctrls)
+		return fail("expected to find %d compound controls, got %d instead\n",
+			    num_compound_ctrls, compound_ctrls);
+
 	for (id = V4L2_CID_BASE; id < V4L2_CID_LASTP1; id++) {
 		memset(&qctrl, 0xff, sizeof(qctrl));
 		qctrl.id = id;
@@ -301,9 +330,13 @@ int testQueryControls(struct node *node)
 {
 	struct v4l2_queryctrl qctrl;
 	unsigned controls = 0;
+	unsigned compound_controls = 0;
 	__u32 id = 0;
 	int ret;
 
+	unsigned num_compound_ctrls = node->std_compound_controls + node->priv_compound_controls;
+	unsigned num_regular_ctrls = node->std_controls + node->priv_controls - num_compound_ctrls;
+
 	for (;;) {
 		memset(&qctrl, 0xff, sizeof(qctrl));
 		qctrl.id = id | V4L2_CTRL_FLAG_NEXT_CTRL;
@@ -316,10 +349,24 @@ int testQueryControls(struct node *node)
 			break;
 		id = qctrl.id;
 		fail_on_test(node->controls.find(qctrl.id) == node->controls.end());
+		fail_on_test(qctrl.step < 0);
 		controls++;
 	}
-	fail_on_test(node->controls.size() !=
-		     controls + node->std_compound_controls + node->priv_compound_controls);
+	fail_on_test(controls != num_regular_ctrls);
+
+	id = 0;
+	for (;;) {
+		memset(&qctrl, 0xff, sizeof(qctrl));
+		qctrl.id = id | V4L2_CTRL_FLAG_NEXT_COMPOUND;
+		ret = doioctl(node, VIDIOC_QUERYCTRL, &qctrl);
+		if (ret)
+			break;
+		id = qctrl.id;
+		fail_on_test(node->controls.find(qctrl.id) == node->controls.end());
+		fail_on_test(qctrl.step || qctrl.minimum || qctrl.maximum || qctrl.default_value);
+		compound_controls++;
+	}
+	fail_on_test(compound_controls != num_compound_ctrls);
 
 	for (id = V4L2_CID_BASE; id < V4L2_CID_LASTP1; id++) {
 		memset(&qctrl, 0xff, sizeof(qctrl));
@@ -332,6 +379,7 @@ int testQueryControls(struct node *node)
 		if (qctrl.id != id)
 			return fail("qctrl.id (%08x) != id (%08x)\n",
 					qctrl.id, id);
+		fail_on_test(qctrl.step < 0);
 	}
 
 	for (id = V4L2_CID_PRIVATE_BASE; ; id++) {
@@ -345,11 +393,12 @@ int testQueryControls(struct node *node)
 		if (qctrl.id != id)
 			return fail("qctrl.id (%08x) != id (%08x)\n",
 					qctrl.id, id);
+		fail_on_test(qctrl.step < 0);
 	}
 	return 0;
 }
 
-static int checkSimpleCtrl(struct v4l2_control &ctrl, struct test_query_ext_ctrl &qctrl)
+static int checkSimpleCtrl(const struct v4l2_control &ctrl, const struct test_query_ext_ctrl &qctrl)
 {
 	if (ctrl.id != qctrl.id)
 		return fail("control id mismatch\n");
@@ -369,7 +418,7 @@ static int checkSimpleCtrl(struct v4l2_control &ctrl, struct test_query_ext_ctrl
 		}
 		break;
 	case V4L2_CTRL_TYPE_BITMASK:
-		if ((__u32)ctrl.value & ~qctrl.maximum)
+		if (static_cast<__u32>(ctrl.value) & ~qctrl.maximum)
 			return fail("returned control value out of range\n");
 		break;
 	case V4L2_CTRL_TYPE_BUTTON:
@@ -406,12 +455,7 @@ int testSimpleControls(struct node *node)
 				return fail("g_ctrl allowed for unsupported type\n");
 			ctrl.id = qctrl.id;
 			ctrl.value = 0;
-			// This call will crash on kernels <= 2.6.37 for control classes due to
-			// a bug in v4l2-ctrls.c. So skip this on those kernels.
-			if (kernel_version < 38 && qctrl.type == V4L2_CTRL_TYPE_CTRL_CLASS)
-				ret = EACCES;
-			else
-				ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
+			ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
 			if (ret != EINVAL &&
 			    !((qctrl.flags & V4L2_CTRL_FLAG_READ_ONLY) && ret == EACCES))
 				return fail("s_ctrl allowed for unsupported type\n");
@@ -429,7 +473,7 @@ int testSimpleControls(struct node *node)
 			return fail("g_ctrl returned an error (%d)\n", ret);
 		else if (checkSimpleCtrl(ctrl, qctrl))
 			return fail("invalid control %08x\n", qctrl.id);
-		
+
 		// Try to set the current value (or the default value for write only controls)
 		ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
 		if (qctrl.flags & V4L2_CTRL_FLAG_READ_ONLY) {
@@ -492,7 +536,7 @@ int testSimpleControls(struct node *node)
 			for (i = qctrl.minimum; i <= max; i++) {
 				bool valid = qctrl.menu_mask & (1ULL << i);
 
-				ctrl.id = qctrl.id; 
+				ctrl.id = qctrl.id;
 				ctrl.value = i;
 				ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
 				if (valid && ret)
@@ -504,7 +548,7 @@ int testSimpleControls(struct node *node)
 			}
 		} else {
 			// at least min, max and default values should work
-			ctrl.id = qctrl.id; 
+			ctrl.id = qctrl.id;
 			ctrl.value = qctrl.minimum;
 			ret = doioctl(node, VIDIOC_S_CTRL, &ctrl);
 			if (ret && ret != EIO && ret != EILSEQ)
@@ -533,7 +577,7 @@ int testSimpleControls(struct node *node)
 	return 0;
 }
 
-static int checkExtendedCtrl(struct v4l2_ext_control &ctrl, struct test_query_ext_ctrl &qctrl)
+static int checkExtendedCtrl(const struct v4l2_ext_control &ctrl, const struct test_query_ext_ctrl &qctrl)
 {
 	int len;
 
@@ -556,7 +600,7 @@ static int checkExtendedCtrl(struct v4l2_ext_control &ctrl, struct test_query_ex
 		}
 		break;
 	case V4L2_CTRL_TYPE_BITMASK:
-		if ((__u32)ctrl.value & ~qctrl.maximum)
+		if (static_cast<__u32>(ctrl.value) & ~qctrl.maximum)
 			return fail("returned control value out of range\n");
 		break;
 	case V4L2_CTRL_TYPE_BUTTON:
@@ -618,14 +662,11 @@ int testExtendedControls(struct node *node)
 
 		if (is_vivid && V4L2_CTRL_ID2WHICH(qctrl.id) == V4L2_CTRL_CLASS_VIVID)
 			continue;
-		// Skip V4L2_CID_MPEG_VIDEO_FWHT_PARAMS for now
-		if (qctrl.id == (V4L2_CID_MPEG_BASE + 292))
-			continue;
 
 		info("checking extended control '%s' (0x%08x)\n", qctrl.name, qctrl.id);
 		ctrl.id = qctrl.id;
 		ctrl.size = 0;
-		ctrl.ptr = NULL;
+		ctrl.ptr = nullptr;
 		ctrl.reserved2[0] = 0;
 		ctrls.count = 1;
 
@@ -664,7 +705,7 @@ int testExtendedControls(struct node *node)
 			if (checkExtendedCtrl(ctrl, qctrl))
 				return fail("invalid control %08x\n", qctrl.id);
 		}
-		
+
 		// Try the current value (or the default value for write only controls)
 		ret = doioctl(node, VIDIOC_TRY_EXT_CTRLS, &ctrls);
 		if (qctrl.flags & V4L2_CTRL_FLAG_READ_ONLY) {
@@ -675,7 +716,7 @@ int testExtendedControls(struct node *node)
 		} else if (ret) {
 			return fail("try_ext_ctrls returned an error (%d)\n", ret);
 		}
-		
+
 		// Try to set the current value (or the default value for write only controls)
 		ret = doioctl(node, VIDIOC_S_EXT_CTRLS, &ctrls);
 		if (qctrl.flags & V4L2_CTRL_FLAG_READ_ONLY) {
@@ -693,13 +734,14 @@ int testExtendedControls(struct node *node)
 			}
 			if (ret)
 				return fail("s_ext_ctrls returned an error (%d)\n", ret);
-		
+
 			if (checkExtendedCtrl(ctrl, qctrl))
 				return fail("s_ext_ctrls returned invalid control contents (%08x)\n", qctrl.id);
 		}
-		if (qctrl.type == V4L2_CTRL_TYPE_STRING)
+
+		if (qctrl.flags & V4L2_CTRL_FLAG_HAS_PAYLOAD)
 			delete [] ctrl.string;
-		ctrl.string = NULL;
+		ctrl.string = nullptr;
 	}
 
 	ctrls.which = 0;
@@ -733,9 +775,7 @@ int testExtendedControls(struct node *node)
 
 		if (qctrl.flags & (V4L2_CTRL_FLAG_READ_ONLY | V4L2_CTRL_FLAG_WRITE_ONLY))
 			continue;
-		// Skip V4L2_CID_MPEG_VIDEO_FWHT_PARAMS for now
-		if (qctrl.id == (V4L2_CID_MPEG_BASE + 292))
-			continue;
+
 		ctrl.id = qctrl.id;
 		ctrl.size = 0;
 		if (qctrl.flags & V4L2_CTRL_FLAG_HAS_PAYLOAD) {
@@ -749,6 +789,8 @@ int testExtendedControls(struct node *node)
 			multiple_classes = true;
 		total_vec.push_back(ctrl);
 	}
+	if (total_vec.empty())
+		return 0;
 
 	ctrls.count = total_vec.size();
 	ctrls.controls = &total_vec[0];
@@ -831,7 +873,7 @@ int testEvents(struct node *node)
 		//if (qctrl.type == V4L2_CTRL_TYPE_CTRL_CLASS)
 		FD_ZERO(&set);
 		FD_SET(node->g_fd(), &set);
-		ret = select(node->g_fd() + 1, NULL, NULL, &set, &timeout);
+		ret = select(node->g_fd() + 1, nullptr, nullptr, &set, &timeout);
 		if (ret == 0) {
 			if (qctrl.type != V4L2_CTRL_TYPE_CTRL_CLASS)
 				return fail("failed to find event for control '%s'\n", qctrl.name);
@@ -844,6 +886,7 @@ int testEvents(struct node *node)
 				return fail("couldn't get event for control '%s'\n", qctrl.name);
 			if (ev.type != V4L2_EVENT_CTRL || ev.id != qctrl.id)
 				return fail("dequeued wrong event\n");
+			fail_on_test(check_0(ev.reserved, sizeof(ev.reserved)));
 		}
 		ret = doioctl(node, VIDIOC_UNSUBSCRIBE_EVENT, &sub);
 		if (ret)
@@ -913,6 +956,89 @@ int testEvents(struct node *node)
 	return 0;
 }
 
+int testVividDisconnect(struct node *node)
+{
+	// Test that disconnecting a device will wake up any processes
+	// that are using select or poll.
+	//
+	// This can only be tested with the vivid driver that enabled
+	// the DISCONNECT control.
+
+	pid_t pid = fork();
+	if (pid == 0) {
+		struct timeval tv = { 5, 0 };
+		fd_set fds;
+
+		FD_ZERO(&fds);
+		FD_SET(node->g_fd(), &fds);
+		int res = select(node->g_fd() + 1, nullptr, nullptr, &fds, &tv);
+		// No POLLPRI seen
+		if (res != 1)
+			exit(1);
+		// POLLPRI seen, but didn't wake up
+		if (!tv.tv_sec)
+			exit(2);
+		v4l2_event ev = {};
+		// Woke up on POLLPRI, but VIDIOC_DQEVENT didn't return
+		// the ENODEV error.
+		if (doioctl(node, VIDIOC_DQEVENT, &ev) != ENODEV)
+			exit(3);
+		exit(0);
+	}
+	v4l2_control ctrl = { VIVID_CID_DISCONNECT, 0 };
+	sleep(1);
+	fail_on_test(doioctl(node, VIDIOC_S_CTRL, &ctrl));
+	int wstatus;
+	fail_on_test(waitpid(pid, &wstatus, 0) != pid);
+	fail_on_test(!WIFEXITED(wstatus));
+	if (WEXITSTATUS(wstatus))
+		return fail("select child exited with status %d\n", WEXITSTATUS(wstatus));
+
+	node->reopen();
+
+	pid = fork();
+	if (pid == 0) {
+		struct epoll_event ep_ev;
+		int epollfd = epoll_create1(0);
+
+		if (epollfd < 0)
+			exit(1);
+
+		ep_ev.events = 0;
+		if (epoll_ctl(epollfd, EPOLL_CTL_ADD, node->g_fd(), &ep_ev))
+			exit(2);
+
+		ep_ev.events = EPOLLPRI;
+		ep_ev.data.fd = node->g_fd();
+		if (epoll_ctl(epollfd, EPOLL_CTL_MOD, node->g_fd(), &ep_ev))
+			exit(3);
+		int ret = epoll_wait(epollfd, &ep_ev, 1, 5000);
+		if (ret == 0)
+			exit(4);
+		if (ret < 0)
+			exit(5);
+		if (ret != 1)
+			exit(6);
+		if (!(ep_ev.events & EPOLLPRI))
+			exit(7);
+		if (!(ep_ev.events & EPOLLERR))
+			exit(8);
+		v4l2_event ev = {};
+		// Woke up on POLLPRI, but VIDIOC_DQEVENT didn't return
+		// the ENODEV error.
+		if (doioctl(node, VIDIOC_DQEVENT, &ev) != ENODEV)
+			exit(9);
+		exit(0);
+	}
+	sleep(1);
+	fail_on_test(doioctl(node, VIDIOC_S_CTRL, &ctrl));
+	fail_on_test(waitpid(pid, &wstatus, 0) != pid);
+	fail_on_test(!WIFEXITED(wstatus));
+	if (WEXITSTATUS(wstatus))
+		return fail("epoll child exited with status %d\n", WEXITSTATUS(wstatus));
+	return 0;
+}
+
 int testJpegComp(struct node *node)
 {
 	struct v4l2_jpegcompression jc;
@@ -922,7 +1048,7 @@ int testJpegComp(struct node *node)
 		V4L2_JPEG_MARKER_DRI | V4L2_JPEG_MARKER_COM |
 		V4L2_JPEG_MARKER_APP;
 	int ret;
-	
+
 	memset(&jc, 0, sizeof(jc));
 	ret = doioctl(node, VIDIOC_G_JPEGCOMP, &jc);
 	if (ret != ENOTTY) {
@@ -930,9 +1056,9 @@ int testJpegComp(struct node *node)
 		if (ret)
 			return fail("VIDIOC_G_JPEGCOMP gave an error\n");
 		have_jpegcomp = true;
-		if (jc.COM_len < 0 || jc.COM_len > (int)sizeof(jc.COM_data))
+		if (jc.COM_len < 0 || jc.COM_len > static_cast<int>(sizeof(jc.COM_data)))
 			return fail("invalid COM_len value\n");
-		if (jc.APP_len < 0 || jc.APP_len > (int)sizeof(jc.APP_data))
+		if (jc.APP_len < 0 || jc.APP_len > static_cast<int>(sizeof(jc.APP_data)))
 			return fail("invalid APP_len value\n");
 		if (jc.quality < 0 || jc.quality > 100)
 			warn("weird quality value: %d\n", jc.quality);
