@@ -18,23 +18,16 @@
     Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <inttypes.h>
+#include <cassert>
+#include <map>
+#include <set>
+
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
-#include <sys/ioctl.h>
-#include <assert.h>
 
 #include "compiler.h"
 #include "v4l2-compliance.h"
 
-static const __u32 buftype2cap[] = {
+static constexpr __u32 buftype2cap[] = {
 	0,
 	V4L2_CAP_VIDEO_CAPTURE | V4L2_CAP_VIDEO_M2M,
 	V4L2_CAP_VIDEO_OUTPUT | V4L2_CAP_VIDEO_M2M,
@@ -71,6 +64,8 @@ static int testEnumFrameIntervals(struct node *node, __u32 pixfmt,
 		ret = doioctl(node, VIDIOC_ENUM_FRAMEINTERVALS, &frmival);
 		if (ret == ENOTTY)
 			return ret;
+		// M2M devices don't support this, except for stateful encoders
+		fail_on_test(node->is_m2m && !(node->codec_mask & STATEFUL_ENCODER));
 		if (f == 0 && ret == EINVAL) {
 			if (type == V4L2_FRMSIZE_TYPE_DISCRETE)
 				warn("found framesize %dx%d, but no frame intervals\n", w, h);
@@ -260,20 +255,37 @@ static int testEnumFormatsType(struct node *node, unsigned type)
 			return fail("fmtdesc.description not set\n");
 		if (!fmtdesc.pixelformat)
 			return fail("fmtdesc.pixelformat not set\n");
+
+		// Check that the driver does not overwrites the kernel pixelformat description
+		std::string descr = pixfmt2s(fmtdesc.pixelformat);
+		if (strcmp(descr.c_str(), (const char *)fmtdesc.description) &&
+		    memcmp(descr.c_str(), "Unknown", 7))
+			return fail("fmtdesc.description mismatch: was '%s', expected '%s'\n",
+				    fmtdesc.description, descr.c_str());
+
 		if (node->g_direct() && (fmtdesc.flags & V4L2_FMT_FLAG_EMULATED))
 			return fail("drivers must never set the emulated flag\n");
 		if (fmtdesc.flags & ~(V4L2_FMT_FLAG_COMPRESSED | V4L2_FMT_FLAG_EMULATED |
 				      V4L2_FMT_FLAG_CONTINUOUS_BYTESTREAM |
-				      V4L2_FMT_FLAG_DYN_RESOLUTION))
+				      V4L2_FMT_FLAG_DYN_RESOLUTION |
+				      V4L2_FMT_FLAG_ENC_CAP_FRAME_INTERVAL |
+				      V4L2_FMT_FLAG_CSC_COLORSPACE |
+				      V4L2_FMT_FLAG_CSC_YCBCR_ENC | V4L2_FMT_FLAG_CSC_HSV_ENC |
+				      V4L2_FMT_FLAG_CSC_QUANTIZATION | V4L2_FMT_FLAG_CSC_XFER_FUNC))
 			return fail("unknown flag %08x returned\n", fmtdesc.flags);
 		if (!(fmtdesc.flags & V4L2_FMT_FLAG_COMPRESSED))
 			fail_on_test(fmtdesc.flags & (V4L2_FMT_FLAG_CONTINUOUS_BYTESTREAM |
-						      V4L2_FMT_FLAG_DYN_RESOLUTION));
+						      V4L2_FMT_FLAG_DYN_RESOLUTION |
+						      V4L2_FMT_FLAG_ENC_CAP_FRAME_INTERVAL));
 		ret = testEnumFrameSizes(node, fmtdesc.pixelformat);
 		if (ret)
 			fail_on_test(node->codec_mask & STATEFUL_ENCODER);
 		if (ret && ret != ENOTTY)
 			return ret;
+		if (fmtdesc.flags & V4L2_FMT_FLAG_ENC_CAP_FRAME_INTERVAL) {
+			fail_on_test(!(node->codec_mask & STATEFUL_ENCODER));
+			node->has_enc_cap_frame_interval = true;
+		}
 		f++;
 		if (type == V4L2_BUF_TYPE_PRIVATE)
 			continue;
@@ -410,7 +422,7 @@ static void createInvalidFmt(struct v4l2_format &fmt, struct v4l2_clip &clip, un
 		clip.next = (struct  v4l2_clip *)0x0eadbeef;
 		fmt.fmt.win.clipcount = 1;
 		fmt.fmt.win.clips = &clip;
-		fmt.fmt.win.bitmap = NULL;
+		fmt.fmt.win.bitmap = nullptr;
 	}
 }
 
@@ -521,11 +533,11 @@ static int testFormatsType(struct node *node, int ret,  unsigned type, struct v4
 		if (ret)
 			return fail("sliced.reserved not zeroed\n");
 		fail_on_test(sliced.service_lines[0][0] || sliced.service_lines[1][0]);
-		for (int f = 0; f < 2; f++) {
-			for (int i = 0; i < 24; i++) {
-				if (sliced.service_lines[f][i])
+		for (const auto &service_line : sliced.service_lines) {
+			for (unsigned short i : service_line) {
+				if (i)
 					cnt++;
-				service_set |= sliced.service_lines[f][i];
+				service_set |= i;
 			}
 		}
 		fail_on_test(sliced.io_size < sizeof(struct v4l2_sliced_vbi_data) * cnt);
@@ -800,6 +812,32 @@ int testTryFormats(struct node *node)
 	return node->valid_buftypes ? result : ENOTTY;
 }
 
+static int testJPEGColorspace(const cv4l_fmt &fmt_raw, const cv4l_fmt &fmt_jpeg,
+			      bool is_decoder)
+{
+	fail_on_test(fmt_raw.g_colorspace() != V4L2_COLORSPACE_SRGB);
+	fail_on_test(fmt_raw.g_xfer_func() &&
+		     fmt_raw.g_xfer_func() != V4L2_XFER_FUNC_SRGB);
+	fail_on_test(fmt_raw.g_ycbcr_enc() && is_decoder &&
+		     fmt_raw.g_ycbcr_enc() != V4L2_YCBCR_ENC_601);
+	if (fmt_jpeg.g_colorspace() == V4L2_COLORSPACE_JPEG) {
+		/*
+		 * If the V4L2_COLORSPACE_JPEG shorthand is used, then the
+		 * other values shall be 0.
+		 */
+		fail_on_test(fmt_jpeg.g_xfer_func() ||
+			     fmt_jpeg.g_ycbcr_enc() ||
+			     fmt_jpeg.g_quantization());
+		return 0;
+	}
+	/* V4L2_COLORSPACE_JPEG is shorthand for these values: */
+	fail_on_test(fmt_jpeg.g_colorspace() != V4L2_COLORSPACE_SRGB);
+	fail_on_test(fmt_jpeg.g_xfer_func() != V4L2_XFER_FUNC_SRGB);
+	fail_on_test(fmt_jpeg.g_ycbcr_enc() != V4L2_YCBCR_ENC_601);
+	fail_on_test(fmt_jpeg.g_quantization() != V4L2_QUANTIZATION_FULL_RANGE);
+	return 0;
+}
+
 static int testM2MFormats(struct node *node)
 {
 	cv4l_fmt fmt_out;
@@ -810,7 +848,17 @@ static int testM2MFormats(struct node *node)
 	__u32 col, ycbcr_enc, quant, xfer_func;
 
 	fail_on_test(node->g_fmt(fmt_out, out_type));
-	node->g_fmt(fmt_cap, cap_type);
+	fail_on_test(node->g_fmt(fmt_cap, cap_type));
+
+	/*
+	 * JPEG codec have fixed colorspace, so these tests
+	 * are different compared to other m2m devices.
+	 */
+	if (node->codec_mask & JPEG_DECODER)
+		return testJPEGColorspace(fmt_cap, fmt_out, true);
+	if (node->codec_mask & JPEG_ENCODER)
+		return testJPEGColorspace(fmt_out, fmt_cap, false);
+
 	fail_on_test(fmt_cap.g_colorspace() != fmt_out.g_colorspace());
 	fail_on_test(fmt_cap.g_ycbcr_enc() != fmt_out.g_ycbcr_enc());
 	fail_on_test(fmt_cap.g_quantization() != fmt_out.g_quantization());
@@ -1137,7 +1185,7 @@ int testSetFormats(struct node *node)
 
 	// Test if setting a format on one fh will set the format for all
 	// filehandles.
-	if (node->node2 == NULL)
+	if (node->node2 == nullptr)
 		return 0;
 
 	// m2m devices are special in that the format is often per-filehandle.
@@ -1199,9 +1247,9 @@ static int testSlicedVBICapType(struct node *node, unsigned type)
 	fail_on_test(cap.type != type);
 	fail_on_test(!sliced_type || !(node->g_caps() & buftype2cap[type]));
 
-	for (int f = 0; f < 2; f++)
-		for (int i = 0; i < 24; i++)
-			service_set |= cap.service_lines[f][i];
+	for (const auto &service_line : cap.service_lines)
+		for (unsigned short i : service_line)
+			service_set |= i;
 	fail_on_test(cap.service_set != service_set);
 	fail_on_test(cap.service_lines[0][0] || cap.service_lines[1][0]);
 	return 0;
@@ -1222,6 +1270,7 @@ int testSlicedVBICap(struct node *node)
 
 static int testParmStruct(struct node *node, struct v4l2_streamparm &parm)
 {
+	bool is_stateful_enc = node->codec_mask & STATEFUL_ENCODER;
 	struct v4l2_captureparm *cap = &parm.parm.capture;
 	struct v4l2_outputparm *out = &parm.parm.output;
 	int ret;
@@ -1239,6 +1288,7 @@ static int testParmStruct(struct node *node, struct v4l2_streamparm &parm)
 			fail_on_test(!cap->readbuffers);
 		fail_on_test(cap->capability & ~V4L2_CAP_TIMEPERFRAME);
 		fail_on_test(node->has_frmintervals && !cap->capability);
+		fail_on_test(is_stateful_enc && !cap->capability);
 		fail_on_test(cap->capturemode & ~V4L2_MODE_HIGHQUALITY);
 		if (cap->capturemode & V4L2_MODE_HIGHQUALITY)
 			warn("V4L2_MODE_HIGHQUALITY is poorly defined\n");
@@ -1257,6 +1307,7 @@ static int testParmStruct(struct node *node, struct v4l2_streamparm &parm)
 		else if (node->g_caps() & V4L2_CAP_STREAMING)
 			fail_on_test(!out->writebuffers);
 		fail_on_test(out->capability & ~V4L2_CAP_TIMEPERFRAME);
+		fail_on_test(is_stateful_enc && !out->capability);
 		fail_on_test(out->outputmode);
 		fail_on_test(out->extendedmode);
 		if (out->capability & V4L2_CAP_TIMEPERFRAME)
@@ -1271,6 +1322,7 @@ static int testParmStruct(struct node *node, struct v4l2_streamparm &parm)
 static int testParmType(struct node *node, unsigned type)
 {
 	struct v4l2_streamparm parm;
+	bool is_stateful_enc = node->codec_mask & STATEFUL_ENCODER;
 	int ret;
 
 	memset(&parm, 0, sizeof(parm));
@@ -1288,10 +1340,10 @@ static int testParmType(struct node *node, unsigned type)
 	case V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE:
 	case V4L2_BUF_TYPE_VIDEO_OUTPUT:
 	case V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE:
-		if (type && (node->g_caps() & buftype2cap[type]))
+		if (node->g_caps() & buftype2cap[type]) {
 			fail_on_test(ret && node->has_frmintervals);
-		if (ret)
-			break;
+			fail_on_test(ret && node->has_enc_cap_frame_interval);
+		}
 		break;
 	default:
 		fail_on_test(ret == 0);
@@ -1300,8 +1352,15 @@ static int testParmType(struct node *node, unsigned type)
 		fail_on_test(!doioctl(node, VIDIOC_S_PARM, &parm));
 		break;
 	}
+	if (ret == ENOTTY) {
+		memset(&parm, 0, sizeof(parm));
+		parm.type = type;
+		fail_on_test(doioctl(node, VIDIOC_S_PARM, &parm) != ENOTTY);
+	}
 	if (ret == ENOTTY)
 		return ret;
+	// M2M devices don't support this, except for stateful encoders
+	fail_on_test(node->is_m2m && !is_stateful_enc);
 	if (ret == EINVAL)
 		return ENOTTY;
 	if (ret)
@@ -1327,11 +1386,29 @@ static int testParmType(struct node *node, unsigned type)
 		cap = parm.parm.output.capability;
 	else
 		cap = parm.parm.capture.capability;
-	fail_on_test(ret && node->has_frmintervals);
-	if (!ret && (cap & V4L2_CAP_TIMEPERFRAME) && !node->has_frmintervals)
+
+	if (is_stateful_enc) {
+		fail_on_test(ret && node->has_enc_cap_frame_interval);
+		if (V4L2_TYPE_IS_OUTPUT(type))
+			fail_on_test(ret);
+	} else {
+		fail_on_test(ret && node->has_frmintervals);
+	}
+
+	/*
+	 * Stateful encoders can support S_PARM without ENUM_FRAMEINTERVALS.
+	 * being present. In that case the limits of the chosen codec apply.
+	 */
+	if (!ret && (cap & V4L2_CAP_TIMEPERFRAME) && !node->has_frmintervals && !is_stateful_enc)
 		warn("S_PARM is supported for buftype %d, but not for ENUM_FRAMEINTERVALS\n", type);
 	if (ret == ENOTTY)
 		return 0;
+	/*
+	 * S_PARM(CAPTURE) is optional for stateful encoders, so EINVAL is a
+	 * valid error code in that case.
+	 */
+	if (is_stateful_enc && V4L2_TYPE_IS_CAPTURE(type) && ret == -EINVAL)
+		return 0;
 	if (ret)
 		return fail("got error %d when setting parms for buftype %d\n", ret, type);
 	fail_on_test(parm.type != type);
@@ -1415,7 +1492,7 @@ static int testBasicSelection(struct node *node, unsigned type, unsigned target)
 	fail_on_test(ret);
 	fail_on_test(check_0(sel.reserved, sizeof(sel.reserved)));
 
-	// selection is not supported (for now) if there is more than one
+	// set selection is not supported (for now) if there is more than one
 	// discrete frame size.
 	if (type == V4L2_BUF_TYPE_VIDEO_CAPTURE)
 		v4l_format_init(&fmt, node->is_planar ?
@@ -1426,8 +1503,13 @@ static int testBasicSelection(struct node *node, unsigned type, unsigned target)
 			V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE :
 			V4L2_BUF_TYPE_VIDEO_OUTPUT);
 	fail_on_test(doioctl(node, VIDIOC_G_FMT, &fmt));
+
+	sel.type = 0xff;
+	bool have_s_sel = doioctl(node, VIDIOC_S_SELECTION, &sel) != ENOTTY;
+
 	__u32 pixfmt = v4l_format_g_pixelformat(&fmt);
-	if (node->frmsizes_count.find(pixfmt) != node->frmsizes_count.end())
+	if (node->frmsizes_count.find(pixfmt) != node->frmsizes_count.end() &&
+	    have_s_sel)
 		fail_on_test(node->frmsizes_count[pixfmt] > 1);
 
 	// Check handling of invalid type.
@@ -1502,9 +1584,17 @@ static int testLegacyCrop(struct node *node)
 					 V4L2_SEL_TGT_COMPOSE_DEFAULT;
 	/*
 	 * If either CROPCAP or G_CROP works, then G_SELECTION should
-	 * work as well.
+	 * work as well. This was typically the case for drivers that
+	 * only implemented G_CROP and not G_SELECTION. However,
+	 * support for G_CROP was removed from the kernel and all drivers
+	 * now support G_SELECTION, so this should no longer happen.
+	 *
 	 * If neither CROPCAP nor G_CROP work, then G_SELECTION shouldn't
-	 * work either.
+	 * work either. If this fails, then this is almost certainly because
+	 * G_SELECTION doesn't support V4L2_SEL_TGT_CROP_DEFAULT and/or
+	 * V4L2_SEL_TGT_CROP_BOUNDS. CROPCAP requires both to be present.
+	 * For output devices this is of course V4L2_SEL_TGT_COMPOSE_DEFAULT
+	 * and/or V4L2_SEL_TGT_COMPOSE_BOUNDS.
 	 */
 	if (!doioctl(node, VIDIOC_CROPCAP, &cap)) {
 		fail_on_test(doioctl(node, VIDIOC_G_SELECTION, &sel));
@@ -1550,8 +1640,12 @@ int testCropping(struct node *node)
 	fail_on_test(testLegacyCrop(node));
 	if (node->can_capture && node->is_video)
 		ret_cap = testBasicSelection(node, V4L2_BUF_TYPE_VIDEO_CAPTURE, V4L2_SEL_TGT_CROP);
+	if (ret_cap && ret_cap != ENOTTY)
+		return ret_cap;
 	if (node->can_output && node->is_video)
 		ret_out = testBasicSelection(node, V4L2_BUF_TYPE_VIDEO_OUTPUT, V4L2_SEL_TGT_CROP);
+	if (ret_out && ret_out != ENOTTY)
+		return ret_out;
 	if ((!node->can_capture && !node->can_output) || !node->is_video) {
 		struct v4l2_selection sel = {
 			V4L2_BUF_TYPE_VIDEO_CAPTURE,
@@ -1857,7 +1951,7 @@ int testScaling(struct node *node)
 			V4L2_BUF_TYPE_VIDEO_CAPTURE_MPLANE :
 			V4L2_BUF_TYPE_VIDEO_CAPTURE);
 		fail_on_test(doioctl(node, VIDIOC_G_FMT, &fmt));
-		testBasicScaling(node, fmt);
+		fail_on_test(testBasicScaling(node, fmt));
 		fail_on_test(doioctl(node, VIDIOC_S_FMT, &fmt));
 	}
 	if (node->can_output) {
@@ -1865,7 +1959,7 @@ int testScaling(struct node *node)
 			V4L2_BUF_TYPE_VIDEO_OUTPUT_MPLANE :
 			V4L2_BUF_TYPE_VIDEO_OUTPUT);
 		fail_on_test(doioctl(node, VIDIOC_G_FMT, &fmt));
-		testBasicScaling(node, fmt);
+		fail_on_test(testBasicScaling(node, fmt));
 		fail_on_test(doioctl(node, VIDIOC_S_FMT, &fmt));
 	}
 	return node->can_scale ? 0 : ENOTTY;
