@@ -18,17 +18,11 @@
     Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <inttypes.h>
+#include <map>
+#include <set>
+
 #include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
-#include <sys/ioctl.h>
+
 #include "v4l2-compliance.h"
 
 static int checkStd(struct node *node, bool has_std, v4l2_std_id mask, bool is_input)
@@ -199,13 +193,13 @@ static int checkTimings(struct node *node, bool has_timings, bool is_input)
 		fail_on_test(doioctl(node, VIDIOC_S_DV_TIMINGS, &enumtimings.timings));
 		fail_on_test(check_0(enumtimings.timings.bt.reserved,
 				     sizeof(enumtimings.timings.bt.reserved)));
-		
+
 		struct v4l2_dv_timings g_timings;
 		fail_on_test(doioctl(node, VIDIOC_G_DV_TIMINGS, &g_timings));
 		fail_on_test(g_timings.bt.width != enumtimings.timings.bt.width);
 		fail_on_test(g_timings.bt.height != enumtimings.timings.bt.height);
 
-		if (node->is_vbi)
+		if (node->is_vbi || node->is_meta)
 			continue;
 		fmt.type = type;
 		fail_on_test(doioctl(node, VIDIOC_G_FMT, &fmt));
@@ -511,7 +505,7 @@ static int checkEdid(struct node *node, unsigned pad, bool is_input)
 	edid.pad = pad;
 	edid.start_block = 0;
 	edid.blocks = 0;
-	edid.edid = (__u8 *)0xdeadbeef;
+	edid.edid = (__u8 *)0x0eadbeef;
 	ret = doioctl(node, VIDIOC_G_EDID, &edid);
 	if (ret == ENOTTY) {
 		memset(&edid, 0, sizeof(edid));
@@ -583,12 +577,12 @@ static int checkEdid(struct node *node, unsigned pad, bool is_input)
 		fail_on_test(edid.blocks == 0 || edid.blocks >= 256);
 		fail_on_test(edid.pad != pad);
 	}
-	edid.blocks = blocks;
+	edid.blocks = 1;
 	edid.pad = pad;
-	edid.edid = NULL;
+	edid.edid = nullptr;
 	ret = doioctl(node, VIDIOC_S_EDID, &edid);
 	fail_on_test(ret != EFAULT);
-	edid.edid = (__u8 *)0xdeadbeef;
+	edid.edid = (__u8 *)0x0eadbeef;
 	ret = doioctl(node, VIDIOC_S_EDID, &edid);
 	fail_on_test(ret != EFAULT);
 	return 0;
