@@ -18,22 +18,12 @@
     Foundation, Inc., 51 Franklin Street, Suite 500, Boston, MA  02110-1335  USA
  */
 
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <inttypes.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/sysmacros.h>
-#include <fcntl.h>
-#include <dirent.h>
-#include <ctype.h>
-#include <errno.h>
-#include <sys/ioctl.h>
-#include <assert.h>
+#include <map>
 #include <set>
-#include <fstream>
+
+#include <dirent.h>
+#include <sys/sysmacros.h>
+#include <sys/types.h>
 
 #include "v4l2-compliance.h"
 
@@ -42,6 +32,7 @@ int testMediaDeviceInfo(struct node *node)
 	struct media_device_info mdinfo;
 
 	memset(&mdinfo, 0xff, sizeof(mdinfo));
+	fail_on_test(doioctl(node, MEDIA_IOC_DEVICE_INFO, nullptr) != EFAULT);
 	fail_on_test(doioctl(node, MEDIA_IOC_DEVICE_INFO, &mdinfo));
 	fail_on_test(check_0(mdinfo.reserved, sizeof(mdinfo.reserved)));
 	fail_on_test(check_string(mdinfo.driver, sizeof(mdinfo.driver)));
@@ -75,14 +66,14 @@ static int checkDevice(__u32 major, __u32 minor, bool iface, __u32 id)
 	fail_on_test(snprintf(dev_path, sizeof(dev_path), "/sys/dev/char/%d:%d",
 			      major, minor) == -1);
 	DIR *dp = opendir(dev_path);
-	if (dp == NULL)	
+	if (dp == nullptr)
 		return fail("couldn't find %s for %s %u\n",
 			    dev_path, iface ? "interface" : "entity", id);
 	closedir(dp);
 	return 0;
 }
 
-typedef std::set<__u32> id_set;
+using id_set = std::set<__u32>;
 
 static media_v2_topology topology;
 static media_v2_entity *v2_ents;
@@ -153,16 +144,16 @@ int testMediaTopology(struct node *node)
 	topology.ptr_links = 0;
 	v2_ents = new media_v2_entity[topology.num_entities];
 	memset(v2_ents, 0xff, topology.num_entities * sizeof(*v2_ents));
-	topology.ptr_entities = (__u64)v2_ents;
+	topology.ptr_entities = (uintptr_t)v2_ents;
 	v2_ifaces = new media_v2_interface[topology.num_interfaces];
 	memset(v2_ifaces, 0xff, topology.num_interfaces * sizeof(*v2_ifaces));
-	topology.ptr_interfaces = (__u64)v2_ifaces;
+	topology.ptr_interfaces = (uintptr_t)v2_ifaces;
 	v2_pads = new media_v2_pad[topology.num_pads];
 	memset(v2_pads, 0xff, topology.num_pads * sizeof(*v2_pads));
-	topology.ptr_pads = (__u64)v2_pads;
+	topology.ptr_pads = (uintptr_t)v2_pads;
 	v2_links = new media_v2_link[topology.num_links];
 	memset(v2_links, 0xff, topology.num_links * sizeof(*v2_links));
-	topology.ptr_links = (__u64)v2_links;
+	topology.ptr_links = (uintptr_t)v2_links;
 	fail_on_test(doioctl(node, MEDIA_IOC_G_TOPOLOGY, &topology));
 	fail_on_test(v2_ents != (media_v2_entity *)topology.ptr_entities);
 	fail_on_test(v2_ifaces != (media_v2_interface *)topology.ptr_interfaces);
@@ -255,7 +246,7 @@ int testMediaTopology(struct node *node)
 			fail_on_test(pad.index == ~0U);
 			fail_on_test(v2_entity_pad_idx_set.find((__u64)pad.entity_id << 32 | pad.index) !=
 				     v2_entity_pad_idx_set.end());
-			v2_entity_pad_idx_set.insert((__u64)pad.entity_id << 32 | pad.index);
+			v2_entity_pad_idx_set.insert(static_cast<__u64>(pad.entity_id) << 32 | pad.index);
 		} else {
 			fail_on_test(pad.index);
 		}
@@ -292,11 +283,15 @@ int testMediaTopology(struct node *node)
 			fail_on_test(v2_pads_set.find(link.source_id) == v2_pads_set.end());
 			fail_on_test(v2_pads_set.find(link.sink_id) == v2_pads_set.end());
 			fail_on_test(link.source_id == link.sink_id);
+			fail_on_test(!(v2_pad_map[link.source_id]->flags & MEDIA_PAD_FL_SOURCE));
+			fail_on_test(!(v2_pad_map[link.sink_id]->flags & MEDIA_PAD_FL_SINK));
 			num_data_links++;
 			if (show_info)
-				printf("\t\tLink: 0x%08x (%s -> %s, %s)\n", link.id,
+				printf("\t\tLink: 0x%08x (%s:%u -> %s:%u, %s)\n", link.id,
 				       v2_entity_map[v2_pad_map[link.source_id]->entity_id]->name,
+				       v2_pad_map[link.source_id]->index,
 				       v2_entity_map[v2_pad_map[link.sink_id]->entity_id]->name,
+				       v2_pad_map[link.sink_id]->index,
 				       mi_linkflags2s(link.flags).c_str());
 		}
 	}
@@ -317,7 +312,7 @@ static media_link_desc link_disabled;
 
 int testMediaEnum(struct node *node)
 {
-	typedef std::map<__u32, media_entity_desc> entity_map;
+	using entity_map = std::map<__u32, media_entity_desc>;
 	entity_map ent_map;
 	id_set has_default_set;
 	struct media_entity_desc ent;
@@ -388,9 +383,8 @@ int testMediaEnum(struct node *node)
 	}
 	fail_on_test(num_data_links != num_links);
 
-	for (entity_map::iterator iter = ent_map.begin();
-	     iter != ent_map.end(); ++iter) {
-		media_entity_desc &ent = iter->second;
+	for (auto & iter : ent_map) {
+		media_entity_desc &ent = iter.second;
 
 		memset(&links, 0, sizeof(links));
 		memset(&links.reserved, 0xff, sizeof(links.reserved));
@@ -402,10 +396,10 @@ int testMediaEnum(struct node *node)
 		fail_on_test(links.links);
 		links.pads = (struct media_pad_desc *)4;
 		fail_on_test(ent.pads && doioctl(node, MEDIA_IOC_ENUM_LINKS, &links) != EFAULT);
-		links.pads = NULL;
+		links.pads = nullptr;
 		links.links = (struct media_link_desc *)4;
 		fail_on_test(ent.links && doioctl(node, MEDIA_IOC_ENUM_LINKS, &links) != EFAULT);
-		links.links = NULL;
+		links.links = nullptr;
 		links.pads = new media_pad_desc[ent.pads];
 		memset(links.pads, 0xff, ent.pads * sizeof(*links.pads));
 		links.links = new media_link_desc[ent.links];
@@ -424,7 +418,7 @@ int testMediaEnum(struct node *node)
 				     (MEDIA_PAD_FL_SINK | MEDIA_PAD_FL_SOURCE));
 			if (node->topology &&
 			    MEDIA_V2_PAD_HAS_INDEX(node->media_version)) {
-				__u64 key = (__u64)ent.id << 32 | links.pads[i].index;
+				__u64 key = static_cast<__u64>(ent.id) << 32 | links.pads[i].index;
 
 				fail_on_test(v2_entity_pad_idx_set.find(key) ==
 					     v2_entity_pad_idx_set.end());
@@ -536,7 +530,7 @@ void walkTopology(struct node &node, struct node &expbuf_node,
 
 	media_v2_interface v2_ifaces[topology.num_interfaces];
 
-	topology.ptr_interfaces = (__u64)v2_ifaces;
+	topology.ptr_interfaces = (uintptr_t)v2_ifaces;
 	if (ioctl(node.g_fd(), MEDIA_IOC_G_TOPOLOGY, &topology))
 		return;
 
