@@ -1,19 +1,14 @@
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <inttypes.h>
-#include <getopt.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
-#include <sys/ioctl.h>
-#include <sys/sysmacros.h>
-#include <sys/time.h>
+#include <algorithm>
+#include <cctype>
+#include <cstring>
+#include <list>
+#include <map>
+#include <vector>
+
 #include <dirent.h>
-#include <math.h>
+#include <sys/stat.h>
+#include <sys/sysmacros.h>
+
 #include <linux/media.h>
 
 #include "v4l2-ctl.h"
@@ -22,40 +17,36 @@
 #include <sys/klog.h>
 #endif
 
-#include <list>
-#include <vector>
-#include <map>
-#include <algorithm>
-
 struct ctrl_subset {
 	unsigned offset[V4L2_CTRL_MAX_DIMS];
 	unsigned size[V4L2_CTRL_MAX_DIMS];
 };
 
-typedef std::map<unsigned, std::vector<struct v4l2_ext_control> > class2ctrls_map;
+using class2ctrls_map = std::map<unsigned int, std::vector<struct v4l2_ext_control> >;
 
-typedef std::map<std::string, struct v4l2_query_ext_ctrl> ctrl_qmap;
+using ctrl_qmap = std::map<std::string, struct v4l2_query_ext_ctrl>;
 static ctrl_qmap ctrl_str2q;
-typedef std::map<unsigned, std::string> ctrl_idmap;
+using ctrl_idmap = std::map<unsigned int, std::string>;
 static ctrl_idmap ctrl_id2str;
 
-typedef std::map<std::string, ctrl_subset> ctrl_subset_map;
+using ctrl_subset_map = std::map<std::string, ctrl_subset>;
 static ctrl_subset_map ctrl_subsets;
 
-typedef std::list<std::string> ctrl_get_list;
+using ctrl_get_list = std::list<std::string>;
 static ctrl_get_list get_ctrls;
 
-typedef std::map<std::string, std::string> ctrl_set_map;
-static ctrl_set_map set_ctrls;
+using ctrl_set_pair = std::pair<std::string, std::string>;
+using ctrl_set_list = std::list<ctrl_set_pair>;
+static ctrl_set_list set_ctrls;
 
-typedef std::vector<std::string> dev_vec;
-typedef std::map<std::string, std::string> dev_map;
+using dev_vec = std::vector<std::string>;
+using dev_map = std::map<std::string, std::string>;
 
 static enum v4l2_priority prio = V4L2_PRIORITY_UNSET;
 
 static bool have_query_ext_ctrl;
 
-void common_usage(void)
+void common_usage()
 {
 	printf("\nGeneral/Common options:\n"
 	       "  --all              display all information available\n"
@@ -121,6 +112,7 @@ void common_usage(void)
 	       "  --silent           only set the result code, do not print any messages\n"
 	       "  --sleep <secs>     sleep <secs>, call QUERYCAP and close the file handle\n"
 	       "  --verbose          turn on verbose ioctl status reporting\n"
+	       "  --version          show version information\n"
 	       );
 }
 
@@ -132,7 +124,7 @@ static const char *prefixes[] = {
 	"v4l-subdev",
 	"v4l-touch",
 	"media",
-	NULL
+	nullptr
 };
 
 static bool is_v4l_dev(const char *name)
@@ -152,7 +144,7 @@ static int calc_node_val(const char *s)
 {
 	int n = 0;
 
-	s = strrchr(s, '/') + 1;
+	s = std::strrchr(s, '/') + 1;
 
 	for (unsigned i = 0; prefixes[i]; i++) {
 		unsigned l = strlen(prefixes[i]);
@@ -182,7 +174,7 @@ static void list_media_devices(const std::string &media_bus_info)
 	std::map<dev_t, std::string> devices;
 
 	dp = opendir("/dev");
-	if (dp == NULL) {
+	if (dp == nullptr) {
 		perror ("Couldn't open the directory");
 		return;
 	}
@@ -224,8 +216,8 @@ static void list_media_devices(const std::string &media_bus_info)
 		return;
 	}
 
-	media_v2_interface *ifaces = new media_v2_interface[topology.num_interfaces];
-	topology.ptr_interfaces = (__u64)ifaces;
+	auto ifaces = new media_v2_interface[topology.num_interfaces];
+	topology.ptr_interfaces = (uintptr_t)ifaces;
 
 	if (!ioctl(media_fd, MEDIA_IOC_G_TOPOLOGY, &topology))
 		for (unsigned i = 0; i < topology.num_interfaces; i++) {
@@ -248,7 +240,7 @@ static void list_devices()
 	struct v4l2_capability vcap;
 
 	dp = opendir("/dev");
-	if (dp == NULL) {
+	if (dp == nullptr) {
 		perror ("Couldn't open the directory");
 		return;
 	}
@@ -258,7 +250,7 @@ static void list_devices()
 	closedir(dp);
 
 	/* Find device nodes which are links to other device nodes */
-	for (dev_vec::iterator iter = files.begin();
+	for (auto iter = files.begin();
 			iter != files.end(); ) {
 		char link[64+1];
 		int link_len;
@@ -290,9 +282,8 @@ static void list_devices()
 
 	std::sort(files.begin(), files.end(), sort_on_device_name);
 
-	for (dev_vec::iterator iter = files.begin();
-			iter != files.end(); ++iter) {
-		int fd = open(iter->c_str(), O_RDWR);
+	for (const auto &file : files) {
+		int fd = open(file.c_str(), O_RDWR);
 		std::string bus_info;
 		std::string card;
 
@@ -314,22 +305,21 @@ static void list_devices()
 					card = mdi.driver;
 			}
 		} else {
-			bus_info = (const char *)vcap.bus_info;
-			card = (const char *)vcap.card;
+			bus_info = reinterpret_cast<const char *>(vcap.bus_info);
+			card = reinterpret_cast<const char *>(vcap.card);
 		}
 		close(fd);
 		if (err)
 			continue;
 		if (cards[bus_info].empty())
 			cards[bus_info] += card + " (" + bus_info + "):\n";
-		cards[bus_info] += "\t" + (*iter);
-		if (!(links[*iter].empty()))
-			cards[bus_info] += " <- " + links[*iter];
+		cards[bus_info] += "\t" + file;
+		if (!(links[file].empty()))
+			cards[bus_info] += " <- " + links[file];
 		cards[bus_info] += "\n";
 	}
-	for (dev_map::iterator iter = cards.begin();
-			iter != cards.end(); ++iter) {
-		printf("%s\n", iter->second.c_str());
+	for (const auto &card : cards) {
+		printf("%s\n", card.second.c_str());
 	}
 }
 
@@ -385,120 +375,296 @@ static std::string safename(const unsigned char *name)
 
 static std::string safename(const char *name)
 {
-	return safename((const unsigned char *)name);
+	return safename(reinterpret_cast<const unsigned char *>(name));
 }
 
-static void print_qctrl(int fd, struct v4l2_query_ext_ctrl *queryctrl,
-		struct v4l2_ext_control *ctrl, int show_menus)
+static bool fill_subset(const struct v4l2_query_ext_ctrl &qc, ctrl_subset &subset)
 {
-	struct v4l2_querymenu qmenu;
-	std::string s = name2var(queryctrl->name);
-	unsigned i;
+	unsigned d;
 
-	memset(&qmenu, 0, sizeof(qmenu));
-	qmenu.id = queryctrl->id;
-	switch (queryctrl->type) {
-	case V4L2_CTRL_TYPE_INTEGER:
-		printf("%31s %#8.8x (int)    : min=%lld max=%lld step=%lld default=%lld",
-				s.c_str(), queryctrl->id,
-				queryctrl->minimum, queryctrl->maximum,
-				queryctrl->step, queryctrl->default_value);
-		break;
-	case V4L2_CTRL_TYPE_INTEGER64:
-		printf("%31s %#8.8x (int64)  : min=%lld max=%lld step=%lld default=%lld",
-				s.c_str(), queryctrl->id,
-				queryctrl->minimum, queryctrl->maximum,
-				queryctrl->step, queryctrl->default_value);
-		break;
-	case V4L2_CTRL_TYPE_STRING:
-		printf("%31s %#8.8x (str)    : min=%lld max=%lld step=%lld",
-				s.c_str(), queryctrl->id,
-				queryctrl->minimum, queryctrl->maximum,
-				queryctrl->step);
-		break;
-	case V4L2_CTRL_TYPE_BOOLEAN:
-		printf("%31s %#8.8x (bool)   : default=%lld",
-				s.c_str(), queryctrl->id, queryctrl->default_value);
-		break;
-	case V4L2_CTRL_TYPE_MENU:
-		printf("%31s %#8.8x (menu)   : min=%lld max=%lld default=%lld",
-				s.c_str(), queryctrl->id,
-				queryctrl->minimum, queryctrl->maximum,
-				queryctrl->default_value);
-		break;
-	case V4L2_CTRL_TYPE_INTEGER_MENU:
-		printf("%31s %#8.8x (intmenu): min=%lld max=%lld default=%lld",
-				s.c_str(), queryctrl->id,
-				queryctrl->minimum, queryctrl->maximum,
-				queryctrl->default_value);
-		break;
-	case V4L2_CTRL_TYPE_BUTTON:
-		printf("%31s %#8.8x (button) :", s.c_str(), queryctrl->id);
-		break;
-	case V4L2_CTRL_TYPE_BITMASK:
-		printf("%31s %#8.8x (bitmask): max=0x%08llx default=0x%08llx",
-				s.c_str(), queryctrl->id, queryctrl->maximum,
-				queryctrl->default_value);
-		break;
-	case V4L2_CTRL_TYPE_U8:
-		printf("%31s %#8.8x (u8)     : min=%lld max=%lld step=%lld default=%lld",
-				s.c_str(), queryctrl->id,
-				queryctrl->minimum, queryctrl->maximum,
-				queryctrl->step, queryctrl->default_value);
-		break;
-	case V4L2_CTRL_TYPE_U16:
-		printf("%31s %#8.8x (u16)    : min=%lld max=%lld step=%lld default=%lld",
-				s.c_str(), queryctrl->id,
-				queryctrl->minimum, queryctrl->maximum,
-				queryctrl->step, queryctrl->default_value);
-		break;
-	case V4L2_CTRL_TYPE_U32:
-		printf("%31s %#8.8x (u32)    : min=%lld max=%lld step=%lld default=%lld",
-				s.c_str(), queryctrl->id,
-				queryctrl->minimum, queryctrl->maximum,
-				queryctrl->step, queryctrl->default_value);
-		break;
-	default:
-		printf("%31s %#8.8x (unknown): type=%x",
-				s.c_str(), queryctrl->id, queryctrl->type);
-		break;
+	if (qc.nr_of_dims == 0)
+		return false;
+
+	for (d = 0; d < qc.nr_of_dims; d++) {
+		subset.offset[d] = 0;
+		subset.size[d] = qc.dims[d];
 	}
-	if (queryctrl->nr_of_dims == 0) {
-		switch (queryctrl->type) {
-		case V4L2_CTRL_TYPE_INTEGER:
-		case V4L2_CTRL_TYPE_BOOLEAN:
-		case V4L2_CTRL_TYPE_MENU:
-		case V4L2_CTRL_TYPE_INTEGER_MENU:
-			printf(" value=%d", ctrl->value);
+
+	std::string s = name2var(qc.name);
+
+	if (ctrl_subsets.find(s) != ctrl_subsets.end()) {
+		unsigned ss_dims;
+
+		subset = ctrl_subsets[s];
+		for (ss_dims = 0; ss_dims < V4L2_CTRL_MAX_DIMS && subset.size[ss_dims]; ss_dims++) ;
+		if (ss_dims != qc.nr_of_dims) {
+			fprintf(stderr, "expected %d dimensions but --subset specified %d\n",
+					qc.nr_of_dims, ss_dims);
+			return true;
+		}
+		for (d = 0; d < qc.nr_of_dims; d++) {
+			if (subset.offset[d] + subset.size[d] > qc.dims[d]) {
+				fprintf(stderr, "the subset offset+size for dimension %d is out of range\n", d);
+				return true;
+			}
+		}
+	}
+	return false;
+}
+
+static void print_array(const v4l2_query_ext_ctrl &qc, const v4l2_ext_control &ctrl,
+			const ctrl_subset &subset)
+{
+	std::string &name = ctrl_id2str[qc.id];
+	unsigned divide[V4L2_CTRL_MAX_DIMS] = { 0 };
+	unsigned from, to;
+	unsigned d, i;
+
+	divide[qc.nr_of_dims - 1] = 1;
+	for (d = 0; d < qc.nr_of_dims - 1; d++) {
+		divide[d] = qc.dims[d + 1];
+		for (i = 0; i < d; i++)
+			divide[i] *= divide[d];
+	}
+
+	from = subset.offset[qc.nr_of_dims - 1];
+	to = subset.offset[qc.nr_of_dims - 1] + subset.size[qc.nr_of_dims - 1] - 1;
+
+	for (unsigned idx = 0; idx < qc.elems; idx += qc.dims[qc.nr_of_dims - 1]) {
+		for (d = 0; d < qc.nr_of_dims - 1; d++) {
+			unsigned i = (idx / divide[d]) % qc.dims[d];
+
+			if (i < subset.offset[d] || i >= subset.offset[d] + subset.size[d])
+				break;
+		}
+		if (d < qc.nr_of_dims - 1)
+			continue;
+
+		printf("%s", name.c_str());
+		for (d = 0; d < qc.nr_of_dims - 1; d++)
+			printf("[%u]", (idx / divide[d]) % qc.dims[d]);
+		printf(": ");
+		switch (qc.type) {
+		case V4L2_CTRL_TYPE_U8:
+			for (i = from; i <= to; i++) {
+				printf("%4u", ctrl.p_u8[idx + i]);
+				if (i < to)
+					printf(", ");
+			}
+			printf("\n");
 			break;
-		case V4L2_CTRL_TYPE_BITMASK:
-			printf(" value=0x%08x", ctrl->value);
+		case V4L2_CTRL_TYPE_U16:
+			for (i = from; i <= to; i++) {
+				printf("%6u", ctrl.p_u16[idx + i]);
+				if (i < to)
+					printf(", ");
+			}
+			printf("\n");
 			break;
-		case V4L2_CTRL_TYPE_INTEGER64:
-			printf(" value=%lld", ctrl->value64);
-			break;
-		case V4L2_CTRL_TYPE_STRING:
-			printf(" value='%s'", safename(ctrl->string).c_str());
-			break;
-		default:
+		case V4L2_CTRL_TYPE_U32:
+			for (i = from; i <= to; i++) {
+				printf("%10u", ctrl.p_u32[idx + i]);
+				if (i < to)
+					printf(", ");
+			}
+			printf("\n");
 			break;
 		}
 	}
-	if (queryctrl->nr_of_dims) {
-		printf(" ");
-		for (i = 0; i < queryctrl->nr_of_dims; i++)
-			printf("[%u]", queryctrl->dims[i]);
+}
+
+static void print_value(int fd, const v4l2_query_ext_ctrl &qc, const v4l2_ext_control &ctrl,
+			bool show_payload, bool fill_in_subset)
+{
+	v4l2_querymenu qmenu;
+
+	if (qc.nr_of_dims) {
+		if (!show_payload)
+			return;
+		ctrl_subset subset;
+		if (fill_in_subset) {
+			if (fill_subset(qc, subset))
+				return;
+		} else {
+			memset(&subset, 0, sizeof(subset));
+			for (unsigned i = 0; i < qc.nr_of_dims; i++)
+				subset.size[i] = qc.dims[i];
+		}
+		print_array(qc, ctrl, subset);
+		return;
 	}
-	if (queryctrl->flags)
-		printf(" flags=%s", ctrlflags2s(queryctrl->flags).c_str());
+
+	if (qc.flags & V4L2_CTRL_FLAG_HAS_PAYLOAD) {
+		switch (qc.type) {
+		case V4L2_CTRL_TYPE_U8:
+			printf("%u", *ctrl.p_u8);
+			break;
+		case V4L2_CTRL_TYPE_U16:
+			printf("%u", *ctrl.p_u16);
+			break;
+		case V4L2_CTRL_TYPE_U32:
+			printf("%u", *ctrl.p_u32);
+			break;
+		case V4L2_CTRL_TYPE_STRING:
+			printf("'%s'", safename(ctrl.string).c_str());
+			break;
+		case V4L2_CTRL_TYPE_AREA:
+			printf("%dx%d", ctrl.p_area->width, ctrl.p_area->height);
+			break;
+		default:
+			printf("unsupported payload type");
+			break;
+		}
+		return;
+	}
+	switch (qc.type) {
+	case V4L2_CTRL_TYPE_INTEGER64:
+		printf("%lld", ctrl.value64);
+		break;
+	case V4L2_CTRL_TYPE_MENU:
+	case V4L2_CTRL_TYPE_INTEGER_MENU:
+		memset(&qmenu, 0, sizeof(qmenu));
+		qmenu.id = ctrl.id;
+		qmenu.index = ctrl.value;
+		if (test_ioctl(fd, VIDIOC_QUERYMENU, &qmenu))
+			printf("%d", ctrl.value);
+		else if (qc.type == V4L2_CTRL_TYPE_MENU)
+			printf("%d (%s)", ctrl.value, qmenu.name);
+		else
+			printf("%d (%lld 0x%llx)", ctrl.value, qmenu.value, qmenu.value);
+		break;
+	default:
+		printf("%d", ctrl.value);
+		break;
+	}
+}
+
+static void print_qctrl(int fd, const v4l2_query_ext_ctrl &qc,
+			const v4l2_ext_control &ctrl, bool show_menus)
+{
+	std::string s = name2var(qc.name);
+	unsigned i;
+
+	switch (qc.type) {
+	case V4L2_CTRL_TYPE_INTEGER:
+		printf("%31s %#8.8x (int)    : min=%lld max=%lld step=%lld default=%lld",
+				s.c_str(), qc.id, qc.minimum, qc.maximum,
+				qc.step, qc.default_value);
+		break;
+	case V4L2_CTRL_TYPE_INTEGER64:
+		printf("%31s %#8.8x (int64)  : min=%lld max=%lld step=%lld default=%lld",
+				s.c_str(), qc.id, qc.minimum, qc.maximum,
+				qc.step, qc.default_value);
+		break;
+	case V4L2_CTRL_TYPE_STRING:
+		printf("%31s %#8.8x (str)    : min=%lld max=%lld step=%lld",
+				s.c_str(), qc.id, qc.minimum, qc.maximum, qc.step);
+		break;
+	case V4L2_CTRL_TYPE_BOOLEAN:
+		printf("%31s %#8.8x (bool)   : default=%lld",
+				s.c_str(), qc.id, qc.default_value);
+		break;
+	case V4L2_CTRL_TYPE_MENU:
+		printf("%31s %#8.8x (menu)   : min=%lld max=%lld default=%lld",
+				s.c_str(), qc.id, qc.minimum, qc.maximum,
+				qc.default_value);
+		break;
+	case V4L2_CTRL_TYPE_INTEGER_MENU:
+		printf("%31s %#8.8x (intmenu): min=%lld max=%lld default=%lld",
+				s.c_str(), qc.id, qc.minimum, qc.maximum,
+				qc.default_value);
+		break;
+	case V4L2_CTRL_TYPE_BUTTON:
+		printf("%31s %#8.8x (button) :", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_BITMASK:
+		printf("%31s %#8.8x (bitmask): max=0x%08llx default=0x%08llx",
+				s.c_str(), qc.id, qc.maximum, qc.default_value);
+		break;
+	case V4L2_CTRL_TYPE_U8:
+		printf("%31s %#8.8x (u8)     : min=%lld max=%lld step=%lld default=%lld",
+				s.c_str(), qc.id, qc.minimum, qc.maximum,
+				qc.step, qc.default_value);
+		break;
+	case V4L2_CTRL_TYPE_U16:
+		printf("%31s %#8.8x (u16)    : min=%lld max=%lld step=%lld default=%lld",
+				s.c_str(), qc.id, qc.minimum, qc.maximum,
+				qc.step, qc.default_value);
+		break;
+	case V4L2_CTRL_TYPE_U32:
+		printf("%31s %#8.8x (u32)    : min=%lld max=%lld step=%lld default=%lld",
+				s.c_str(), qc.id, qc.minimum, qc.maximum,
+				qc.step, qc.default_value);
+		break;
+	case V4L2_CTRL_TYPE_AREA:
+		printf("%31s %#8.8x (area)   :", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_HDR10_CLL_INFO:
+		printf("%31s %#8.8x (hdr10-cll-info):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_HDR10_MASTERING_DISPLAY:
+		printf("%31s %#8.8x (hdr10-mastering-display):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_H264_SPS:
+		printf("%31s %#8.8x (h264-sps):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_H264_PPS:
+		printf("%31s %#8.8x (h264-pps):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_H264_SCALING_MATRIX:
+		printf("%31s %#8.8x (h264-scaling-matrix):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_H264_SLICE_PARAMS:
+		printf("%31s %#8.8x (h264-slice-params):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_H264_DECODE_PARAMS:
+		printf("%31s %#8.8x (h264-decode-params):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_H264_PRED_WEIGHTS:
+		printf("%31s %#8.8x (h264-pred-weights):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_VP8_FRAME:
+		printf("%31s %#8.8x (vp8-frame):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_MPEG2_QUANTISATION:
+		printf("%31s %#8.8x (mpeg2-quantisation):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_MPEG2_SEQUENCE:
+		printf("%31s %#8.8x (mpeg2-sequence):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_MPEG2_PICTURE:
+		printf("%31s %#8.8x (mpeg2-picture):", s.c_str(), qc.id);
+		break;
+	case V4L2_CTRL_TYPE_FWHT_PARAMS:
+		printf("%31s %#8.8x (fwht-params):", s.c_str(), qc.id);
+		break;
+	default:
+		printf("%31s %#8.8x (unknown): type=%x",
+				s.c_str(), qc.id, qc.type);
+		break;
+	}
+	if (qc.nr_of_dims == 0) {
+		printf(" value=");
+		print_value(fd, qc, ctrl, false, false);
+	} else {
+		printf(" dims=");
+		for (i = 0; i < qc.nr_of_dims; i++)
+			printf("[%u]", qc.dims[i]);
+	}
+	if (qc.flags)
+		printf(" flags=%s", ctrlflags2s(qc.flags).c_str());
 	printf("\n");
-	if ((queryctrl->type == V4L2_CTRL_TYPE_MENU ||
-	     queryctrl->type == V4L2_CTRL_TYPE_INTEGER_MENU) && show_menus) {
-		for (i = queryctrl->minimum; i <= queryctrl->maximum; i++) {
+	if ((qc.type == V4L2_CTRL_TYPE_MENU ||
+	     qc.type == V4L2_CTRL_TYPE_INTEGER_MENU) && show_menus) {
+		v4l2_querymenu qmenu;
+
+		memset(&qmenu, 0, sizeof(qmenu));
+		qmenu.id = ctrl.id;
+		for (i = qc.minimum; i <= qc.maximum; i++) {
 			qmenu.index = i;
 			if (test_ioctl(fd, VIDIOC_QUERYMENU, &qmenu))
 				continue;
-			if (queryctrl->type == V4L2_CTRL_TYPE_MENU)
+			if (qc.type == V4L2_CTRL_TYPE_MENU)
 				printf("\t\t\t\t%d: %s\n", i, qmenu.name);
 			else
 				printf("\t\t\t\t%d: %lld (0x%llx)\n", i, qmenu.value, qmenu.value);
@@ -506,7 +672,7 @@ static void print_qctrl(int fd, struct v4l2_query_ext_ctrl *queryctrl,
 	}
 }
 
-static int print_control(int fd, struct v4l2_query_ext_ctrl &qctrl, int show_menus)
+static int print_control(int fd, struct v4l2_query_ext_ctrl &qctrl, bool show_menus)
 {
 	struct v4l2_control ctrl;
 	struct v4l2_ext_control ext_ctrl;
@@ -524,11 +690,7 @@ static int print_control(int fd, struct v4l2_query_ext_ctrl &qctrl, int show_men
 	ext_ctrl.id = qctrl.id;
 	if ((qctrl.flags & V4L2_CTRL_FLAG_WRITE_ONLY) ||
 	    qctrl.type == V4L2_CTRL_TYPE_BUTTON) {
-		print_qctrl(fd, &qctrl, &ext_ctrl, show_menus);
-		return 1;
-	}
-	if (qctrl.type >= V4L2_CTRL_COMPOUND_TYPES) {
-		print_qctrl(fd, &qctrl, NULL, show_menus);
+		print_qctrl(fd, qctrl, ext_ctrl, show_menus);
 		return 1;
 	}
 	ctrls.which = V4L2_CTRL_ID2WHICH(qctrl.id);
@@ -536,12 +698,12 @@ static int print_control(int fd, struct v4l2_query_ext_ctrl &qctrl, int show_men
 	ctrls.controls = &ext_ctrl;
 	if (qctrl.type == V4L2_CTRL_TYPE_INTEGER64 ||
 	    qctrl.type == V4L2_CTRL_TYPE_STRING ||
+	    qctrl.type >= V4L2_CTRL_COMPOUND_TYPES ||
 	    (V4L2_CTRL_ID2WHICH(qctrl.id) != V4L2_CTRL_CLASS_USER &&
 	     qctrl.id < V4L2_CID_PRIVATE_BASE)) {
-		if (qctrl.type == V4L2_CTRL_TYPE_STRING) {
-		    ext_ctrl.size = qctrl.maximum + 1;
-		    ext_ctrl.string = (char *)malloc(ext_ctrl.size);
-		    ext_ctrl.string[0] = 0;
+		if (qctrl.flags & V4L2_CTRL_FLAG_HAS_PAYLOAD) {
+			ext_ctrl.size = qctrl.elems * qctrl.elem_size;
+			ext_ctrl.ptr = calloc(1, ext_ctrl.size);
 		}
 		if (test_ioctl(fd, VIDIOC_G_EXT_CTRLS, &ctrls)) {
 			printf("error %d getting ext_ctrl %s\n",
@@ -558,9 +720,9 @@ static int print_control(int fd, struct v4l2_query_ext_ctrl &qctrl, int show_men
 		}
 		ext_ctrl.value = ctrl.value;
 	}
-	print_qctrl(fd, &qctrl, &ext_ctrl, show_menus);
-	if (qctrl.type == V4L2_CTRL_TYPE_STRING)
-		free(ext_ctrl.string);
+	print_qctrl(fd, qctrl, ext_ctrl, show_menus);
+	if (qctrl.flags & V4L2_CTRL_FLAG_HAS_PAYLOAD)
+		free(ext_ctrl.ptr);
 	return 1;
 }
 
@@ -581,8 +743,8 @@ static int query_ext_ctrl_ioctl(int fd, struct v4l2_query_ext_ctrl &qctrl)
 		memcpy(qctrl.name, qc.name, sizeof(qctrl.name));
 		qctrl.minimum = qc.minimum;
 		if (qc.type == V4L2_CTRL_TYPE_BITMASK) {
-			qctrl.maximum = (__u32)qc.maximum;
-			qctrl.default_value = (__u32)qc.default_value;
+			qctrl.maximum = static_cast<__u32>(qc.maximum);
+			qctrl.default_value = static_cast<__u32>(qc.default_value);
 		} else {
 			qctrl.maximum = qc.maximum;
 			qctrl.default_value = qc.default_value;
@@ -610,7 +772,7 @@ static int query_ext_ctrl_ioctl(int fd, struct v4l2_query_ext_ctrl &qctrl)
 	return rc;
 }
 
-static void list_controls(int fd, int show_menus)
+static void list_controls(int fd, bool show_menus)
 {
 	const unsigned next_fl = V4L2_CTRL_FLAG_NEXT_CTRL | V4L2_CTRL_FLAG_NEXT_COMPOUND;
 	struct v4l2_query_ext_ctrl qctrl;
@@ -689,17 +851,29 @@ void common_process_controls(cv4l_fd &fd)
 	have_query_ext_ctrl = rc == 0;
 
 	find_controls(fd);
-	for (ctrl_get_list::iterator iter = get_ctrls.begin(); iter != get_ctrls.end(); ++iter) {
-	    if (ctrl_str2q.find(*iter) == ctrl_str2q.end()) {
-		fprintf(stderr, "unknown control '%s'\n", iter->c_str());
-		exit(1);
-	    }
+	for (const auto &get_ctrl : get_ctrls) {
+		std::string s = get_ctrl;
+		if (isdigit(s[0])) {
+			__u32 id = strtoul(s.c_str(), nullptr, 0);
+			if (ctrl_id2str.find(id) != ctrl_id2str.end())
+				s = ctrl_id2str[id];
+		}
+		if (ctrl_str2q.find(s) == ctrl_str2q.end()) {
+			fprintf(stderr, "unknown control '%s'\n", s.c_str());
+			std::exit(EXIT_FAILURE);
+		}
 	}
-	for (ctrl_set_map::iterator iter = set_ctrls.begin(); iter != set_ctrls.end(); ++iter) {
-	    if (ctrl_str2q.find(iter->first) == ctrl_str2q.end()) {
-		fprintf(stderr, "unknown control '%s'\n", iter->first.c_str());
-		exit(1);
-	    }
+	for (const auto &set_ctrl : set_ctrls) {
+		std::string s = set_ctrl.first;
+		if (isdigit(s[0])) {
+			__u32 id = strtoul(s.c_str(), nullptr, 0);
+			if (ctrl_id2str.find(id) != ctrl_id2str.end())
+				s = ctrl_id2str[id];
+		}
+		if (ctrl_str2q.find(s) == ctrl_str2q.end()) {
+			fprintf(stderr, "unknown control '%s'\n", s.c_str());
+			std::exit(EXIT_FAILURE);
+		}
 	}
 }
 
@@ -738,7 +912,7 @@ static bool parse_subset(char *optarg)
 
 	memset(&subset, 0, sizeof(subset));
 	while (*optarg) {
-		p = strchr(optarg, ',');
+		p = std::strchr(optarg, ',');
 		if (p)
 			*p = 0;
 		if (optarg[0] == 0) {
@@ -753,9 +927,9 @@ static bool parse_subset(char *optarg)
 				return true;
 			}
 			if (idx & 1)
-				subset.offset[idx / 2] = strtoul(optarg, 0, 0);
+				subset.offset[idx / 2] = strtoul(optarg, nullptr, 0);
 			else {
-				subset.size[idx / 2 - 1] = strtoul(optarg, 0, 0);
+				subset.size[idx / 2 - 1] = strtoul(optarg, nullptr, 0);
 				if (subset.size[idx / 2 - 1] == 0) {
 					fprintf(stderr, "<size> cannot be 0\n");
 					return true;
@@ -763,7 +937,7 @@ static bool parse_subset(char *optarg)
 			}
 		}
 		idx++;
-		if (p == NULL)
+		if (p == nullptr)
 			break;
 		optarg = p + 1;
 	}
@@ -783,7 +957,7 @@ static bool parse_subset(char *optarg)
 static bool parse_next_subopt(char **subs, char **value)
 {
 	static char *const subopts[] = {
-	    NULL
+	    nullptr
 	};
 	int opt = getsubopt(subs, subopts, value);
 
@@ -803,11 +977,11 @@ void common_cmd(const std::string &media_bus_info, int ch, char *optarg)
 		while (*subs != '\0') {
 			if (parse_next_subopt(&subs, &value)) {
 				common_usage();
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
-			if (strchr(value, '=')) {
+			if (std::strchr(value, '=')) {
 				common_usage();
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 			else {
 				get_ctrls.push_back(value);
@@ -819,25 +993,26 @@ void common_cmd(const std::string &media_bus_info, int ch, char *optarg)
 		while (*subs != '\0') {
 			if (parse_next_subopt(&subs, &value)) {
 				common_usage();
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
-			if (const char *equal = strchr(value, '=')) {
-				set_ctrls[std::string(value, (equal - value))] = equal + 1;
+			if (const char *equal = std::strchr(value, '=')) {
+				set_ctrls.emplace_back(std::string(value, (equal - value)),
+						       std::string(equal + 1));
 			}
 			else {
 				fprintf(stderr, "control '%s' without '='\n", value);
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 		}
 		break;
 	case OptSubset:
 		if (parse_subset(optarg)) {
 			common_usage();
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		break;
 	case OptSetPriority:
-		prio = (enum v4l2_priority)strtoul(optarg, 0L, 0);
+		prio = static_cast<enum v4l2_priority>(strtoul(optarg, nullptr, 0));
 		break;
 	case OptListDevices:
 		if (media_bus_info.empty())
@@ -848,40 +1023,6 @@ void common_cmd(const std::string &media_bus_info, int ch, char *optarg)
 	}
 }
 
-static bool fill_subset(const struct v4l2_query_ext_ctrl &qc, ctrl_subset &subset)
-{
-	unsigned d;
-
-	if (qc.nr_of_dims == 0)
-		return false;
-
-	for (d = 0; d < qc.nr_of_dims; d++) {
-		subset.offset[d] = 0;
-		subset.size[d] = qc.dims[d];
-	}
-
-	std::string s = name2var(qc.name);
-
-	if (ctrl_subsets.find(s) != ctrl_subsets.end()) {
-		unsigned ss_dims;
-
-		subset = ctrl_subsets[s];
-		for (ss_dims = 0; ss_dims < V4L2_CTRL_MAX_DIMS && subset.size[ss_dims]; ss_dims++) ;
-		if (ss_dims != qc.nr_of_dims) {
-			fprintf(stderr, "expected %d dimensions but --subset specified %d\n",
-					qc.nr_of_dims, ss_dims);
-			return true;
-		}
-		for (d = 0; d < qc.nr_of_dims; d++) {
-			if (subset.offset[d] + subset.size[d] > qc.dims[d]) {
-				fprintf(stderr, "the subset offset+size for dimension %d is out of range\n", d);
-				return true;
-			}
-		}
-	}
-	return false;
-}
-
 static bool idx_in_subset(const struct v4l2_query_ext_ctrl &qc, const ctrl_subset &subset,
 			  const unsigned *divide, unsigned idx)
 {
@@ -910,10 +1051,14 @@ void common_set(cv4l_fd &_fd)
 		bool use_ext_ctrls = false;
 
 		memset(&ctrls, 0, sizeof(ctrls));
-		for (ctrl_set_map::iterator iter = set_ctrls.begin();
-				iter != set_ctrls.end(); ++iter) {
+		for (const auto &set_ctrl : set_ctrls) {
+			std::string s = set_ctrl.first;
+			if (isdigit(s[0])) {
+				__u32 id = strtoul(s.c_str(), nullptr, 0);
+				s = ctrl_id2str[id];
+			}
 			struct v4l2_ext_control ctrl;
-			struct v4l2_query_ext_ctrl &qc = ctrl_str2q[iter->first];
+			struct v4l2_query_ext_ctrl &qc = ctrl_str2q[s];
 
 			memset(&ctrl, 0, sizeof(ctrl));
 			ctrl.id = qc.id;
@@ -921,7 +1066,7 @@ void common_set(cv4l_fd &_fd)
 			    qc.flags & V4L2_CTRL_FLAG_UPDATE)
 				use_ext_ctrls = true;
 			if (qc.flags & V4L2_CTRL_FLAG_HAS_PAYLOAD) {
-				struct v4l2_ext_controls ctrls = { 0, 1 };
+				struct v4l2_ext_controls ctrls = { { 0 }, 1 };
 				unsigned divide[V4L2_CTRL_MAX_DIMS] = { 0 };
 				ctrl_subset subset;
 				long long v;
@@ -949,27 +1094,31 @@ void common_set(cv4l_fd &_fd)
 
 				switch (qc.type) {
 				case V4L2_CTRL_TYPE_U8:
-					v = strtoul(iter->second.c_str(), NULL, 0);
+					v = strtoul(set_ctrl.second.c_str(), nullptr, 0);
 					for (i = 0; i < qc.elems; i++)
 						if (idx_in_subset(qc, subset, divide, i))
 							ctrl.p_u8[i] = v;
 					break;
 				case V4L2_CTRL_TYPE_U16:
-					v = strtoul(iter->second.c_str(), NULL, 0);
+					v = strtoul(set_ctrl.second.c_str(), nullptr, 0);
 					for (i = 0; i < qc.elems; i++)
 						if (idx_in_subset(qc, subset, divide, i))
 							ctrl.p_u16[i] = v;
 					break;
 				case V4L2_CTRL_TYPE_U32:
-					v = strtoul(iter->second.c_str(), NULL, 0);
+					v = strtoul(set_ctrl.second.c_str(), nullptr, 0);
 					for (i = 0; i < qc.elems; i++)
 						if (idx_in_subset(qc, subset, divide, i))
 							ctrl.p_u32[i] = v;
 					break;
 				case V4L2_CTRL_TYPE_STRING:
-					strncpy(ctrl.string, iter->second.c_str(), qc.maximum);
+					strncpy(ctrl.string, set_ctrl.second.c_str(), qc.maximum);
 					ctrl.string[qc.maximum] = 0;
 					break;
+				case V4L2_CTRL_TYPE_AREA:
+					sscanf(set_ctrl.second.c_str(), "%ux%u",
+					       &ctrl.p_area->width, &ctrl.p_area->height);
+					break;
 				default:
 					fprintf(stderr, "%s: unsupported payload type\n",
 							qc.name);
@@ -978,20 +1127,22 @@ void common_set(cv4l_fd &_fd)
 			} else {
 				if (V4L2_CTRL_DRIVER_PRIV(ctrl.id))
 					use_ext_ctrls = true;
-				ctrl.value = strtol(iter->second.c_str(), NULL, 0);
+				if (qc.type == V4L2_CTRL_TYPE_INTEGER64)
+					ctrl.value64 = strtoll(set_ctrl.second.c_str(), nullptr, 0);
+				else
+					ctrl.value = strtol(set_ctrl.second.c_str(), nullptr, 0);
 			}
 			class2ctrls[V4L2_CTRL_ID2WHICH(ctrl.id)].push_back(ctrl);
 		}
-		for (class2ctrls_map::iterator iter = class2ctrls.begin();
-				iter != class2ctrls.end(); ++iter) {
+		for (auto &class2ctrl : class2ctrls) {
 			if (!use_ext_ctrls &&
-			    (iter->first == V4L2_CTRL_CLASS_USER ||
-			     iter->first == V4L2_CID_PRIVATE_BASE)) {
-				for (unsigned i = 0; i < iter->second.size(); i++) {
+			    (class2ctrl.first == V4L2_CTRL_CLASS_USER ||
+			     class2ctrl.first == V4L2_CID_PRIVATE_BASE)) {
+				for (const auto &i : class2ctrl.second) {
 					struct v4l2_control ctrl;
 
-					ctrl.id = iter->second[i].id;
-					ctrl.value = iter->second[i].value;
+					ctrl.id = i.id;
+					ctrl.value = i.value;
 					if (doioctl(fd, VIDIOC_S_CTRL, &ctrl)) {
 						fprintf(stderr, "%s: %s\n",
 								ctrl_id2str[ctrl.id].c_str(),
@@ -1000,10 +1151,10 @@ void common_set(cv4l_fd &_fd)
 				}
 				continue;
 			}
-			if (iter->second.size()) {
-				ctrls.which = iter->first;
-				ctrls.count = iter->second.size();
-				ctrls.controls = &iter->second[0];
+			if (!class2ctrl.second.empty()) {
+				ctrls.which = class2ctrl.first;
+				ctrls.count = class2ctrl.second.size();
+				ctrls.controls = &class2ctrl.second[0];
 				if (doioctl(fd, VIDIOC_S_EXT_CTRLS, &ctrls)) {
 					if (ctrls.error_idx >= ctrls.count) {
 						fprintf(stderr, "Error setting controls: %s\n",
@@ -1011,7 +1162,7 @@ void common_set(cv4l_fd &_fd)
 					}
 					else {
 						fprintf(stderr, "%s: %s\n",
-								ctrl_id2str[iter->second[ctrls.error_idx].id].c_str(),
+								ctrl_id2str[class2ctrl.second[ctrls.error_idx].id].c_str(),
 								strerror(errno));
 					}
 				}
@@ -1020,69 +1171,6 @@ void common_set(cv4l_fd &_fd)
 	}
 }
 
-static void print_array(const struct v4l2_query_ext_ctrl &qc, void *p)
-{
-	ctrl_subset subset;
-	unsigned divide[V4L2_CTRL_MAX_DIMS] = { 0 };
-	unsigned from, to;
-	unsigned d, i;
-
-	if (fill_subset(qc, subset))
-		return;
-
-	divide[qc.nr_of_dims - 1] = 1;
-	for (d = 0; d < qc.nr_of_dims - 1; d++) {
-		divide[d] = qc.dims[d + 1];
-		for (i = 0; i < d; i++)
-			divide[i] *= divide[d];
-	}
-
-	from = subset.offset[qc.nr_of_dims - 1];
-	to = subset.offset[qc.nr_of_dims - 1] + subset.size[qc.nr_of_dims - 1] - 1;
-
-	for (unsigned idx = 0; idx < qc.elems; idx += qc.dims[qc.nr_of_dims - 1]) {
-		for (d = 0; d < qc.nr_of_dims - 1; d++) {
-			unsigned i = (idx / divide[d]) % qc.dims[d];
-
-			if (i < subset.offset[d] || i >= subset.offset[d] + subset.size[d])
-				break;
-		}
-		if (d < qc.nr_of_dims - 1)
-			continue;
-
-		printf("%s", qc.name);
-		for (d = 0; d < qc.nr_of_dims - 1; d++)
-			printf("[%u]", (idx / divide[d]) % qc.dims[d]);
-		printf(": ");
-		switch (qc.type) {
-		case V4L2_CTRL_TYPE_U8:
-			for (i = from; i <= to; i++) {
-				printf("%4d", ((__u8 *)p)[idx + i]);
-				if (i < to)
-					printf(", ");
-			}
-			printf("\n");
-			break;
-		case V4L2_CTRL_TYPE_U16:
-			for (i = from; i <= to; i++) {
-				printf("%6d", ((__u16 *)p)[idx + i]);
-				if (i < to)
-					printf(", ");
-			}
-			printf("\n");
-			break;
-		case V4L2_CTRL_TYPE_U32:
-			for (i = from; i <= to; i++) {
-				printf("%6d", ((__u32 *)p)[idx + i]);
-				if (i < to)
-					printf(", ");
-			}
-			printf("\n");
-			break;
-		}
-	}
-}
-
 void common_get(cv4l_fd &_fd)
 {
 	int fd = _fd.g_fd();
@@ -1093,10 +1181,14 @@ void common_get(cv4l_fd &_fd)
 		bool use_ext_ctrls = false;
 
 		memset(&ctrls, 0, sizeof(ctrls));
-		for (ctrl_get_list::iterator iter = get_ctrls.begin();
-				iter != get_ctrls.end(); ++iter) {
+		for (const auto &get_ctrl : get_ctrls) {
+			std::string s = get_ctrl;
+			if (isdigit(s[0])) {
+				__u32 id = strtoul(s.c_str(), nullptr, 0);
+				s = ctrl_id2str[id];
+			}
 			struct v4l2_ext_control ctrl;
-			struct v4l2_query_ext_ctrl &qc = ctrl_str2q[*iter];
+			struct v4l2_query_ext_ctrl &qc = ctrl_str2q[s];
 
 			memset(&ctrl, 0, sizeof(ctrl));
 			ctrl.id = qc.id;
@@ -1112,48 +1204,36 @@ void common_get(cv4l_fd &_fd)
 				use_ext_ctrls = true;
 			class2ctrls[V4L2_CTRL_ID2WHICH(ctrl.id)].push_back(ctrl);
 		}
-		for (class2ctrls_map::iterator iter = class2ctrls.begin();
-				iter != class2ctrls.end(); ++iter) {
+		for (auto &class2ctrl : class2ctrls) {
 			if (!use_ext_ctrls &&
-			    (iter->first == V4L2_CTRL_CLASS_USER ||
-			     iter->first == V4L2_CID_PRIVATE_BASE)) {
-				for (unsigned i = 0; i < iter->second.size(); i++) {
+			    (class2ctrl.first == V4L2_CTRL_CLASS_USER ||
+			     class2ctrl.first == V4L2_CID_PRIVATE_BASE)) {
+				for (const auto &i : class2ctrl.second) {
 					struct v4l2_control ctrl;
 
-					ctrl.id = iter->second[i].id;
+					ctrl.id = i.id;
 					doioctl(fd, VIDIOC_G_CTRL, &ctrl);
 					printf("%s: %d\n", ctrl_id2str[ctrl.id].c_str(), ctrl.value);
 				}
 				continue;
 			}
-			if (iter->second.size()) {
-				ctrls.which = iter->first;
-				ctrls.count = iter->second.size();
-				ctrls.controls = &iter->second[0];
+			if (!class2ctrl.second.empty()) {
+				ctrls.which = class2ctrl.first;
+				ctrls.count = class2ctrl.second.size();
+				ctrls.controls = &class2ctrl.second[0];
 				doioctl(fd, VIDIOC_G_EXT_CTRLS, &ctrls);
-				for (unsigned i = 0; i < iter->second.size(); i++) {
-					struct v4l2_ext_control ctrl = iter->second[i];
+				for (auto ctrl : class2ctrl.second) {
 					std::string &name = ctrl_id2str[ctrl.id];
 					struct v4l2_query_ext_ctrl &qc = ctrl_str2q[name];
 
-					if (qc.flags & V4L2_CTRL_FLAG_HAS_PAYLOAD) {
-						switch (qc.type) {
-						case V4L2_CTRL_TYPE_U8:
-						case V4L2_CTRL_TYPE_U16:
-						case V4L2_CTRL_TYPE_U32:
-							print_array(qc, ctrl.ptr);
-							break;
-						case V4L2_CTRL_TYPE_STRING:
-							printf("%s: '%s'\n", name.c_str(),
-								safename(ctrl.string).c_str());
-							break;
-						default:
-							fprintf(stderr, "%s: unsupported payload type\n",
-									qc.name);
-							break;
-						}
-					} else
-						printf("%s: %d\n", name.c_str(), ctrl.value);
+					if (qc.nr_of_dims) {
+						print_value(fd, qc, ctrl, true, true);
+						return;
+					}
+
+					printf("%s: ", name.c_str());
+					print_value(fd, qc, ctrl, true, false);
+					printf("\n");
 				}
 			}
 		}
@@ -1168,7 +1248,7 @@ void common_get(cv4l_fd &_fd)
 		static char buf[40960];
 		int len = -1;
 
-		if (doioctl(fd, VIDIOC_LOG_STATUS, NULL) == 0) {
+		if (doioctl(fd, VIDIOC_LOG_STATUS, nullptr) == 0) {
 			printf("\nStatus Log:\n\n");
 #ifdef HAVE_KLOGCTL
 			len = klogctl(3, buf, sizeof(buf) - 1);
