@@ -1,18 +1,6 @@
-#include <unistd.h>
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <inttypes.h>
-#include <getopt.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <fcntl.h>
-#include <ctype.h>
-#include <errno.h>
-#include <sys/ioctl.h>
-#include <sys/time.h>
-#include <dirent.h>
-#include <math.h>
+#include <cctype>
+
+#include <endian.h>
 
 #include "v4l2-ctl.h"
 
@@ -20,14 +8,24 @@ static struct v4l2_frmsizeenum frmsize; /* list frame sizes */
 static struct v4l2_frmivalenum frmival; /* list frame intervals */
 static unsigned set_fmts;
 static __u32 width, height, pixfmt, field, flags;
+static __u32 ycbcr, quantization, xfer_func, colorspace;
 static __u32 bytesperline[VIDEO_MAX_PLANES];
+static __u32 sizeimage[VIDEO_MAX_PLANES];
+static unsigned mbus_code;
 
-void vidcap_usage(void)
+void vidcap_usage()
 {
 	printf("\nVideo Capture Formats options:\n"
-	       "  --list-formats     display supported video formats [VIDIOC_ENUM_FMT]\n"
-	       "  --list-formats-ext display supported video formats including frame sizes\n"
-	       "                     and intervals\n"
+	       "  --list-formats [<mbus_code>]\n"
+	       "		     display supported video formats. <mbus_code> is an optional\n"
+	       "		     media bus code, if the device has capability V4L2_CAP_IO_MC\n"
+	       "		     then only formats that support this media bus code are listed\n"
+	       "		     [VIDIOC_ENUM_FMT]\n"
+	       "  --list-formats-ext [<mbus_code>]\n"
+	       "		     display supported video formats including frame sizes and intervals\n"
+	       "		     <mbus_code> is an optional media bus code, if the device has\n"
+	       "		     capability V4L2_CAP_IO_MC then only formats that support this\n"
+	       "		     media bus code are listed [VIDIOC_ENUM_FMT]\n"
 	       "  --list-framesizes <f>\n"
 	       "                     list supported framesizes for pixelformat <f>\n"
 	       "                     [VIDIOC_ENUM_FRAMESIZES]\n"
@@ -42,11 +40,12 @@ void vidcap_usage(void)
 	       "  -v, --set-fmt-video\n"
 	       "  --try-fmt-video width=<w>,height=<h>,pixelformat=<pf>,field=<f>,colorspace=<c>,\n"
 	       "                  xfer=<xf>,ycbcr=<y>,hsv=<hsv>,quantization=<q>,\n"
-	       "                  premul-alpha,bytesperline=<bpl>\n"
+	       "                  premul-alpha,bytesperline=<bpl>,sizeimage=<sz>\n"
 	       "                     set/try the video capture format [VIDIOC_S/TRY_FMT]\n"
 	       "                     pixelformat is either the format index as reported by\n"
 	       "                       --list-formats, or the fourcc value as a string.\n"
-	       "                     The bytesperline option can be used multiple times, once for each plane.\n"
+	       "                     The bytesperline and sizeimage options can be used multiple times,\n"
+	       "                       once for each plane.\n"
 	       "                     premul-alpha sets V4L2_PIX_FMT_FLAG_PREMUL_ALPHA.\n"
 	       "                     <f> can be one of the following field layouts:\n"
 	       "                       any, none, top, bottom, interlaced, seq_tb, seq_bt,\n"
@@ -98,7 +97,6 @@ static void print_video_fields(int fd)
 
 void vidcap_cmd(int ch, char *optarg)
 {
-	__u32 colorspace, xfer_func, ycbcr, quantization;
 	char *value, *subs;
 	bool be_pixfmt;
 
@@ -106,13 +104,18 @@ void vidcap_cmd(int ch, char *optarg)
 	case OptSetVideoFormat:
 	case OptTryVideoFormat:
 		set_fmts = parse_fmt(optarg, width, height, pixfmt, field, colorspace,
-				xfer_func, ycbcr, quantization, flags, bytesperline);
-		if (!set_fmts ||
-		    (set_fmts & (FmtColorspace | FmtYCbCr | FmtQuantization | FmtXferFunc))) {
+				xfer_func, ycbcr, quantization, flags, bytesperline,
+				sizeimage);
+		if (!set_fmts) {
 			vidcap_usage();
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		break;
+	case OptListFormats:
+	case OptListFormatsExt:
+		if (optarg)
+			mbus_code = strtoul(optarg, nullptr, 0);
+		break;
 	case OptListFrameSizes:
 		be_pixfmt = strlen(optarg) == 7 && !memcmp(optarg + 4, "-BE", 3);
 		if (be_pixfmt || strlen(optarg) == 4) {
@@ -121,28 +124,28 @@ void vidcap_cmd(int ch, char *optarg)
 			if (be_pixfmt)
 				frmsize.pixel_format |= 1U << 31;
 		} else if (isdigit(optarg[0])) {
-			frmsize.pixel_format = strtol(optarg, 0L, 0);
+			frmsize.pixel_format = strtol(optarg, nullptr, 0);
 		} else {
 			fprintf(stderr, "The pixelformat '%s' is invalid\n", optarg);
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 		break;
 	case OptListFrameIntervals:
 		subs = optarg;
 		while (*subs != '\0') {
-			static const char *const subopts[] = {
+			static constexpr const char *subopts[] = {
 				"width",
 				"height",
 				"pixelformat",
-				NULL
+				nullptr
 			};
 
 			switch (parse_subopt(&subs, subopts, &value)) {
 			case 0:
-				frmival.width = strtol(value, 0L, 0);
+				frmival.width = strtol(value, nullptr, 0);
 				break;
 			case 1:
-				frmival.height = strtol(value, 0L, 0);
+				frmival.height = strtol(value, nullptr, 0);
 				break;
 			case 2:
 				be_pixfmt = strlen(value) == 7 && !memcmp(value + 4, "-BE", 3);
@@ -153,15 +156,15 @@ void vidcap_cmd(int ch, char *optarg)
 					if (be_pixfmt)
 						frmival.pixel_format |= 1U << 31;
 				} else if (isdigit(optarg[0])) {
-					frmival.pixel_format = strtol(value, 0L, 0);
+					frmival.pixel_format = strtol(value, nullptr, 0);
 				} else {
 					fprintf(stderr, "The pixelformat '%s' is invalid\n", optarg);
-					exit(1);
+					std::exit(EXIT_FAILURE);
 				}
 				break;
 			default:
 				vidcap_usage();
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 		}
 		break;
@@ -211,6 +214,28 @@ int vidcap_get_and_update_fmt(cv4l_fd &_fd, struct v4l2_format &vfmt)
 			for (unsigned i = 0; i < vfmt.fmt.pix_mp.num_planes; i++)
 				vfmt.fmt.pix_mp.plane_fmt[i].bytesperline = 0;
 		}
+		if (set_fmts & FmtSizeImage) {
+			for (unsigned i = 0; i < VIDEO_MAX_PLANES; i++)
+				vfmt.fmt.pix_mp.plane_fmt[i].sizeimage =
+					sizeimage[i];
+		}
+
+		if (set_fmts & FmtColorspace) {
+			vfmt.fmt.pix_mp.flags |= V4L2_PIX_FMT_FLAG_SET_CSC;
+			vfmt.fmt.pix_mp.colorspace = colorspace;
+		}
+		if (set_fmts & FmtYCbCr) {
+			vfmt.fmt.pix_mp.flags |= V4L2_PIX_FMT_FLAG_SET_CSC;
+			vfmt.fmt.pix_mp.ycbcr_enc = ycbcr;
+		}
+		if (set_fmts & FmtQuantization) {
+			vfmt.fmt.pix_mp.flags |= V4L2_PIX_FMT_FLAG_SET_CSC;
+			vfmt.fmt.pix_mp.quantization = quantization;
+		}
+		if (set_fmts & FmtXferFunc) {
+			vfmt.fmt.pix_mp.flags |= V4L2_PIX_FMT_FLAG_SET_CSC;
+			vfmt.fmt.pix_mp.xfer_func = xfer_func;
+		}
 	} else {
 		if (set_fmts & FmtWidth)
 			vfmt.fmt.pix.width = width;
@@ -238,6 +263,25 @@ int vidcap_get_and_update_fmt(cv4l_fd &_fd, struct v4l2_format &vfmt)
 			 */
 			vfmt.fmt.pix.bytesperline = 0;
 		}
+		if (set_fmts & FmtSizeImage)
+			vfmt.fmt.pix.sizeimage = sizeimage[0];
+		if (set_fmts & FmtColorspace) {
+			vfmt.fmt.pix.flags |= V4L2_PIX_FMT_FLAG_SET_CSC;
+			vfmt.fmt.pix.colorspace = colorspace;
+		}
+		if (set_fmts & FmtYCbCr) {
+			vfmt.fmt.pix.flags |= V4L2_PIX_FMT_FLAG_SET_CSC;
+			vfmt.fmt.pix.ycbcr_enc = ycbcr;
+		}
+		if (set_fmts & FmtQuantization) {
+			vfmt.fmt.pix.flags |= V4L2_PIX_FMT_FLAG_SET_CSC;
+			vfmt.fmt.pix.quantization = quantization;
+		}
+		if (set_fmts & FmtXferFunc) {
+			vfmt.fmt.pix.flags |= V4L2_PIX_FMT_FLAG_SET_CSC;
+			vfmt.fmt.pix.xfer_func = xfer_func;
+		}
+
 	}
 
 	if ((set_fmts & FmtPixelFormat) &&
@@ -287,12 +331,12 @@ void vidcap_list(cv4l_fd &fd)
 {
 	if (options[OptListFormats]) {
 		printf("ioctl: VIDIOC_ENUM_FMT\n");
-		print_video_formats(fd, vidcap_buftype);
+		print_video_formats(fd, vidcap_buftype, mbus_code);
 	}
 
 	if (options[OptListFormatsExt]) {
 		printf("ioctl: VIDIOC_ENUM_FMT\n");
-		print_video_formats_ext(fd, vidcap_buftype);
+		print_video_formats_ext(fd, vidcap_buftype, mbus_code);
 	}
 
 	if (options[OptListFields]) {
@@ -307,14 +351,14 @@ void vidcap_list(cv4l_fd &fd)
 						  false, is_multiplanar);
 			if (!frmsize.pixel_format) {
 				fprintf(stderr, "The pixelformat index was invalid\n");
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 		}
 		if (!valid_pixel_format(fd.g_fd(), frmsize.pixel_format, false, is_multiplanar) &&
 		    !valid_pixel_format(fd.g_fd(), frmsize.pixel_format, true, is_multiplanar)) {
 			fprintf(stderr, "The pixelformat '%s' is invalid\n",
 				fcc2s(frmsize.pixel_format).c_str());
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 
 		printf("ioctl: VIDIOC_ENUM_FRAMESIZES\n");
@@ -332,13 +376,13 @@ void vidcap_list(cv4l_fd &fd)
 						  false, is_multiplanar);
 			if (!frmival.pixel_format) {
 				fprintf(stderr, "The pixelformat index was invalid\n");
-				exit(1);
+				std::exit(EXIT_FAILURE);
 			}
 		}
 		if (!valid_pixel_format(fd.g_fd(), frmival.pixel_format, false, is_multiplanar)) {
 			fprintf(stderr, "The pixelformat '%s' is invalid\n",
 				fcc2s(frmival.pixel_format).c_str());
-			exit(1);
+			std::exit(EXIT_FAILURE);
 		}
 
 		printf("ioctl: VIDIOC_ENUM_FRAMEINTERVALS\n");
@@ -348,3 +392,37 @@ void vidcap_list(cv4l_fd &fd)
 		}
 	}
 }
+
+void print_touch_buffer(FILE *f, cv4l_buffer &buf, cv4l_fmt &fmt, cv4l_queue &q)
+{
+	static constexpr char img[16] = {
+		'.', ',', ':', ';', '!', '|', 'i', 'c',
+		'n', 'o', 'm', 'I', 'C', 'N', 'O', 'M',
+	};
+	auto vbuf = static_cast<__s16 *>(q.g_dataptr(buf.g_index(), 0));
+	__u32 x, y;
+
+	switch (fmt.g_pixelformat()) {
+	case V4L2_TCH_FMT_DELTA_TD16:
+		for (y = 0; y < fmt.g_height(); y++) {
+			fprintf(f, "TD16: ");
+
+			for (x = 0; x < fmt.g_width(); x++, vbuf++) {
+				auto v = static_cast<__s16>(le16toh(*vbuf));
+
+				if (!options[OptConcise])
+					fprintf(f, "% 4d", v);
+				else if (v > 255)
+					fprintf(f, "*");
+				else if (v < -32)
+					fprintf(f, "-");
+				else if (v < 0)
+					fprintf(f, "%c", img[0]);
+				else
+					fprintf(f, "%c", img[v / 16]);
+			}
+			fprintf(f, "\n");
+		}
+		break;
+	}
+}
