@@ -7,8 +7,6 @@
 #include <config.h>
 #endif
 
-#include <string>
-
 #include <linux/videodev2.h>
 #include <linux/v4l2-subdev.h>
 
@@ -196,6 +194,7 @@ enum Option {
 	OptClearEdid,
 	OptGetEdid,
 	OptInfoEdid,
+	OptShowEdid,
 	OptFixEdidChecksums,
 	OptFreqSeek,
 	OptEncoderCmd,
@@ -223,6 +222,7 @@ enum Option {
 	OptStreamToHdr,
 	OptStreamToHost,
 	OptStreamLossless,
+	OptStreamShowDeltaNow,
 	OptStreamBufCaps,
 	OptStreamMmap,
 	OptStreamUser,
@@ -263,6 +263,7 @@ enum Option {
 	OptHelpStreaming,
 	OptHelpEdid,
 	OptHelpAll,
+	OptVersion,
 	OptLast = 512
 };
 
@@ -276,11 +277,6 @@ extern __u32 vidcap_buftype;
 extern __u32 vidout_buftype;
 extern int verbose;
 
-typedef struct {
-	unsigned flag;
-	const char *str;
-} flag_def;
-
 /* fmts specified */
 #define FmtWidth		(1L<<0)
 #define FmtHeight		(1L<<1)
@@ -300,7 +296,7 @@ typedef struct {
 
 // v4l2-ctl.cpp
 int doioctl_name(int fd, unsigned long int request, void *parm, const char *name);
-int test_ioctl(int fd, int cmd, void *arg);
+int test_ioctl(int fd, unsigned long cmd, void *arg);
 int parse_subopt(char **subs, const char * const *subopts, char **value);
 __u32 parse_field(const char *s);
 __u32 parse_colorspace(const char *s);
@@ -320,8 +316,8 @@ bool valid_pixel_format(int fd, __u32 pixelformat, bool output, bool mplane);
 void print_frmsize(const struct v4l2_frmsizeenum &frmsize, const char *prefix);
 void print_frmival(const struct v4l2_frmivalenum &frmival, const char *prefix);
 void printfmt(int fd, const struct v4l2_format &vfmt);
-void print_video_formats(cv4l_fd &fd, __u32 type);
-void print_video_formats_ext(cv4l_fd &fd, __u32 type);
+void print_video_formats(cv4l_fd &fd, __u32 type, unsigned int mbus_code);
+void print_video_formats_ext(cv4l_fd &fd, __u32 type, unsigned int mbus_code);
 
 static inline bool subscribe_event(cv4l_fd &fd, __u32 type)
 {
